@@ -1103,8 +1103,8 @@ run (void *cls,
   }
   {
     struct GNUNET_DB_EventHeaderP es = {
-      .size = ntohs (sizeof (es)),
-      .type = ntohs (TALER_DBEVENT_MERCHANT_EXCHANGE_KEYS)
+      .size = htons (sizeof (es)),
+      .type = htons (TALER_DBEVENT_MERCHANT_EXCHANGE_KEYS)
     };
 
     keys_eh = db_plugin->event_listen (db_plugin->cls,
