@@ -1042,8 +1042,8 @@ run (void *cls,
   }
   {
     struct GNUNET_DB_EventHeaderP es = {
-      .size = ntohs (sizeof(es)),
-      .type = ntohs (TALER_DBEVENT_MERCHANT_DONAU_KEYS)
+      .size = htons (sizeof(es)),
+      .type = htons (TALER_DBEVENT_MERCHANT_DONAU_KEYS)
     };
 
     eh = db_plugin->event_listen (db_plugin->cls,
@@ -1054,8 +1054,8 @@ run (void *cls,
   }
   {
     struct GNUNET_DB_EventHeaderP es = {
-      .size = ntohs (sizeof(es)),
-      .type = ntohs (TALER_DBEVENT_MERCHANT_DONAU_CHARITY_ID)
+      .size = htons (sizeof(es)),
+      .type = htons (TALER_DBEVENT_MERCHANT_DONAU_CHARITY_ID)
     };
 
     eh_charity = db_plugin->event_listen (
