@@ -430,8 +430,8 @@ insert_keys_data (const struct TALER_EXCHANGE_Keys *keys,
 
   {
     struct GNUNET_DB_EventHeaderP es = {
-      .size = ntohs (sizeof (es)),
-      .type = ntohs (TALER_DBEVENT_MERCHANT_EXCHANGE_KEYS)
+      .size = htons (sizeof (es)),
+      .type = htons (TALER_DBEVENT_MERCHANT_EXCHANGE_KEYS)
     };
 
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
@@ -1010,8 +1010,8 @@ run (void *cls,
   }
   {
     struct GNUNET_DB_EventHeaderP es = {
-      .size = ntohs (sizeof (es)),
-      .type = ntohs (TALER_DBEVENT_MERCHANT_EXCHANGE_FORCE_KEYS)
+      .size = htons (sizeof (es)),
+      .type = htons (TALER_DBEVENT_MERCHANT_EXCHANGE_FORCE_KEYS)
     };
 
     eh = db_plugin->event_listen (db_plugin->cls,
