@@ -38,6 +38,70 @@
         } while (0)
 
 
+/**
+ * Possible authorization scopes. This is a bit mask.
+ */
+enum TMH_AuthScope
+{
+  /**
+   * Nothing is authorized.
+   */
+  TMH_AS_NONE = 0,
+
+  /**
+   * Read-only access is OK. Any GET request is
+   * automatically OK.
+   */
+  TMH_AS_READ_ONLY = 1,
+
+  /**
+   * 2 is Reserved. Was refreshable pre v42
+   */
+
+  /**
+   * Order creation and payment status check only
+   */
+  TMH_AS_ORDER_SIMPLE = 3,
+
+  /**
+   * Order creation and inventory locking,
+   * includes #TMH_AS_ORDER_SIMPLE
+   */
+  TMH_AS_ORDER_POS = 4,
+
+  /**
+   * Order creation and refund
+   */
+  TMH_AS_ORDER_MGMT = 5,
+
+  /**
+   * Order full
+   * Includes #TMH_AS_ORDER_POS and #TMH_AS_ORDER_MGMT
+   */
+  TMH_AS_ORDER_FULL = 6,
+
+  /**
+   * Full access is granted to everything.
+   * We want to deprecate and remove this!
+   * Old scope "write"
+   */
+  TMH_AS_ALL = 7 | 1 << 30,
+
+  /**
+   * Full access is granted to everything.
+   */
+  TMH_AS_SPA = 8,
+
+  /**
+   * /login access to renew the token is OK.
+   * This is actually combined with other scopes
+   * and not (usually) used as a scope itself.
+   */
+  TMH_AS_REFRESHABLE = 1 << 30,
+
+};
+
+
 /**
  * Supported wire method.  Kept in a DLL.
  */
@@ -189,33 +253,6 @@ struct TMH_MerchantInstance
 GNUNET_NETWORK_STRUCT_BEGIN
 
 
-/**
- * Event triggered when an order is paid.
- */
-struct TMH_OrderPayEventP
-{
-  /**
-   * Type is #TALER_DBEVENT_MERCHANT_ORDER_PAID
-   */
-  struct GNUNET_DB_EventHeaderP header;
-
-  /**
-   * Always zero (for alignment).
-   */
-  uint32_t reserved GNUNET_PACKED;
-
-  /**
-   * Merchant's public key
-   */
-  struct TALER_MerchantPublicKeyP merchant_pub;
-
-  /**
-   * Hash of the order ID.
-   */
-  struct GNUNET_HashCode h_order_id;
-};
-
-
 /**
  * Event triggered when a fulfillment URL is
  * bound to a session (as paid).
@@ -408,70 +445,6 @@ struct TMH_RequestHandler;
 struct TMH_HandlerContext;
 
 
-/**
- * Possible authorization scopes. This is a bit mask.
- */
-enum TMH_AuthScope
-{
-  /**
-   * Nothing is authorized.
-   */
-  TMH_AS_NONE = 0,
-
-  /**
-   * Read-only access is OK. Any GET request is
-   * automatically OK.
-   */
-  TMH_AS_READ_ONLY = 1,
-
-  /**
-   * 2 is Reserved. Was refreshable pre v42
-   */
-
-  /**
-   * Order creation and payment status check only
-   */
-  TMH_AS_ORDER_SIMPLE = 3,
-
-  /**
-   * Order creation and inventory locking,
-   * includes #TMH_AS_ORDER_SIMPLE
-   */
-  TMH_AS_ORDER_POS = 4,
-
-  /**
-   * Order creation and refund
-   */
-  TMH_AS_ORDER_MGMT = 5,
-
-  /**
-   * Order full
-   * Includes #TMH_AS_ORDER_POS and #TMH_AS_ORDER_MGMT
-   */
-  TMH_AS_ORDER_FULL = 6,
-
-  /**
-   * Full access is granted to everything.
-   * We want to deprecate and remove this!
-   * Old scope "write"
-   */
-  TMH_AS_ALL = 7 | 1 << 30,
-
-  /**
-   * Full access is granted to everything.
-   */
-  TMH_AS_SPA = 8,
-
-  /**
-   * /login access to renew the token is OK.
-   * This is actually combined with other scopes
-   * and not (usually) used as a scope itself.
-   */
-  TMH_AS_REFRESHABLE = 1 << 30,
-
-};
-
-
 /**
  * @brief Struct describing an URL and the handler for it.
  *
@@ -772,6 +745,11 @@ extern struct TALER_CurrencySpecification *TMH_cspecs;
  */
 extern int TMH_force_audit;
 
+/**
+ * Our configuration.
+ */
+extern const struct GNUNET_CONFIGURATION_Handle *TMH_cfg;
+
 /**
  * Context for all CURL operations (useful to the event loop)
  */
@@ -916,66 +894,4 @@ void
 TMH_reload_instances (const char *id);
 
 
-/**
- * Check that @a token hashes to @a hash under @a salt for
- * merchant instance authentication.
- *
- * @param token the token to check
- * @param salt the salt to use when hashing
- * @param hash the hash to check against
- * @return #GNUNET_OK if the @a token matches
- */
-enum GNUNET_GenericReturnValue
-TMH_check_auth (const char *token,
-                struct TALER_MerchantAuthenticationSaltP *salt,
-                struct TALER_MerchantAuthenticationHashP *hash);
-
-
-/**
- * Compute a @a hash from @a token hashes for
- * merchant instance authentication.
- *
- * @param password the password to check
- * @param[out] salt set to a fresh random salt
- * @param[out] hash set to the hash of @a token under @a salt
- */
-void
-TMH_compute_auth (const char *password,
-                  struct TALER_MerchantAuthenticationSaltP *salt,
-                  struct TALER_MerchantAuthenticationHashP *hash);
-
-
-/**
- * Check if @a candidate permissions are a subset of @a as permissions
- *
- * @param as scope to check against
- * @param candidate scope to check if its permissions are a subset of @a as permissions.
- * @return true if it was a subset, false otherwise.
- */
-bool
-TMH_scope_is_subset (enum TMH_AuthScope as,
-                     enum TMH_AuthScope candidate);
-
-
-/**
- * Return the TMH_AuthScope corresponding to @a name.
- *
- * @param name the name to look for
- * @return the scope corresponding to the name, or TMH_AS_NONE.
- */
-enum TMH_AuthScope
-TMH_get_scope_by_name (const char *name);
-
-
-/**
- * Return the name corresponding to @a scop.
- *
- * @param scope the scope to look for
- * @param[out] refreshable outputs if scope value was refreshable
- * @return the name corresponding to the scope, or NULL.
- */
-const char *
-TMH_get_name_by_scope (enum TMH_AuthScope scope,
-                       bool *refreshable);
-
 #endif
