@@ -238,15 +238,8 @@ TMH_EXCHANGES_get_currency (
 }
 
 
-/**
- * Lookup exchange by @a exchange_url. Create one
- * if it does not exist.
- *
- * @param exchange_url base URL to match against
- * @return fresh entry if exchange was not yet known
- */
-static struct TMH_Exchange *
-lookup_exchange (const char *exchange_url)
+struct TMH_Exchange *
+TMH_EXCHANGES_lookup_exchange (const char *exchange_url)
 {
   for (struct TMH_Exchange *exchange = exchange_head;
        NULL != exchange;
@@ -262,7 +255,7 @@ bool
 TMH_EXCHANGES_check_trusted (
   const char *exchange_url)
 {
-  struct TMH_Exchange *exchange = lookup_exchange (exchange_url);
+  struct TMH_Exchange *exchange = TMH_EXCHANGES_lookup_exchange (exchange_url);
 
   if (NULL == exchange)
     return false;
@@ -424,8 +417,8 @@ retry_exchange (void *cls)
 {
   struct TMH_Exchange *exchange = cls;
   struct GNUNET_DB_EventHeaderP es = {
-    .size = ntohs (sizeof (es)),
-    .type = ntohs (TALER_DBEVENT_MERCHANT_EXCHANGE_FORCE_KEYS)
+    .size = htons (sizeof (es)),
+    .type = htons (TALER_DBEVENT_MERCHANT_EXCHANGE_FORCE_KEYS)
   };
 
   exchange->retry_task = NULL;
@@ -480,7 +473,7 @@ TMH_EXCHANGES_keys4exchange (
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Trying to find chosen exchange `%s'\n",
               chosen_exchange);
-  exchange = lookup_exchange (chosen_exchange);
+  exchange = TMH_EXCHANGES_lookup_exchange (chosen_exchange);
   if (NULL == exchange)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
@@ -1033,7 +1026,7 @@ accept_exchanges (void *cls,
                                "EXCHANGE_BASE_URL");
     return;
   }
-  exchange = lookup_exchange (url);
+  exchange = TMH_EXCHANGES_lookup_exchange (url);
   if (NULL != exchange)
   {
     GNUNET_log_config_invalid (GNUNET_ERROR_TYPE_ERROR,
@@ -1140,7 +1133,7 @@ update_exchange_keys (void *cls,
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Received keys change notification: reload `%s'\n",
               url);
-  exchange = lookup_exchange (url);
+  exchange = TMH_EXCHANGES_lookup_exchange (url);
   GNUNET_break (NULL != exchange);
   if (NULL != exchange)
     reload_exchange_keys (exchange);
@@ -1185,7 +1178,7 @@ TMH_EXCHANGES_get_limit (
   struct TMH_Exchange *exchange;
   const struct TALER_EXCHANGE_Keys *keys;
 
-  exchange = lookup_exchange (exchange_url);
+  exchange = TMH_EXCHANGES_lookup_exchange (exchange_url);
   if ( (NULL == exchange) ||
        (NULL == (keys = exchange->keys)) )
   {
@@ -1215,8 +1208,8 @@ TMH_EXCHANGES_init (const struct GNUNET_CONFIGURATION_Handle *cfg)
   /* get exchanges from the merchant configuration and try to connect to them */
   {
     struct GNUNET_DB_EventHeaderP es = {
-      .size = ntohs (sizeof (es)),
-      .type = ntohs (TALER_DBEVENT_MERCHANT_EXCHANGE_KEYS)
+      .size = htons (sizeof (es)),
+      .type = htons (TALER_DBEVENT_MERCHANT_EXCHANGE_KEYS)
     };
 
     GNUNET_assert (NULL == keys_eh);
