@@ -151,6 +151,17 @@ TMH_EXCHANGES_get_master_pub (
   const struct TMH_Exchange *exchange);
 
 
+/**
+ * Lookup exchange by @a exchange_url. Create one
+ * if it does not exist.
+ *
+ * @param exchange_url base URL to match against
+ * @return fresh entry if exchange was not yet known
+ */
+struct TMH_Exchange *
+TMH_EXCHANGES_lookup_exchange (const char *exchange_url);
+
+
 /**
  * Return the currency of the given @a exchange.
  * Will be returned from configuration for trusted
