@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2014--2023 Taler Systems SA
+  (C) 2014--2026 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Lesser General Public License as published by the Free Software
@@ -27,295 +27,6 @@
 #include <taler/taler_dbevents.h>
 
 
-enum GNUNET_GenericReturnValue
-TMH_parse_fractional_string (const char *value,
-                             int64_t *integer_part,
-                             uint32_t *fractional_part)
-{
-  const char *ptr;
-  uint64_t integer = 0;
-  uint32_t frac = 0;
-  unsigned int digits = 0;
-
-  GNUNET_assert (NULL != integer_part);
-  GNUNET_assert (NULL != fractional_part);
-
-  if (NULL == value)
-  {
-    GNUNET_break_op (0);
-    return GNUNET_SYSERR;
-  }
-  ptr = value;
-  if ('\0' == *ptr)
-  {
-    GNUNET_break_op (0);
-    return GNUNET_SYSERR;
-  }
-  if ('-' == *ptr)
-  {
-    GNUNET_break_op (0);
-    return GNUNET_SYSERR;
-  }
-  if (! isdigit ((unsigned char) *ptr))
-  {
-    GNUNET_break_op (0);
-    return GNUNET_SYSERR;
-  }
-  while (isdigit ((unsigned char) *ptr))
-  {
-    unsigned int digit = (unsigned int) (*ptr - '0');
-
-    if (integer > (UINT64_MAX - digit) / 10)
-    {
-      GNUNET_break_op (0);
-      return GNUNET_SYSERR;
-    }
-    integer = integer * 10 + digit;
-    ptr++;
-  }
-  if ('.' == *ptr)
-  {
-    ptr++;
-    if ('\0' == *ptr)
-    {
-      GNUNET_break_op (0);
-      return GNUNET_SYSERR;
-    }
-    while (isdigit ((unsigned char) *ptr))
-    {
-      unsigned int digit = (unsigned int) (*ptr - '0');
-
-      if (digits >= MERCHANT_UNIT_FRAC_MAX_DIGITS)
-      {
-        GNUNET_break_op (0);
-        return GNUNET_SYSERR;
-      }
-      frac = (uint32_t) (frac * 10 + digit);
-      digits++;
-      ptr++;
-    }
-    while (digits < MERCHANT_UNIT_FRAC_MAX_DIGITS)
-    {
-      frac *= 10;
-      digits++;
-    }
-  }
-  if ('\0' != *ptr)
-  {
-    GNUNET_break_op (0);
-    return GNUNET_SYSERR;
-  }
-  if (integer > (uint64_t) INT64_MAX)
-  {
-    GNUNET_break_op (0);
-    return GNUNET_SYSERR;
-  }
-  *integer_part = integer;
-  *fractional_part = frac;
-  return GNUNET_OK;
-}
-
-
-enum GNUNET_GenericReturnValue
-TMH_process_quantity_inputs (enum TMH_ValueKind kind,
-                             bool allow_fractional,
-                             bool int_missing,
-                             int64_t int_raw,
-                             bool str_missing,
-                             const char *str_raw,
-                             uint64_t *int_out,
-                             uint32_t *frac_out,
-                             const char **error_param)
-{
-  static char errbuf[128];
-  int64_t parsed_int = 0;
-  uint32_t parsed_frac = 0;
-  const char *int_field = (TMH_VK_STOCK == kind)
-                          ? "total_stock"
-                          : "quantity";
-  const char *str_field = (TMH_VK_STOCK == kind)
-                          ? "unit_total_stock"
-                          : "unit_quantity";
-
-  *error_param = NULL;
-
-  if (int_missing && str_missing)
-  {
-    GNUNET_snprintf (errbuf,
-                     sizeof (errbuf),
-                     "missing %s and %s",
-                     int_field,
-                     str_field);
-    *error_param = errbuf;
-    GNUNET_break_op (0);
-    return GNUNET_SYSERR;
-  }
-
-  if (! str_missing)
-  {
-    if ( (TMH_VK_STOCK == kind) &&
-         (0 == strcmp ("-1",
-                       str_raw)) )
-    {
-      parsed_int = -1;
-      parsed_frac = 0;
-    }
-    else
-    {
-      if (GNUNET_OK !=
-          TMH_parse_fractional_string (str_raw,
-                                       &parsed_int,
-                                       &parsed_frac))
-      {
-        GNUNET_snprintf (errbuf,
-                         sizeof (errbuf),
-                         "malformed %s",
-                         str_field);
-        *error_param = errbuf;
-        GNUNET_break_op (0);
-        return GNUNET_SYSERR;
-      }
-    }
-  }
-
-  if ( (! int_missing) && (! str_missing) )
-  {
-    if ( (parsed_int != int_raw) || (0 != parsed_frac) )
-    {
-      GNUNET_snprintf (errbuf,
-                       sizeof (errbuf),
-                       "%s/%s mismatch",
-                       int_field,
-                       str_field);
-      *error_param = errbuf;
-      GNUNET_break_op (0);
-      return GNUNET_SYSERR;
-    }
-  }
-  else if (int_missing)
-  {
-    int_raw = parsed_int;
-  }
-
-  if ( (TMH_VK_STOCK == kind) && (-1 == int_raw) )
-  {
-    if ( (! str_missing) && (0 != parsed_frac) )
-    {
-      GNUNET_snprintf (errbuf,
-                       sizeof (errbuf),
-                       "fractional part forbidden with %s='-1'",
-                       str_field);
-      *error_param = errbuf;
-      GNUNET_break_op (0);
-      return GNUNET_SYSERR;
-    }
-    *int_out = INT64_MAX;
-    *frac_out = INT32_MAX;
-    return GNUNET_OK;
-  }
-
-  if (int_raw < 0)
-  {
-    GNUNET_snprintf (errbuf,
-                     sizeof (errbuf),
-                     "%s must be non-negative",
-                     int_field);
-    *error_param = errbuf;
-    GNUNET_break_op (0);
-    return GNUNET_SYSERR;
-  }
-
-  if (! allow_fractional)
-  {
-    if ( (! str_missing) && (0 != parsed_frac) )
-    {
-      GNUNET_snprintf (errbuf,
-                       sizeof (errbuf),
-                       "fractional part not allowed for %s",
-                       str_field);
-      *error_param = errbuf;
-      GNUNET_break_op (0);
-      return GNUNET_SYSERR;
-    }
-    parsed_frac = 0;
-  }
-  else if (! str_missing)
-  {
-    if (parsed_frac >= MERCHANT_UNIT_FRAC_BASE)
-    {
-      GNUNET_snprintf (errbuf,
-                       sizeof (errbuf),
-                       "%s fractional part exceeds base %u",
-                       str_field,
-                       MERCHANT_UNIT_FRAC_BASE);
-      *error_param = errbuf;
-      GNUNET_break_op (0);
-      return GNUNET_SYSERR;
-    }
-  }
-
-  *int_out = (uint64_t) int_raw;
-  *frac_out = parsed_frac;
-  return GNUNET_OK;
-}
-
-
-void
-TMH_format_fractional_string (enum TMH_ValueKind kind,
-                              uint64_t integer,
-                              uint32_t fractional,
-                              size_t buffer_length,
-                              char buffer[static buffer_length])
-{
-  GNUNET_assert (0 < buffer_length);
-
-  if ( (TMH_VK_STOCK == kind) &&
-       (INT64_MAX == (int64_t) integer) &&
-       (INT32_MAX == (int32_t) fractional) )
-  {
-    GNUNET_snprintf (buffer,
-                     buffer_length,
-                     "-1");
-    return;
-  }
-
-  GNUNET_assert ( (TMH_VK_QUANTITY != kind) ||
-                  ((INT64_MAX != (int64_t) integer) &&
-                   (INT32_MAX != (int32_t) fractional)) );
-  GNUNET_assert (fractional < MERCHANT_UNIT_FRAC_BASE);
-
-  if (0 == fractional)
-  {
-    GNUNET_snprintf (buffer,
-                     buffer_length,
-                     "%lu",
-                     integer);
-    return;
-  }
-  {
-    char frac_buf[MERCHANT_UNIT_FRAC_MAX_DIGITS + 1];
-    size_t idx;
-
-    GNUNET_snprintf (frac_buf,
-                     sizeof (frac_buf),
-                     "%0*u",
-                     MERCHANT_UNIT_FRAC_MAX_DIGITS,
-                     (unsigned int) fractional);
-    for (idx = strlen (frac_buf); idx > 0; idx--)
-    {
-      if ('0' != frac_buf[idx - 1])
-        break;
-      frac_buf[idx - 1] = '\0';
-    }
-    GNUNET_snprintf (buffer,
-                     buffer_length,
-                     "%lu.%s",
-                     integer,
-                     frac_buf);
-  }
-}
-
-
 void
 TMH_quantity_defaults_from_unit (const struct TMH_MerchantInstance *mi,
                                  const char *unit,
@@ -663,6 +374,7 @@ TMH_products_array_valid (const json_t *products)
   }
   json_array_foreach ((json_t *) products, idx, product)
   {
+    /* FIXME: use TALER_MERCHANT_parse_product() instead? */
     const char *product_id = NULL;
     const char *description;
     uint64_t quantity = 0;
@@ -735,13 +447,13 @@ TMH_products_array_valid (const json_t *products)
       }
     }
     if ( (NULL != image_data_url) &&
-         (! TMH_image_data_url_valid (image_data_url)) )
+         (! TALER_MERCHANT_image_data_url_valid (image_data_url)) )
     {
       GNUNET_break_op (0);
       valid = false;
     }
     if ( (NULL != taxes) &&
-         (! TMH_taxes_array_valid (taxes)) )
+         (! TALER_MERCHANT_taxes_array_valid (taxes)) )
     {
       GNUNET_break_op (0);
       valid = false;
@@ -755,110 +467,86 @@ TMH_products_array_valid (const json_t *products)
 }
 
 
-bool
-TMH_image_data_url_valid (const char *image_data_url)
+enum GNUNET_GenericReturnValue
+TMH_validate_unit_price_array (const struct TALER_Amount *prices,
+                               size_t prices_len)
 {
-  if (0 == strcmp (image_data_url,
-                   ""))
-    return true;
-  if (0 != strncasecmp ("data:image/",
-                        image_data_url,
-                        strlen ("data:image/")))
+  /* Check for duplicate currencies */
+  for (size_t i = 0; i < prices_len; i++)
   {
-    GNUNET_break_op (0);
-    return false;
-  }
-  if (NULL == strstr (image_data_url,
-                      ";base64,"))
-  {
-    GNUNET_break_op (0);
-    return false;
-  }
-  if (! TALER_url_valid_charset (image_data_url))
-  {
-    GNUNET_break_op (0);
-    return false;
-  }
-  return true;
-}
-
-
-bool
-TMH_template_contract_valid (const json_t *template_contract)
-{
-  const char *summary;
-  const char *currency;
-  struct TALER_Amount amount = { .value = 0};
-  uint32_t minimum_age = 0;
-  struct GNUNET_TIME_Relative pay_duration = { 0 };
-  struct GNUNET_JSON_Specification spec[] = {
-    GNUNET_JSON_spec_mark_optional (
-      GNUNET_JSON_spec_string ("summary",
-                               &summary),
-      NULL),
-    GNUNET_JSON_spec_mark_optional (
-      GNUNET_JSON_spec_string ("currency",
-                               &currency),
-      NULL),
-    GNUNET_JSON_spec_mark_optional (
-      TALER_JSON_spec_amount_any ("amount",
-                                  &amount),
-      NULL),
-    GNUNET_JSON_spec_uint32 ("minimum_age",
-                             &minimum_age),
-    GNUNET_JSON_spec_relative_time ("pay_duration",
-                                    &pay_duration),
-    GNUNET_JSON_spec_end ()
-  };
-  const char *ename;
-  unsigned int eline;
-
-  if (GNUNET_OK !=
-      GNUNET_JSON_parse (template_contract,
-                         spec,
-                         &ename,
-                         &eline))
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Invalid template_contract for field %s\n",
-                ename);
-    return false;
-  }
-  return true;
-}
-
-
-bool
-TMH_taxes_array_valid (const json_t *taxes)
-{
-  json_t *tax;
-  size_t idx;
-
-  if (! json_is_array (taxes))
-    return false;
-  json_array_foreach (taxes, idx, tax)
-  {
-    struct TALER_Amount amount;
-    const char *name;
-    struct GNUNET_JSON_Specification spec[] = {
-      GNUNET_JSON_spec_string ("name",
-                               &name),
-      TALER_JSON_spec_amount_any ("tax",
-                                  &amount),
-      GNUNET_JSON_spec_end ()
-    };
-    enum GNUNET_GenericReturnValue res;
-
-    res = TALER_MHD_parse_json_data (NULL,
-                                     tax,
-                                     spec);
-    if (GNUNET_OK != res)
+    for (size_t j = i + 1; j < prices_len; j++)
     {
-      GNUNET_break_op (0);
-      return false;
+      if (GNUNET_YES ==
+          TALER_amount_cmp_currency (&prices[i],
+                                     &prices[j]))
+      {
+        /* duplicate currency, not allowed! */
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
     }
   }
-  return true;
+  return GNUNET_OK;
+}
+
+
+bool
+TMH_category_set_contains (const struct TMH_CategorySet *set,
+                           uint64_t id)
+{
+  for (size_t i = 0; i < set->len; i++)
+    if (set->ids[i] == id)
+      return true;
+  return false;
+}
+
+
+void
+TMH_category_set_add (struct TMH_CategorySet *set,
+                      uint64_t id)
+{
+  if (TMH_category_set_contains (set,
+                                 id))
+    return;
+  GNUNET_array_append (set->ids,
+                       set->len,
+                       id);
+}
+
+
+bool
+TMH_unit_set_contains (const struct TMH_UnitSet *set,
+                       const char *unit)
+{
+  for (unsigned int i = 0; i < set->len; i++)
+    if (0 == strcmp (set->units[i],
+                     unit))
+      return true;
+  return false;
+}
+
+
+void
+TMH_unit_set_add (struct TMH_UnitSet *set,
+                  const char *unit)
+{
+  if (TMH_unit_set_contains (set,
+                             unit))
+    return;
+  GNUNET_array_append (set->units,
+                       set->len,
+                       GNUNET_strdup (unit));
+}
+
+
+void
+TMH_unit_set_clear (struct TMH_UnitSet *set)
+{
+  for (unsigned int i = 0; i < set->len; i++)
+    GNUNET_free (set->units[i]);
+  GNUNET_free (set->units);
+  set->units = NULL;
+  set->len = 0;
 }
 
 
@@ -1121,6 +809,7 @@ trigger_webhook_cb (void *cls,
   if (NULL != body_template)
   {
     int ret;
+
     ret = TALER_TEMPLATING_fill (body_template,
                                  t->root,
                                  &body,
@@ -1157,6 +846,7 @@ trigger_webhook_cb (void *cls,
                           &extra,
                           extra_size);
   }
+  /* allocated by mustach, hence use free(), not GNUNET_free() */
   free (header);
   free (body);
 }
