@@ -27,6 +27,7 @@
 
 
 #include "taler-merchant-httpd.h"
+#include "taler_merchant_util.h"
 
 /**
  * check @a accounts for well-formedness
@@ -38,18 +39,6 @@ bool
 TMH_accounts_array_valid (const json_t *accounts);
 
 
-/**
- * Check if @a taxes is an array of valid Taxes in the sense of
- * Taler's API definition.
- *
- * @param taxes array to check
- * @return true if @a taxes is an array and all
- *         entries are valid Taxes.
- */
-bool
-TMH_taxes_array_valid (const json_t *taxes);
-
-
 /**
  * Check if @a location is a valid Location object in the sense of Taler's API
  * definition.
@@ -88,49 +77,98 @@ TMH_parse_fractional_string (const char *value,
                              uint32_t *fractional_part);
 
 /**
- * Kind of fixed-decimal value the helpers operate on.
- * Primarily distinguishes how special sentinel values (such as "-1"
- * meaning infinity for stock) must be encoded.
+ * Check that no two prices use the same currency.
+ *
+ * @param prices price list to check
+ * @param prices_len length of @a prices
+ * @return #GNUNET_OK if unique, #GNUNET_SYSERR otherwise
  */
-enum TMH_ValueKind
+enum GNUNET_GenericReturnValue
+TMH_validate_unit_price_array (const struct TALER_Amount *prices,
+                               size_t prices_len);
+
+
+/**
+ * Set of category IDs.
+ */
+struct TMH_CategorySet
 {
-  TMH_VK_QUANTITY,  /* -1 is illegal                    */
-  TMH_VK_STOCK      /* -1 means "infinity"              */
+  /**
+   * Category IDs.
+   */
+  uint64_t *ids;
+
+  /**
+   * Number of entries in @e ids.
+   */
+  unsigned int len;
 };
 
 /**
- * Extract a fixed-decimal number that may be supplied either
- *   - as pure integer  (e.g. "total_stock"), or
- *   - as decimal text (e.g. "unit_total_stock").
- *
- * Rules:
- *  - If both forms are missing          -> error.
- *  - If both are present                -> they must match and the decimal must have no fraction.
- *  - For kind == TMH_VK_STOCK the integer value -1 represents infinity.
- *
- * @param kind             See #TMH_ValueKind
- * @param allow_fractional False: any fractional part is rejected
- * @param int_missing      True if client omitted the integer field
- * @param int_raw          Raw integer (undefined if @a int_missing is true)
- * @param str_missing      True if client omitted the string field
- * @param str_raw          Raw UTF-8 string (undefined if @a str_missing is true)
- * @param[out] int_out     Canonicalised integer part
- * @param[out] frac_out    Canonicalised fractional part
- * @param[out] error_param Set to offending field name on failure
- * @param int_field        Integer field name (for error reporting)
- * @param str_field        String field name (for error reporting)
- * @return #GNUNET_OK on success, #GNUNET_SYSERR otherwise
+ * Set of unit identifiers.
  */
-enum GNUNET_GenericReturnValue
-TMH_process_quantity_inputs (enum TMH_ValueKind kind,
-                             bool allow_fractional,
-                             bool int_missing,
-                             int64_t int_raw,
-                             bool str_missing,
-                             const char *str_raw,
-                             uint64_t *int_out,
-                             uint32_t *frac_out,
-                             const char **error_param);
+struct TMH_UnitSet
+{
+  /**
+   * Unit identifiers.
+   */
+  char **units;
+
+  /**
+   * Number of entries in @e units.
+   */
+  unsigned int len;
+};
+/**
+ * Check if a category set already contains a given ID.
+ *
+ * @param set category set
+ * @param id category id
+ * @return true if present
+ */
+bool
+TMH_category_set_contains (const struct TMH_CategorySet *set,
+                           uint64_t id);
+
+/**
+ * Add a category ID to a set if not already present.
+ *
+ * @param set category set
+ * @param id category id
+ */
+void
+TMH_category_set_add (struct TMH_CategorySet *set,
+                      uint64_t id);
+
+/**
+ * Check if a unit set already contains a given unit.
+ *
+ * @param set unit set
+ * @param unit unit identifier
+ * @return true if present
+ */
+bool
+TMH_unit_set_contains (const struct TMH_UnitSet *set,
+                       const char *unit);
+
+/**
+ * Add a unit identifier to a set if not already present.
+ *
+ * @param set unit set
+ * @param unit unit identifier
+ */
+void
+TMH_unit_set_add (struct TMH_UnitSet *set,
+                  const char *unit);
+
+/**
+ * Clear a unit set and free its contents.
+ *
+ * @param set unit set to clear
+ */
+void
+TMH_unit_set_clear (struct TMH_UnitSet *set);
+
 
 /**
  * Lookup the defaults for @a unit within @a mi and fall back to sane
@@ -165,49 +203,6 @@ TMH_unit_defaults_for_instance (const struct TMH_MerchantInstance *mi,
                                 bool *allow_fractional,
                                 uint32_t *precision_level);
 
-/**
- * Format a fixed-decimal pair into canonical string representation.
- * Recognises INT64_MAX / INT32_MAX as the "-1" sentinel used for
- * infinite stock if @a kind equals #TMH_VK_STOCK.
- *
- * @param kind specifies whether sentinel values are permitted
- * @param integer integer portion
- * @param fractional fractional portion (0..MERCHANT_UNIT_FRAC_BASE-1 or sentinel)
- * @param buffer output buffer
- * @param buffer_length length of @a buffer
- */
-void
-TMH_format_fractional_string (enum TMH_ValueKind kind,
-                              uint64_t integer,
-                              uint32_t fractional,
-                              size_t buffer_length,
-                              char buffer[static buffer_length]);
-
-/**
- * Check if @a image_data_url is a valid image
- * data URL. Does not validate the actual payload,
- * only the syntax and that it properly claims to
- * be an image.
- *
- * @param image_data_url string to check
- * @return true if @a image_data_url is a data
- *         URL with an "image/" mime-type
- */
-bool
-TMH_image_data_url_valid (const char *image_data_url);
-
-
-/**
- * Check if @a template_contract is a valid template_contract  object in the sense of Taler's API
- * definition.
- *
- * @param template_contract object to check
- * @return true if @a template_location is an object
- *         representing a template_location.
- */
-bool
-TMH_template_contract_valid (const json_t *template_contract);
-
 
 /**
  * Setup new wire method for the given @ payto_uri.
@@ -349,13 +344,13 @@ TMH_make_order_status_url (struct MHD_Connection *con,
  * @param hr a `TALER_EXCHANGE_HttpResponse`
  */
 #define TMH_pack_exchange_reply(hr) \
-        GNUNET_JSON_pack_uint64 ("exchange_code", (hr)->ec),                \
-        GNUNET_JSON_pack_uint64 ("exchange_http_status", (hr)->http_status), \
-        GNUNET_JSON_pack_uint64 ("exchange_ec", (hr)->ec), /* LEGACY */  \
-        GNUNET_JSON_pack_uint64 ("exchange_hc", (hr)->http_status), /* LEGACY */ \
-        GNUNET_JSON_pack_allow_null ( \
-          GNUNET_JSON_pack_object_incref ("exchange_reply", (json_t *) (hr)-> \
-                                          reply))
+  GNUNET_JSON_pack_uint64 ("exchange_code", (hr)->ec),                \
+  GNUNET_JSON_pack_uint64 ("exchange_http_status", (hr)->http_status), \
+  GNUNET_JSON_pack_uint64 ("exchange_ec", (hr)->ec),       /* LEGACY */  \
+  GNUNET_JSON_pack_uint64 ("exchange_hc", (hr)->http_status),       /* LEGACY */ \
+  GNUNET_JSON_pack_allow_null ( \
+    GNUNET_JSON_pack_object_incref ("exchange_reply", (json_t *) (hr)-> \
+                                    reply))
 
 
 /**
