@@ -20,6 +20,7 @@
  */
 #include "platform.h"
 #include "taler-merchant-httpd_private-delete-orders-ID.h"
+#include <stdint.h>
 #include <taler/taler_json_lib.h>
 
 
@@ -80,6 +81,7 @@ TMH_private_delete_orders_ID (const struct TMH_RequestHandler *rh,
       bool paid = false;
       bool wired = false;
       bool matches = false;
+      int16_t choice_index;
 
       qs = TMH_db->lookup_order (TMH_db->cls,
                                  mi->settings.id,
@@ -98,7 +100,8 @@ TMH_private_delete_orders_ID (const struct TMH_RequestHandler *rh,
                                              &paid,
                                              &wired,
                                              &matches,
-                                             NULL);
+                                             NULL,
+                                             &choice_index);
       }
       if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
         return TALER_MHD_reply_with_error (connection,
