@@ -1,6 +1,6 @@
 /*
   This file is part of GNU Taler
-  (C) 2021-2024 Taler Systems SA
+  (C) 2021-2026 Taler Systems SA
 
   GNU Taler is free software; you can redistribute it and/or modify
   it under the terms of the GNU Affero General Public License as
@@ -107,8 +107,7 @@ struct ExchangeKycRequest
   struct GNUNET_TIME_Timestamp last_check;
 
   /**
-   * Last HTTP status code obtained via /kyc-check from
-   * the exchange.
+   * Last HTTP status code obtained via /kyc-check from the exchange.
    */
   unsigned int last_http_status;
 
@@ -118,8 +117,7 @@ struct ExchangeKycRequest
   enum TALER_ErrorCode last_ec;
 
   /**
-   * True if this account
-   * cannot work at this exchange because KYC auth is
+   * True if this account cannot work at this exchange because KYC auth is
    * impossible.
    */
   bool kyc_auth_conflict;
@@ -146,7 +144,6 @@ struct ExchangeKycRequest
    */
   bool in_aml_review;
 
-
 };
 
 
@@ -203,12 +200,6 @@ struct KycContext
    */
   struct ExchangeKycRequest *exchange_pending_tail;
 
-  /**
-   * Set to the exchange URL, or NULL to not filter by
-   * exchange.
-   */
-  const char *exchange_url;
-
   /**
    * Notification handler from database on changes
    * to the KYC status.
@@ -216,22 +207,49 @@ struct KycContext
   struct GNUNET_DB_EventHandler *eh;
 
   /**
-   * Set to the h_wire of the merchant account if
-   * @a have_h_wire is true, used to filter by account.
+   * Set to the exchange URL, or NULL to not filter by
+   * exchange.  "exchange_url" query parameter.
    */
-  struct TALER_MerchantWireHashP h_wire;
+  const char *exchange_url;
 
   /**
    * How long are we willing to wait for the exchange(s)?
+   * Based on "timeout_ms" query parameter.
    */
   struct GNUNET_TIME_Absolute timeout;
 
   /**
-   * HTTP status code to use for the reply, i.e 200 for "OK".
-   * Special value UINT_MAX is used to indicate hard errors
-   * (no reply, return #MHD_NO).
+   * Set to the h_wire of the merchant account if
+   * @a have_h_wire is true, used to filter by account.
+   * Set from "h_wire" query parameter.
    */
-  unsigned int response_code;
+  struct TALER_MerchantWireHashP h_wire;
+
+  /**
+   * Set to the Etag of a response already known to the
+   * client. We should only return from long-polling
+   * on timeout (with "Not Modified") or when the Etag
+   * of the response differs from what is given here.
+   * Only set if @a have_lp_not_etag is true.
+   * Set from "lp_etag" query parameter.
+   */
+  struct GNUNET_ShortHashCode lp_not_etag;
+
+  /**
+   * Specifies what status change we are long-polling for.  If specified, the
+   * endpoint will only return once the status *matches* the given value.  If
+   * multiple accounts or exchanges match the query, any account reaching the
+   * STATUS will cause the response to be returned.
+   */
+  const char *lp_status;
+
+  /**
+   * Specifies what status change we are long-polling for.  If specified, the
+   * endpoint will only return once the status no longer matches the given
+   * value.  If multiple accounts or exchanges *no longer matches* the given
+   * STATUS will cause the response to be returned.
+   */
+  const char *lp_not_status;
 
   /**
    * #GNUNET_NO if the @e connection was not suspended,
@@ -242,15 +260,27 @@ struct KycContext
   enum GNUNET_GenericReturnValue suspended;
 
   /**
-   * What state are we long-polling for?
+   * What state are we long-polling for? "lpt" argument.
    */
   enum TALER_EXCHANGE_KycLongPollTarget lpt;
 
+  /**
+   * HTTP status code to use for the reply, i.e 200 for "OK".
+   * Special value UINT_MAX is used to indicate hard errors
+   * (no reply, return #MHD_NO).
+   */
+  unsigned int response_code;
+
   /**
    * True if @e h_wire was given.
    */
   bool have_h_wire;
 
+  /**
+   * True if @e lp_not_etag was given.
+   */
+  bool have_lp_not_etag;
+
   /**
    * We're still waiting on the exchange to determine
    * the KYC status of our deposit(s).
@@ -334,6 +364,29 @@ kyc_context_cleanup (void *cls)
 }
 
 
+/**
+ * We have found an exchange in status @a status. Clear any
+ * long-pollers that wait for us having (or not having) this
+ * status.
+ *
+ * @param[in,out] kc context
+ * @param status the status we encountered
+ */
+static void
+clear_status (struct KycContext *kc,
+              const char *status)
+{
+  if ( (NULL != kc->lp_status) &&
+       (0 == strcmp (kc->lp_status,
+                     status)) )
+    kc->lp_status = NULL; /* satisfied! */
+  if ( (NULL != kc->lp_not_status) &&
+       (0 != strcmp (kc->lp_not_status,
+                     status) ) )
+    kc->lp_not_status = NULL; /* satisfied! */
+}
+
+
 /**
  * Resume the given KYC context and send the final response.  Stores the
  * response in the @a kc and signals MHD to resume the connection.  Also
@@ -344,10 +397,86 @@ kyc_context_cleanup (void *cls)
 static void
 resume_kyc_with_response (struct KycContext *kc)
 {
-  kc->response_code = MHD_HTTP_OK;
+  struct GNUNET_ShortHashCode sh;
+  bool not_modified;
+  char *can;
+
+  if ( (! GNUNET_TIME_absolute_is_past (kc->timeout)) &&
+       ( (NULL != kc->lp_not_status) ||
+         (NULL != kc->lp_status) ) )
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Long-poll target status not reached, not returning response yet\n");
+    if (GNUNET_NO == kc->suspended)
+    {
+      MHD_suspend_connection (kc->connection);
+      kc->suspended = GNUNET_YES;
+    }
+    return;
+  }
+  can = TALER_JSON_canonicalize (kc->kycs_data);
+  GNUNET_assert (GNUNET_YES ==
+                 GNUNET_CRYPTO_kdf (&sh,
+                                    sizeof (sh),
+                                    "KYC-SALT",
+                                    strlen ("KYC-SALT"),
+                                    can,
+                                    strlen (can),
+                                    NULL,
+                                    0));
+  not_modified = kc->have_lp_not_etag &&
+                 (0 == GNUNET_memcmp (&sh,
+                                      &kc->lp_not_etag));
+  if (not_modified &&
+      (! GNUNET_TIME_absolute_is_past (kc->timeout)) )
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Status unchanged, not returning response yet\n");
+    if (GNUNET_NO == kc->suspended)
+    {
+      MHD_suspend_connection (kc->connection);
+      kc->suspended = GNUNET_YES;
+    }
+    GNUNET_free (can);
+    return;
+  }
+  {
+    const char *inm;
+
+    inm = MHD_lookup_connection_value (kc->connection,
+                                       MHD_GET_ARGUMENT_KIND,
+                                       MHD_HTTP_HEADER_IF_NONE_MATCH);
+    if ( (NULL == inm) ||
+         ('"' != inm[0]) ||
+         ('"' != inm[strlen (inm) - 1]) ||
+         (0 != strncmp (inm + 1,
+                        can,
+                        strlen (can))) )
+      not_modified = false; /* must return full response */
+  }
+  GNUNET_free (can);
+  kc->response_code = not_modified
+    ? MHD_HTTP_NOT_MODIFIED
+    : MHD_HTTP_OK;
   kc->response = TALER_MHD_MAKE_JSON_PACK (
     GNUNET_JSON_pack_array_incref ("kyc_data",
                                    kc->kycs_data));
+  {
+    char *etag;
+    char *qetag;
+
+    etag = GNUNET_STRINGS_data_to_string_alloc (&sh,
+                                                sizeof (sh));
+    GNUNET_asprintf (&qetag,
+                     "\"%s\"",
+                     etag);
+    GNUNET_break (MHD_YES ==
+                  MHD_add_response_header (kc->response,
+                                           MHD_HTTP_HEADER_ETAG,
+                                           qetag));
+    GNUNET_free (qetag);
+    GNUNET_free (etag);
+  }
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Resuming /kyc handling as exchange interaction is done (%u)\n",
               MHD_HTTP_OK);
@@ -495,6 +624,9 @@ map_to_status (const struct ExchangeKycRequest *ekr)
   {
     return "no-exchange-keys";
   }
+  if (TALER_EC_MERCHANT_PRIVATE_ACCOUNT_NOT_ELIGIBLE_FOR_EXCHANGE ==
+      ekr->last_ec)
+    return "unsupported-account";
   if (ekr->kyc_ok)
   {
     if (NULL != ekr->jlimits)
@@ -627,14 +759,18 @@ map_to_status (const struct ExchangeKycRequest *ekr)
 static void
 ekr_expand_response (struct ExchangeKycRequest *ekr)
 {
+  struct TMH_Exchange *e = TMH_EXCHANGES_lookup_exchange (ekr->exchange_url);
   const char *status;
 
+  GNUNET_assert (NULL != e);
   status = map_to_status (ekr);
   if (NULL == status)
   {
     GNUNET_break (0);
     status = "logic-bug";
   }
+  clear_status (ekr->kc,
+                status);
   GNUNET_assert (
     0 ==
     json_array_append_new (
@@ -652,6 +788,9 @@ ekr_expand_response (struct ExchangeKycRequest *ekr)
         GNUNET_JSON_pack_string (
           "exchange_url",
           ekr->exchange_url),
+        GNUNET_JSON_pack_string (
+          "exchange_currency",
+          TMH_EXCHANGES_get_currency (e)),
         GNUNET_JSON_pack_bool ("no_keys",
                                ekr->no_keys),
         GNUNET_JSON_pack_bool ("auth_conflict",
@@ -928,6 +1067,8 @@ add_unreachable_status (void *cls,
   struct UnreachableContext *uc = cls;
   struct KycContext *kc = uc->kc;
 
+  clear_status (kc,
+                "exchange-unreachable");
   GNUNET_assert (
     0 ==
     json_array_append_new (
@@ -939,6 +1080,9 @@ add_unreachable_status (void *cls,
         GNUNET_JSON_pack_data_auto (
           "h_wire",
           uc->h_wire),
+        GNUNET_JSON_pack_string (
+          "exchange_currency",
+          TMH_EXCHANGES_get_currency (exchange)),
         GNUNET_JSON_pack_string (
           "status",
           "exchange-unreachable"),
@@ -1147,14 +1291,24 @@ get_instances_ID_kyc (
         TALER_EC_GENERIC_PARAMETER_MALFORMED,
         "exchange_url must be a valid HTTP(s) URL");
     }
-
+    kc->lp_status = MHD_lookup_connection_value (
+      connection,
+      MHD_GET_ARGUMENT_KIND,
+      "lp_status");
+    kc->lp_not_status = MHD_lookup_connection_value (
+      connection,
+      MHD_GET_ARGUMENT_KIND,
+      "lp_not_status");
     TALER_MHD_parse_request_arg_auto (connection,
                                       "h_wire",
                                       &kc->h_wire,
                                       kc->have_h_wire);
+    TALER_MHD_parse_request_arg_auto (connection,
+                                      "lp_not_etag",
+                                      &kc->lp_not_etag,
+                                      kc->have_lp_not_etag);
 
-    if ( (TALER_EXCHANGE_KLPT_NONE != kc->lpt) &&
-         (! GNUNET_TIME_absolute_is_past (kc->timeout)) )
+    if (! GNUNET_TIME_absolute_is_past (kc->timeout))
     {
       if (kc->have_h_wire)
       {
@@ -1210,6 +1364,9 @@ get_instances_ID_kyc (
                 mi->settings.id,
                 kc->have_h_wire,
                 kc->exchange_url);
+    /* We may run repeatedly due to long-polling; clear data
+       from previous runs first */
+    GNUNET_break (0 == json_array_clear (kc->kycs_data));
     qs = TMH_db->account_kyc_get_status (
       TMH_db->cls,
       mi->settings.id,
@@ -1239,13 +1396,43 @@ get_instances_ID_kyc (
     }
     if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
     {
+      /* We use an Etag of all zeros for the 204 status code */
+      static struct GNUNET_ShortHashCode zero_etag;
+
       /* no matching accounts, could not have suspended */
       GNUNET_assert (GNUNET_NO == kc->suspended);
-      return TALER_MHD_reply_static (connection,
-                                     MHD_HTTP_NO_CONTENT,
-                                     NULL,
-                                     NULL,
-                                     0);
+      if (kc->have_lp_not_etag &&
+          (0 == GNUNET_memcmp (&zero_etag,
+                               &kc->lp_not_etag)) &&
+          (! GNUNET_TIME_absolute_is_past (kc->timeout)) )
+      {
+        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                    "No matching accounts, suspending to wait for this to change\n");
+        MHD_suspend_connection (kc->connection);
+        kc->suspended = GNUNET_YES;
+        return MHD_YES;
+      }
+      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                  "No matching accounts, returning empty response\n");
+      kc->response_code = MHD_HTTP_NO_CONTENT;
+      kc->response = MHD_create_response_from_buffer_static (0,
+                                                             NULL);
+      TALER_MHD_add_global_headers (kc->response,
+                                    false);
+      {
+        char *etag;
+
+        etag = GNUNET_STRINGS_data_to_string_alloc (&zero_etag,
+                                                    sizeof (zero_etag));
+        GNUNET_break (MHD_YES ==
+                      MHD_add_response_header (kc->response,
+                                               MHD_HTTP_HEADER_ETAG,
+                                               etag));
+        GNUNET_free (etag);
+      }
+      return MHD_queue_response (connection,
+                                 kc->response_code,
+                                 kc->response);
     }
   }
   if (GNUNET_YES == kc->suspended)
