@@ -74,9 +74,6 @@ add_instance (void *cls,
                    GNUNET_JSON_PACK (
                      GNUNET_JSON_pack_string ("name",
                                               mi->settings.name),
-                     GNUNET_JSON_pack_string (
-                       "user_type",
-                       "business"),
                      GNUNET_JSON_pack_allow_null (
                        GNUNET_JSON_pack_string ("website",
                                                 mi->settings.website)),
