@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2017-2024 Taler Systems SA
+  (C) 2017-2024, 2026 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -284,6 +284,16 @@ struct GetOrderRequestContext
    */
   struct TALER_PrivateContractHashP h_contract_terms;
 
+  /**
+   * Set to the Etag of a response already known to the
+   * client. We should only return from long-polling
+   * on timeout (with "Not Modified") or when the Etag
+   * of the response differs from what is given here.
+   * Only set if @a have_lp_not_etag is true.
+   * Set from "lp_etag" query parameter.
+   */
+  struct GNUNET_ShortHashCode lp_not_etag;
+
   /**
    * Total amount the exchange deposited into our bank account
    * (confirmed or unconfirmed), excluding fees.
@@ -362,6 +372,11 @@ struct GetOrderRequestContext
    */
   bool refunded;
 
+  /**
+   * True if @e lp_not_etag was given.
+   */
+  bool have_lp_not_etag;
+
   /**
    * True if the order was paid.
    */
@@ -543,7 +558,7 @@ phase_init (struct GetOrderRequestContext *gorc)
   struct TMH_HandlerContext *hc = gorc->hc;
   struct TMH_OrderPayEventP pay_eh = {
     .header.size = htons (sizeof (pay_eh)),
-    .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_PAID),
+    .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_STATUS_CHANGED),
     .merchant_pub = hc->instance->merchant_pub
   };
 
@@ -842,6 +857,103 @@ phase_check_paid (struct GetOrderRequestContext *gorc)
 }
 
 
+/**
+ * Check if the @a reply satisfies the long-poll not_etag
+ * constraint. If so, return it as a response for @a gorc,
+ * otherwise suspend and wait for a change.
+ *
+ * @param[in,out] gorc request to handle
+ * @param reply body for JSON response (#MHD_HTTP_OK)
+ */
+static void
+check_reply (struct GetOrderRequestContext *gorc,
+             const json_t *reply)
+{
+  struct GNUNET_ShortHashCode sh;
+  unsigned int http_response_code;
+  bool not_modified;
+  struct MHD_Response *response;
+  char *can;
+
+  can = TALER_JSON_canonicalize (reply);
+  GNUNET_assert (GNUNET_YES ==
+                 GNUNET_CRYPTO_kdf (&sh,
+                                    sizeof (sh),
+                                    "GOR-SALT",
+                                    strlen ("GOR-SALT"),
+                                    can,
+                                    strlen (can),
+                                    NULL,
+                                    0));
+  not_modified = gorc->have_lp_not_etag &&
+                 (0 == GNUNET_memcmp (&sh,
+                                      &gorc->lp_not_etag));
+
+  if (not_modified &&
+      (! GNUNET_TIME_absolute_is_past (gorc->sc.long_poll_timeout)) )
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Status unchanged, not returning response yet\n");
+    GNUNET_assert (GNUNET_NO == gorc->suspended);
+    /* note: not necessarily actually unpaid ... */
+    GNUNET_CONTAINER_DLL_insert (gorc_head,
+                                 gorc_tail,
+                                 gorc);
+    gorc->phase = GOP_SUSPENDED_ON_UNPAID;
+    gorc->suspended = GNUNET_YES;
+    MHD_suspend_connection (gorc->sc.con);
+    GNUNET_free (can);
+    return;
+  }
+  {
+    const char *inm;
+
+    inm = MHD_lookup_connection_value (gorc->sc.con,
+                                       MHD_GET_ARGUMENT_KIND,
+                                       MHD_HTTP_HEADER_IF_NONE_MATCH);
+    if ( (NULL == inm) ||
+         ('"' != inm[0]) ||
+         ('"' != inm[strlen (inm) - 1]) ||
+         (0 != strncmp (inm + 1,
+                        can,
+                        strlen (can))) )
+      not_modified = false; /* must return full response */
+  }
+  GNUNET_free (can);
+  http_response_code = not_modified
+    ? MHD_HTTP_NOT_MODIFIED
+    : MHD_HTTP_OK;
+  response = TALER_MHD_make_json (reply);
+  {
+    char *etag;
+    char *qetag;
+
+    etag = GNUNET_STRINGS_data_to_string_alloc (&sh,
+                                                sizeof (sh));
+    GNUNET_asprintf (&qetag,
+                     "\"%s\"",
+                     etag);
+    GNUNET_break (MHD_YES ==
+                  MHD_add_response_header (response,
+                                           MHD_HTTP_HEADER_ETAG,
+                                           qetag));
+    GNUNET_free (qetag);
+    GNUNET_free (etag);
+  }
+
+  {
+    MHD_RESULT ret;
+
+    ret = MHD_queue_response (gorc->sc.con,
+                              http_response_code,
+                              response);
+    MHD_destroy_response (response);
+    phase_end (gorc,
+               ret);
+  }
+}
+
+
 /**
  * Check if re-purchase detection applies to the order.
  *
@@ -855,7 +967,7 @@ phase_check_repurchase (struct GetOrderRequestContext *gorc)
   enum GNUNET_DB_QueryStatus qs;
   char *taler_pay_uri;
   char *order_status_url;
-  MHD_RESULT ret;
+  json_t *reply;
 
   if ( (gorc->paid) ||
        (NULL == gorc->contract_terms->fulfillment_url) ||
@@ -904,6 +1016,16 @@ phase_check_repurchase (struct GetOrderRequestContext *gorc)
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Found already paid order %s\n",
               already_paid_order_id);
+  // FIXME-#10966 (discuss with Florian): it is a bit wild that we use the session ID of the *client's request*
+  // (which we usually use to check that the right session got paid!) here to
+  // construct the payment URI when we actually _also_ have a session_id in the
+  // orders table! CG thinks we probably should change this and *ignore* the
+  // client's session ID and when building the taler://pay/ URI only use the
+  // session ID of the order's table (is that one even used right now?).
+  // Right now, test_merchant_templates.sh has to pass the PAIVANA_ID when
+  // calling this endpoint, otherwise the taler://pay/-URI would have no
+  // session ID, despite the templating mechanism having set one explicitly
+  // when instantiating the order.
   taler_pay_uri = TMH_make_taler_pay_uri (gorc->sc.con,
                                           hc->infix,
                                           gorc->session_id,
@@ -928,9 +1050,7 @@ phase_check_repurchase (struct GetOrderRequestContext *gorc)
                                            "host"));
     return;
   }
-  ret = TALER_MHD_REPLY_JSON_PACK (
-    gorc->sc.con,
-    MHD_HTTP_OK,
+  reply = GNUNET_JSON_PACK (
     GNUNET_JSON_pack_string ("taler_pay_uri",
                              taler_pay_uri),
     GNUNET_JSON_pack_string ("order_status_url",
@@ -947,17 +1067,21 @@ phase_check_repurchase (struct GetOrderRequestContext *gorc)
                               TALER_amount_is_valid (&gorc->contract_amount)
                               ? &gorc->contract_amount
                               : NULL)),
+    GNUNET_JSON_pack_object_incref ("proto_contract_terms",
+                                    gorc->contract_terms_json),
     GNUNET_JSON_pack_string ("summary",
                              gorc->contract_terms->summary),
     GNUNET_JSON_pack_timestamp ("pay_deadline",
                                 gorc->contract_terms->pay_deadline),
     GNUNET_JSON_pack_timestamp ("creation_time",
                                 gorc->contract_terms->timestamp));
+
   GNUNET_free (order_status_url);
   GNUNET_free (taler_pay_uri);
   GNUNET_free (already_paid_order_id);
-  phase_end (gorc,
-             ret);
+  check_reply (gorc,
+               reply);
+  json_decref (reply);
 }
 
 
@@ -970,9 +1094,7 @@ static void
 phase_unpaid_finish (struct GetOrderRequestContext *gorc)
 {
   struct TMH_HandlerContext *hc = gorc->hc;
-  char *taler_pay_uri;
   char *order_status_url;
-  MHD_RESULT ret;
 
   if (gorc->paid)
   {
@@ -981,7 +1103,8 @@ phase_unpaid_finish (struct GetOrderRequestContext *gorc)
   }
   /* User never paid for this order, suspend waiting
      on payment or return details. */
-  if (GNUNET_TIME_absolute_is_future (gorc->sc.long_poll_timeout))
+  if (GNUNET_TIME_absolute_is_future (gorc->sc.long_poll_timeout) &&
+      (! gorc->have_lp_not_etag) )
   {
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Suspending GET /private/orders/%s\n",
@@ -1002,49 +1125,55 @@ phase_unpaid_finish (struct GetOrderRequestContext *gorc)
                                                 NULL);
   if (! gorc->order_only)
   {
+    json_t *reply;
+
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Order %s claimed but not paid yet\n",
                 hc->infix);
-    phase_end (gorc,
-               TALER_MHD_REPLY_JSON_PACK (
-                 gorc->sc.con,
-                 MHD_HTTP_OK,
-                 GNUNET_JSON_pack_string ("order_status_url",
-                                          order_status_url),
-                 GNUNET_JSON_pack_object_incref ("contract_terms",
-                                                 gorc->contract_terms_json),
-                 GNUNET_JSON_pack_string ("order_status",
-                                          "claimed")));
+    reply = GNUNET_JSON_PACK (
+      GNUNET_JSON_pack_string ("order_status_url",
+                               order_status_url),
+      GNUNET_JSON_pack_object_incref ("contract_terms",
+                                      gorc->contract_terms_json),
+      GNUNET_JSON_pack_string ("order_status",
+                               "claimed"));
     GNUNET_free (order_status_url);
+    check_reply (gorc,
+                 reply);
+    json_decref (reply);
     return;
   }
-  taler_pay_uri = TMH_make_taler_pay_uri (gorc->sc.con,
-                                          hc->infix,
-                                          gorc->session_id,
-                                          hc->instance->settings.id,
-                                          &gorc->claim_token);
-  ret = TALER_MHD_REPLY_JSON_PACK (
-    gorc->sc.con,
-    MHD_HTTP_OK,
-    GNUNET_JSON_pack_string ("taler_pay_uri",
-                             taler_pay_uri),
-    GNUNET_JSON_pack_string ("order_status_url",
-                             order_status_url),
-    GNUNET_JSON_pack_string ("order_status",
-                             "unpaid"),
-    /* undefined for unpaid v1 contracts */
-    GNUNET_JSON_pack_allow_null (
-      TALER_JSON_pack_amount ("total_amount",
-                              &gorc->contract_amount)),
-    GNUNET_JSON_pack_string ("summary",
-                             gorc->contract_terms->summary),
-    GNUNET_JSON_pack_timestamp ("creation_time",
-                                gorc->contract_terms->timestamp));
-  GNUNET_free (taler_pay_uri);
-  GNUNET_free (order_status_url);
-  phase_end (gorc,
-             ret);
+  {
+    char *taler_pay_uri;
+    json_t *reply;
 
+    taler_pay_uri = TMH_make_taler_pay_uri (gorc->sc.con,
+                                            hc->infix,
+                                            gorc->session_id,
+                                            hc->instance->settings.id,
+                                            &gorc->claim_token);
+    reply = GNUNET_JSON_PACK (
+      GNUNET_JSON_pack_string ("taler_pay_uri",
+                               taler_pay_uri),
+      GNUNET_JSON_pack_string ("order_status_url",
+                               order_status_url),
+      GNUNET_JSON_pack_string ("order_status",
+                               "unpaid"),
+      GNUNET_JSON_pack_object_incref ("proto_contract_terms",
+                                      gorc->contract_terms_json),
+      /* undefined for unpaid v1 contracts */
+      GNUNET_JSON_pack_allow_null (
+        TALER_JSON_pack_amount ("total_amount",
+                                &gorc->contract_amount)),
+      GNUNET_JSON_pack_string ("summary",
+                               gorc->contract_terms->summary),
+      GNUNET_JSON_pack_timestamp ("creation_time",
+                                  gorc->contract_terms->timestamp));
+    check_reply (gorc,
+                 reply);
+    GNUNET_free (taler_pay_uri);
+  }
+  GNUNET_free (order_status_url);
 }
 
 
@@ -1157,7 +1286,7 @@ phase_check_refunds (struct GetOrderRequestContext *gorc)
   GNUNET_assert (GNUNET_OK ==
                  TALER_amount_set_zero (gorc->contract_amount.currency,
                                         &gorc->refund_amount));
-
+  json_array_clear (gorc->refund_details);
   qs = TMH_db->lookup_refunds_detailed (
     TMH_db->cls,
     hc->instance->settings.id,
@@ -1295,7 +1424,8 @@ process_transfer_details (
   struct GNUNET_TIME_Timestamp execution_time,
   const struct TALER_Amount *deposit_value,
   const struct TALER_Amount *deposit_fee,
-  bool transfer_confirmed)
+  bool transfer_confirmed,
+  uint64_t expected_credit_serial)
 {
   struct GetOrderRequestContext *gorc = cls;
   json_t *wire_details = gorc->wire_details;
@@ -1338,7 +1468,9 @@ process_transfer_details (
                      GNUNET_JSON_pack_timestamp ("execution_time",
                                                  execution_time),
                      GNUNET_JSON_pack_bool ("confirmed",
-                                            transfer_confirmed))));
+                                            transfer_confirmed),
+                     GNUNET_JSON_pack_uint64 ("expected_transfer_serial_id",
+                                              expected_credit_serial))));
 }
 
 
@@ -1362,7 +1494,9 @@ phase_check_local_transfers (struct GetOrderRequestContext *gorc)
   GNUNET_assert (GNUNET_OK ==
                  TALER_amount_set_zero (gorc->contract_amount.currency,
                                         &gorc->deposit_fees_total));
-
+  GNUNET_assert (NULL != gorc->wire_details);
+  /* We may be running again due to long-polling, clear state first */
+  json_array_clear (gorc->wire_details);
   qs = TMH_db->lookup_transfer_details_by_order (TMH_db->cls,
                                                  gorc->order_serial,
                                                  &process_transfer_details,
@@ -1451,7 +1585,6 @@ static void
 phase_reply_result (struct GetOrderRequestContext *gorc)
 {
   struct TMH_HandlerContext *hc = gorc->hc;
-  MHD_RESULT ret;
   char *order_status_url;
 
   GNUNET_assert (gorc->paid);
@@ -1483,54 +1616,55 @@ phase_reply_result (struct GetOrderRequestContext *gorc)
                   TALER_amount_is_zero (&gorc->contract_amount));
     gorc->last_payment = gorc->contract_terms->timestamp;
   }
-  ret = TALER_MHD_REPLY_JSON_PACK (
-    gorc->sc.con,
-    MHD_HTTP_OK,
-    // Deprecated in protocol v6.
-    GNUNET_JSON_pack_array_steal ("wire_reports",
-                                  json_array ()),
-    GNUNET_JSON_pack_uint64 ("exchange_code",
-                             gorc->exchange_ec),
-    GNUNET_JSON_pack_uint64 ("exchange_http_status",
-                             gorc->exchange_hc),
-    /* legacy: */
-    GNUNET_JSON_pack_uint64 ("exchange_ec",
-                             gorc->exchange_ec),
-    /* legacy: */
-    GNUNET_JSON_pack_uint64 ("exchange_hc",
-                             gorc->exchange_hc),
-    TALER_JSON_pack_amount ("deposit_total",
-                            &gorc->deposits_total),
-    GNUNET_JSON_pack_object_incref ("contract_terms",
-                                    gorc->contract_terms_json),
-    GNUNET_JSON_pack_string ("order_status",
-                             "paid"),
-    GNUNET_JSON_pack_timestamp ("last_payment",
-                                gorc->last_payment),
-    GNUNET_JSON_pack_bool ("refunded",
-                           gorc->refunded),
-    GNUNET_JSON_pack_bool ("wired",
-                           gorc->wired),
-    GNUNET_JSON_pack_bool ("refund_pending",
-                           gorc->refund_pending),
-    GNUNET_JSON_pack_allow_null (
-      TALER_JSON_pack_amount ("refund_amount",
-                              &gorc->refund_amount)),
-    GNUNET_JSON_pack_array_steal ("wire_details",
-                                  gorc->wire_details),
-    GNUNET_JSON_pack_array_steal ("refund_details",
-                                  gorc->refund_details),
-    GNUNET_JSON_pack_string ("order_status_url",
-                             order_status_url),
-    (gorc->choice_index >= 0)
+  {
+    json_t *reply;
+
+    reply = GNUNET_JSON_PACK (
+      // Deprecated in protocol v6!
+      GNUNET_JSON_pack_array_steal ("wire_reports",
+                                    json_array ()),
+      GNUNET_JSON_pack_uint64 ("exchange_code",
+                               gorc->exchange_ec),
+      GNUNET_JSON_pack_uint64 ("exchange_http_status",
+                               gorc->exchange_hc),
+      /* legacy: */
+      GNUNET_JSON_pack_uint64 ("exchange_ec",
+                               gorc->exchange_ec),
+      /* legacy: */
+      GNUNET_JSON_pack_uint64 ("exchange_hc",
+                               gorc->exchange_hc),
+      TALER_JSON_pack_amount ("deposit_total",
+                              &gorc->deposits_total),
+      GNUNET_JSON_pack_object_incref ("contract_terms",
+                                      gorc->contract_terms_json),
+      GNUNET_JSON_pack_string ("order_status",
+                               "paid"),
+      GNUNET_JSON_pack_timestamp ("last_payment",
+                                  gorc->last_payment),
+      GNUNET_JSON_pack_bool ("refunded",
+                             gorc->refunded),
+      GNUNET_JSON_pack_bool ("wired",
+                             gorc->wired),
+      GNUNET_JSON_pack_bool ("refund_pending",
+                             gorc->refund_pending),
+      GNUNET_JSON_pack_allow_null (
+        TALER_JSON_pack_amount ("refund_amount",
+                                &gorc->refund_amount)),
+      GNUNET_JSON_pack_array_incref ("wire_details",
+                                     gorc->wire_details),
+      GNUNET_JSON_pack_array_incref ("refund_details",
+                                     gorc->refund_details),
+      GNUNET_JSON_pack_string ("order_status_url",
+                               order_status_url),
+      (gorc->choice_index >= 0)
       ? GNUNET_JSON_pack_int64 ("choice_index",
                                 gorc->choice_index)
       : GNUNET_JSON_pack_end_ ());
+    check_reply (gorc,
+                 reply);
+    json_decref (reply);
+  }
   GNUNET_free (order_status_url);
-  gorc->wire_details = NULL;
-  gorc->refund_details = NULL;
-  phase_end (gorc,
-             ret);
 }
 
 
@@ -1585,6 +1719,10 @@ TMH_private_get_orders_ID (
                                          "allow_refunded_for_repurchase");
     TALER_MHD_parse_request_timeout (connection,
                                      &gorc->sc.long_poll_timeout);
+    TALER_MHD_parse_request_arg_auto (connection,
+                                      "lp_not_etag",
+                                      &gorc->lp_not_etag,
+                                      gorc->have_lp_not_etag);
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "Starting GET /private/orders/%s processing with timeout %s\n",
                 hc->infix,
