@@ -119,11 +119,12 @@ add_product (void *cls,
     total_stock_api = -1;
   else
     total_stock_api = (int64_t) pd->total_stock;
-  TMH_format_fractional_string (TMH_VK_STOCK,
-                                pd->total_stock,
-                                pd->total_stock_frac,
-                                sizeof (unit_total_stock_buf),
-                                unit_total_stock_buf);
+  TALER_MERCHANT_vk_format_fractional_string (
+    TALER_MERCHANT_VK_STOCK,
+    pd->total_stock,
+    pd->total_stock_frac,
+    sizeof (unit_total_stock_buf),
+    unit_total_stock_buf);
 
   GNUNET_assert (
     0 ==
@@ -138,8 +139,12 @@ add_product (void *cls,
                                         (json_t *) pd->description_i18n),
         GNUNET_JSON_pack_string ("unit",
                                  pd->unit),
-        TALER_JSON_pack_amount ("price",
-                                &pd->price),
+        // Note: deprecated field
+        GNUNET_JSON_pack_allow_null (
+          TALER_JSON_pack_amount ("price",
+                                  (0 == pd->price_array_length)
+                                  ? NULL
+                                  : &pd->price_array[0])),
         TALER_JSON_pack_amount_array ("unit_price",
                                       pd->price_array_length,
                                       pd->price_array),
