@@ -192,7 +192,7 @@ TMH_private_get_statistics_amount_SLUG (const struct TMH_RequestHandler *rh,
         GNUNET_break_op (0);
         return TALER_MHD_reply_with_error (
           connection,
-          MHD_HTTP_INTERNAL_SERVER_ERROR,
+          MHD_HTTP_BAD_REQUEST,
           TALER_EC_GENERIC_PARAMETER_MALFORMED,
           "by");
       }
