@@ -71,7 +71,11 @@ transfer_cb (void *cls,
                            true),
     GNUNET_JSON_pack_bool ("expected",
                            expected),
-    GNUNET_JSON_pack_timestamp (
+    GNUNET_TIME_absolute_is_zero (execution_time)
+    ? GNUNET_JSON_pack_allow_null (
+      GNUNET_JSON_pack_string ("dummy",
+                               NULL))
+    : GNUNET_JSON_pack_timestamp (
       "execution_time",
       GNUNET_TIME_absolute_to_timestamp (execution_time)));
   GNUNET_assert (0 ==
