@@ -69,6 +69,7 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
   const char *name;
   struct TMH_WireMethod *wm_head = NULL;
   struct TMH_WireMethod *wm_tail = NULL;
+  const char *iphone = NULL;
   bool no_transfer_delay;
   bool no_pay_delay;
   bool no_refund_delay;
@@ -85,7 +86,7 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
       NULL),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_string ("phone_number",
-                               (const char **) &is.phone),
+                               &iphone),
       NULL),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_string ("logo",
@@ -143,7 +144,7 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
                                        "address");
   }
   if ( (NULL != is.logo) &&
-       (! TMH_image_data_url_valid (is.logo)) )
+       (! TALER_MERCHANT_image_data_url_valid (is.logo)) )
   {
     GNUNET_break_op (0);
     GNUNET_JSON_parse_free (spec);
@@ -196,16 +197,40 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
                                        TALER_EC_GENERIC_PARAMETER_MALFORMED,
                                        "default_wire_transfer_delay");
   }
-
+  if (NULL != iphone)
+  {
+    is.phone = TALER_MERCHANT_phone_validate_normalize (iphone,
+                                                        false);
+    if (NULL == is.phone)
+    {
+      GNUNET_break_op (0);
+      GNUNET_JSON_parse_free (spec);
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_BAD_REQUEST,
+                                         TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                         "phone_number");
+    }
+  }
+  if ( (NULL != is.email) &&
+       (! TALER_MERCHANT_email_valid (is.email)) )
+  {
+    GNUNET_break_op (0);
+    GNUNET_JSON_parse_free (spec);
+    GNUNET_free (is.phone);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                       "email");
+  }
   if ( (NULL != is.phone) &&
        (NULL != mi->settings.phone) &&
-       0 == strcmp (mi->settings.phone,
-                    is.phone) )
+       (0 == strcmp (mi->settings.phone,
+                     is.phone)) )
     is.phone_validated = mi->settings.phone_validated;
   if ( (NULL != is.email) &&
        (NULL != mi->settings.email) &&
-       0 == strcmp (mi->settings.email,
-                    is.email) )
+       (0 == strcmp (mi->settings.email,
+                     is.email)) )
     is.email_validated = mi->settings.email_validated;
   if (mfa_check)
   {
@@ -213,10 +238,12 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
     enum TEH_TanChannelSet mtc = TEH_mandatory_tan_channels;
 
     if ( (0 != (mtc & TEH_TCS_SMS)) &&
+         (NULL != mi->settings.phone) &&
          (NULL == is.phone) )
     {
       GNUNET_break_op (0);
       GNUNET_JSON_parse_free (spec);
+      GNUNET_free (is.phone);
       return TALER_MHD_reply_with_error (
         connection,
         MHD_HTTP_BAD_REQUEST,
@@ -224,20 +251,24 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
         "phone_number");
     }
     if ( (0 != (mtc & TEH_TCS_EMAIL)) &&
+         (NULL != mi->settings.email) &&
          (NULL == is.email) )
     {
       GNUNET_break_op (0);
       GNUNET_JSON_parse_free (spec);
+      GNUNET_free (is.phone);
       return TALER_MHD_reply_with_error (
         connection,
         MHD_HTTP_BAD_REQUEST,
         TALER_EC_GENERIC_PARAMETER_MISSING,
         "email");
     }
-    if ( (is.phone_validated) &&
+    if ( (is.phone_validated ||
+          (NULL == is.phone) ) &&
          (0 != (mtc & TEH_TCS_SMS)) )
       mtc -= TEH_TCS_SMS;
-    if ( (is.email_validated) &&
+    if ( (is.email_validated ||
+          (NULL == is.email) ) &&
          (0 != (mtc & TEH_TCS_EMAIL)) )
       mtc -= TEH_TCS_EMAIL;
     switch (mtc)
@@ -246,16 +277,7 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
       ret = GNUNET_OK;
       break;
     case TEH_TCS_SMS:
-      if (NULL == is.phone)
-      {
-        GNUNET_break_op (0);
-        GNUNET_JSON_parse_free (spec);
-        return TALER_MHD_reply_with_error (
-          connection,
-          MHD_HTTP_BAD_REQUEST,
-          TALER_EC_GENERIC_PARAMETER_MISSING,
-          "phone_number");
-      }
+      GNUNET_assert (NULL != is.phone);
       is.phone_validated = true;
       /* validate new phone number, if possible require old e-mail
          address for authorization */
@@ -272,6 +294,7 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
                                    TALER_MERCHANT_MFA_CHANNEL_NONE);
       break;
     case TEH_TCS_EMAIL:
+      GNUNET_assert (NULL != is.email);
       is.email_validated = true;
       /* validate new e-mail address, if possible require old phone
          address for authorization */
@@ -288,6 +311,8 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
                                    TALER_MERCHANT_MFA_CHANNEL_NONE);
       break;
     case TEH_TCS_EMAIL_AND_SMS:
+      GNUNET_assert (NULL != mi->settings.phone);
+      GNUNET_assert (NULL != mi->settings.email);
       is.phone_validated = true;
       is.email_validated = true;
       /* To change both, we require both old and both new
@@ -309,6 +334,7 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
     if (GNUNET_OK != ret)
     {
       GNUNET_JSON_parse_free (spec);
+      GNUNET_free (is.phone);
       return (GNUNET_NO == ret)
         ? MHD_YES
         : MHD_NO;
@@ -334,6 +360,7 @@ patch_instances_ID (struct TMH_MerchantInstance *mi,
                        "PATCH /instances"))
     {
       GNUNET_JSON_parse_free (spec);
+      GNUNET_free (is.phone);
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_INTERNAL_SERVER_ERROR,
                                          TALER_EC_GENERIC_DB_START_FAILED,
@@ -406,8 +433,8 @@ giveup:
   mi->settings.name = GNUNET_strdup (name);
   if (NULL != is.email)
     mi->settings.email = GNUNET_strdup (is.email);
-  if (NULL != is.phone)
-    mi->settings.phone = GNUNET_strdup (is.phone);
+  mi->settings.phone = is.phone;
+  is.phone = NULL;
   if (NULL != is.website)
     mi->settings.website = GNUNET_strdup (is.website);
   if (NULL != is.logo)
