@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2020--2025 Taler Systems SA
+  (C) 2020--2026 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU Affero General Public License as
@@ -50,6 +50,7 @@ TMH_private_patch_products_ID (
   const char *unit_total_stock = NULL;
   bool unit_total_stock_missing;
   bool total_stock_missing;
+  struct TALER_Amount price;
   bool price_missing;
   bool unit_price_missing;
   bool unit_allow_fraction;
@@ -71,10 +72,16 @@ TMH_private_patch_products_ID (
       NULL),
     GNUNET_JSON_spec_string ("unit",
                              (const char **) &pd.unit),
+    // FIXME: deprecated API
     GNUNET_JSON_spec_mark_optional (
       TALER_JSON_spec_amount_any ("price",
-                                  &pd.price),
+                                  &price),
       &price_missing),
+    GNUNET_JSON_spec_mark_optional (
+      TALER_JSON_spec_amount_any_array ("unit_price",
+                                        &pd.price_array_length,
+                                        &pd.price_array),
+      &unit_price_missing),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_string ("image",
                                (const char **) &pd.image),
@@ -103,15 +110,18 @@ TMH_private_patch_products_ID (
       GNUNET_JSON_spec_uint32 ("unit_precision_level",
                                &unit_precision_level),
       &unit_precision_missing),
-    GNUNET_JSON_spec_mark_optional (
-      TALER_JSON_spec_amount_any_array ("unit_price",
-                                        &pd.price_array_length,
-                                        &pd.price_array),
-      &unit_price_missing),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_uint64 ("total_lost",
                                &pd.total_lost),
       NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_uint64 ("product_group_id",
+                               &pd.product_group_id),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_uint64 ("money_pot_id",
+                               &pd.money_pot_id),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_json ("address",
                              &pd.address),
@@ -135,8 +145,9 @@ TMH_private_patch_products_ID (
   bool lost_reduced;
   bool sold_reduced;
   bool stock_reduced;
+  bool no_group;
+  bool no_pot;
 
-  pd.total_sold = 0; /* will be ignored anyway */
   GNUNET_assert (NULL != mi);
   GNUNET_assert (NULL != product_id);
   {
@@ -158,7 +169,7 @@ TMH_private_patch_products_ID (
   {
     if (! price_missing)
     {
-      if (0 != TALER_amount_cmp (&pd.price,
+      if (0 != TALER_amount_cmp (&price,
                                  &pd.price_array[0]))
       {
         ret = TALER_MHD_reply_with_error (connection,
@@ -168,10 +179,15 @@ TMH_private_patch_products_ID (
         goto cleanup;
       }
     }
-    else
+    if (GNUNET_OK !=
+        TMH_validate_unit_price_array (pd.price_array,
+                                       pd.price_array_length))
     {
-      pd.price = pd.price_array[0];
-      price_missing = false;
+      ret = TALER_MHD_reply_with_error (connection,
+                                        MHD_HTTP_BAD_REQUEST,
+                                        TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                        "unit_price");
+      goto cleanup;
     }
   }
   else
@@ -186,7 +202,7 @@ TMH_private_patch_products_ID (
     }
     pd.price_array = GNUNET_new_array (1,
                                        struct TALER_Amount);
-    pd.price_array[0] = pd.price;
+    pd.price_array[0] = price;
     pd.price_array_length = 1;
   }
   if (! unit_precision_missing)
@@ -229,15 +245,16 @@ TMH_private_patch_products_ID (
   {
     const char *eparam;
     if (GNUNET_OK !=
-        TMH_process_quantity_inputs (TMH_VK_STOCK,
-                                     unit_allow_fraction,
-                                     total_stock_missing,
-                                     total_stock,
-                                     unit_total_stock_missing,
-                                     unit_total_stock,
-                                     &pd.total_stock,
-                                     &pd.total_stock_frac,
-                                     &eparam))
+        TALER_MERCHANT_vk_process_quantity_inputs (
+          TALER_MERCHANT_VK_STOCK,
+          unit_allow_fraction,
+          total_stock_missing,
+          total_stock,
+          unit_total_stock_missing,
+          unit_total_stock,
+          &pd.total_stock,
+          &pd.total_stock_frac,
+          &eparam))
     {
       ret = TALER_MHD_reply_with_error (
         connection,
@@ -298,7 +315,7 @@ TMH_private_patch_products_ID (
   if (NULL == pd.taxes)
     pd.taxes = json_array ();
   /* check taxes is well-formed */
-  if (! TMH_taxes_array_valid (pd.taxes))
+  if (! TALER_MERCHANT_taxes_array_valid (pd.taxes))
   {
     GNUNET_break_op (0);
     ret = TALER_MHD_reply_with_error (connection,
@@ -310,7 +327,7 @@ TMH_private_patch_products_ID (
 
   if (NULL == pd.image)
     pd.image = (char *) "";
-  if (! TMH_image_data_url_valid (pd.image))
+  if (! TALER_MERCHANT_image_data_url_valid (pd.image))
   {
     GNUNET_break_op (0);
     ret = TALER_MHD_reply_with_error (connection,
@@ -343,7 +360,9 @@ TMH_private_patch_products_ID (
                                &no_product,
                                &lost_reduced,
                                &sold_reduced,
-                               &stock_reduced);
+                               &stock_reduced,
+                               &no_group,
+                               &no_pot);
   switch (qs)
   {
   case GNUNET_DB_STATUS_HARD_ERROR:
@@ -401,6 +420,24 @@ TMH_private_patch_products_ID (
                                       product_id);
     goto cleanup;
   }
+  if (no_group)
+  {
+    ret = TALER_MHD_reply_with_error (
+      connection,
+      MHD_HTTP_NOT_FOUND,
+      TALER_EC_MERCHANT_GENERIC_PRODUCT_GROUP_UNKNOWN,
+      NULL);
+    goto cleanup;
+  }
+  if (no_pot)
+  {
+    ret = TALER_MHD_reply_with_error (
+      connection,
+      MHD_HTTP_NOT_FOUND,
+      TALER_EC_MERCHANT_GENERIC_MONEY_POT_UNKNOWN,
+      NULL);
+    goto cleanup;
+  }
   if (lost_reduced)
   {
     ret = TALER_MHD_reply_with_error (
