@@ -49,26 +49,46 @@ trigger_refund_notification (
   struct TMH_HandlerContext *hc,
   const struct TALER_Amount *amount)
 {
-  const char *as;
-  struct TMH_OrderRefundEventP refund_eh = {
-    .header.size = htons (sizeof (refund_eh)),
-    .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_REFUND),
-    .merchant_pub = hc->instance->merchant_pub
-  };
+  {
+    const char *as;
+    struct TMH_OrderRefundEventP refund_eh = {
+      .header.size = htons (sizeof (refund_eh)),
+      .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_REFUND),
+      .merchant_pub = hc->instance->merchant_pub
+    };
 
-  /* Resume clients that may wait for this refund */
-  as = TALER_amount2s (amount);
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Awakening clients on %s waiting for refund of no more than %s\n",
-              hc->infix,
-              as);
-  GNUNET_CRYPTO_hash (hc->infix,
-                      strlen (hc->infix),
-                      &refund_eh.h_order_id);
-  TMH_db->event_notify (TMH_db->cls,
-                        &refund_eh.header,
-                        as,
-                        strlen (as));
+    /* Resume clients that may wait for this refund */
+    as = TALER_amount2s (amount);
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Awakening clients on %s waiting for refund of no more than %s\n",
+                hc->infix,
+                as);
+    GNUNET_CRYPTO_hash (hc->infix,
+                        strlen (hc->infix),
+                        &refund_eh.h_order_id);
+    TMH_db->event_notify (TMH_db->cls,
+                          &refund_eh.header,
+                          as,
+                          strlen (as));
+  }
+  {
+    struct TMH_OrderPayEventP pay_eh = {
+      .header.size = htons (sizeof (pay_eh)),
+      .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_STATUS_CHANGED),
+      .merchant_pub = hc->instance->merchant_pub
+    };
+
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Notifying clients about status change of order %s\n",
+                hc->infix);
+    GNUNET_CRYPTO_hash (hc->infix,
+                        strlen (hc->infix),
+                        &pay_eh.h_order_id);
+    TMH_db->event_notify (TMH_db->cls,
+                          &pay_eh.header,
+                          NULL,
+                          0);
+  }
 }
 
 
