@@ -23,6 +23,14 @@
 
 #include "taler-merchant-httpd.h"
 
+
+/**
+ * Force resuming all suspended orders on shutdown.
+ */
+void
+TMH_force_orders_resume (void);
+
+
 /**
  * Generate an order.  We add the fields 'exchanges', 'merchant_pub', and
  * 'H_wire' to the order gotten from the frontend, as well as possibly other
@@ -38,25 +46,5 @@ TMH_private_post_orders (const struct TMH_RequestHandler *rh,
                          struct MHD_Connection *connection,
                          struct TMH_HandlerContext *hc);
 
-/**
- * Generate an order.  We add the fields 'exchanges', 'merchant_pub', and
- * 'H_wire' to the order gotten from the frontend, as well as possibly other
- * fields if the frontend did not provide them. Returns the order_id.
- *
- * @param rh context of the handler
- * @param connection the MHD connection to handle
- * @param[in,out] hc context with further information about the request
- * @param pos_key key identifying the POS, can be NULL
- * @param pos_algorithm algorithm for computing the POS confirmation
- * @return MHD result code
- */
-MHD_RESULT
-TMH_private_post_orders_with_pos_secrets (
-  const struct TMH_RequestHandler *rh,
-  struct MHD_Connection *connection,
-  struct TMH_HandlerContext *hc,
-  const char *pos_key,
-  enum TALER_MerchantConfirmationAlgorithm pos_algorithm);
-
 
 #endif
