@@ -140,15 +140,16 @@ TMH_private_post_products_ID_lock (
     {
       const char *eparam;
       if (GNUNET_OK !=
-          TMH_process_quantity_inputs (TMH_VK_QUANTITY,
-                                       pd.allow_fractional_quantity,
-                                       quantity_missing,
-                                       (int64_t) quantity,
-                                       unit_quantity_missing,
-                                       unit_quantity,
-                                       &normalized_quantity,
-                                       &normalized_quantity_frac,
-                                       &eparam))
+          TALER_MERCHANT_vk_process_quantity_inputs (
+            TALER_MERCHANT_VK_QUANTITY,
+            pd.allow_fractional_quantity,
+            quantity_missing,
+            (int64_t) quantity,
+            unit_quantity_missing,
+            unit_quantity,
+            &normalized_quantity,
+            &normalized_quantity_frac,
+            &eparam))
       {
         TALER_MERCHANTDB_product_details_free (&pd);
         GNUNET_break_op (0);
