@@ -101,7 +101,7 @@ TMH_private_post_templates (const struct TMH_RequestHandler *rh,
              : MHD_NO;
     }
   }
-  if (! TMH_template_contract_valid (tp.template_contract))
+  if (! TALER_MERCHANT_template_contract_valid (tp.template_contract))
   {
     GNUNET_break_op (0);
     json_dumpf (tp.template_contract,
