@@ -341,6 +341,19 @@ static struct GNUNET_DB_EventHandler *eh_keys;
  */
 static struct GNUNET_DB_EventHandler *eh_rule;
 
+/**
+ * Event handler to learn that higher-frequency KYC
+ * checks were forced by an application actively inspecting
+ * some KYC status values.
+ */
+static struct GNUNET_DB_EventHandler *eh_update_forced;
+
+/**
+ * Event handler to learn that we got new /keys
+ * from an exchange and should reconsider eligibility.
+ */
+static struct GNUNET_DB_EventHandler *keys_rule;
+
 /**
  * Main task to discover (new) accounts.
  */
@@ -640,6 +653,8 @@ exchange_check_cb (
       &i->a->h_wire,
       i->e->keys->exchange_url,
       i->last_kyc_check,
+      i->due,
+      i->backoff,
       i->last_http_status,
       i->last_ec,
       i->rule_gen,
@@ -657,7 +672,7 @@ exchange_check_cb (
       return;
     }
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                "account_set_kyc_status (%s, %u, %s, %s) returned %d\n",
+                "account_kyc_set_status (%s, %u, %s, %s) returned %d\n",
                 i->e->keys->exchange_url,
                 i->last_http_status,
                 i->auth_ok ? "auth OK" : "auth needed",
@@ -814,6 +829,8 @@ start_inquiry (struct Exchange *e,
                                   &i->last_ec,
                                   &i->rule_gen,
                                   &i->last_kyc_check,
+                                  &i->due,
+                                  &i->backoff,
                                   &i->aml_review,
                                   &i->jlimits);
   if (qs < 0)
@@ -825,54 +842,8 @@ start_inquiry (struct Exchange *e,
   }
   if (qs > 0)
     i->not_first_time = true;
-  switch (i->last_http_status)
-  {
-  case MHD_HTTP_OK:
-    /* KYC is OK, but we could have missed some triggers,
-       so let's check, but slowly within the next minute
-       so that we do not overdo it if this process happens
-       to be restarted a lot. */
-    if (GNUNET_YES != test_mode)
-    {
-      i->due = GNUNET_TIME_relative_to_absolute (
-        GNUNET_TIME_randomize (GNUNET_TIME_UNIT_MINUTES));
-      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                  "Previous KYC status is OK, randomizing inquiry to start at %s\n",
-                  GNUNET_TIME_absolute2s (i->due));
-    }
-    break;
-  case MHD_HTTP_ACCEPTED:
-    /* KYC required, due immediately */
-    break;
-  case MHD_HTTP_NO_CONTENT:
-    /* KYC is OFF, only check again if triggered */
-    if (GNUNET_YES != test_mode)
-    {
-      i->due = GNUNET_TIME_relative_to_absolute (
-        GNUNET_TIME_randomize (aml_low_freq));
-      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                  "KYC was disabled, randomizing inquiry to start at %s\n",
-                  GNUNET_TIME_absolute2s (i->due));
-    }
-    break;
-  case MHD_HTTP_FORBIDDEN: /* bad signature */
-  case MHD_HTTP_NOT_FOUND: /* account unknown */
-  case MHD_HTTP_CONFLICT: /* no account_pub known */
-    /* go immediately into long-polling */
-    break;
-  default:
-    /* start with decent back-off after hard failure */
-    if (GNUNET_YES != test_mode)
-    {
-      i->backoff
-        = GNUNET_TIME_randomize (GNUNET_TIME_UNIT_MINUTES);
-      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                  "Last KYC check failed, starting with backoff %s\n",
-                  GNUNET_TIME_relative2s (i->backoff,
-                                          true));
-    }
-    break;
-  }
+  if (GNUNET_YES == test_mode)
+    i->due = GNUNET_TIME_UNIT_ZERO_ABS; /* immediately */
   inquiry_work (i);
 }
 
@@ -934,6 +905,54 @@ stop_inquiry_at (struct Exchange *e,
 }
 
 
+/**
+ * Set the account @a h_wire of @a instance_id to be ineligible
+ * for the exchange at @a exchange_url and thus no need to do KYC checks.
+ *
+ * @param instance_id instance that has the account
+ * @param exchange_url base URL of the exchange
+ * @param h_wire hash of the merchant bank account that is ineligible
+ */
+static void
+flag_ineligible (const char *instance_id,
+                 const char *exchange_url,
+                 const struct TALER_MerchantWireHashP *h_wire)
+{
+  enum GNUNET_DB_QueryStatus qs;
+
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Account %s not eligible at exchange %s\n",
+              TALER_B2S (h_wire),
+              exchange_url);
+  qs = db_plugin->account_kyc_set_status (
+    db_plugin->cls,
+    instance_id,
+    h_wire,
+    exchange_url,
+    GNUNET_TIME_timestamp_get (),
+    GNUNET_TIME_UNIT_FOREVER_ABS,
+    GNUNET_TIME_UNIT_FOREVER_REL,
+    0,
+    TALER_EC_MERCHANT_PRIVATE_ACCOUNT_NOT_ELIGIBLE_FOR_EXCHANGE,
+    0,
+    NULL,
+    NULL,
+    false,
+    false);
+  if (qs < 0)
+  {
+    GNUNET_break (0);
+    global_ret = EXIT_FAILURE;
+    GNUNET_SCHEDULER_shutdown ();
+    return;
+  }
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "account_kyc_set_status (%s) returned %d\n",
+              exchange_url,
+              (int) qs);
+}
+
+
 /**
  * Start inquries for all exchanges on account @a a.
  *
@@ -948,13 +967,16 @@ start_inquiries (struct Account *a)
   {
     if (is_eligible (e->keys,
                      a))
+    {
       start_inquiry (e,
                      a);
+    }
     else
-      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                  "Account %s not eligible at exchange %s\n",
-                  a->merchant_account_uri.full_payto,
-                  e->keys->exchange_url);
+    {
+      flag_ineligible (a->instance_id,
+                       e->keys->exchange_url,
+                       &a->h_wire);
+    }
   }
 }
 
@@ -1053,6 +1075,8 @@ find_accounts (void *cls)
   if (qs < 0)
   {
     GNUNET_break (0);
+    global_ret = EXIT_FAILURE;
+    GNUNET_SCHEDULER_shutdown ();
     return;
   }
   for (struct Account *a = a_head;
@@ -1062,6 +1086,15 @@ find_accounts (void *cls)
     if (a->account_gen < database_gen)
       stop_inquiries (a);
   }
+  if ( (! at_limit) &&
+       (0 == active_inquiries) &&
+       (test_mode) )
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "No more open inquiries and in test mode. Existing.\n");
+    GNUNET_SCHEDULER_shutdown ();
+    return;
+  }
 }
 
 
@@ -1083,6 +1116,8 @@ account_changed (void *cls,
   (void) extra_size;
   if (NULL != account_task)
     return;
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Received account change notification: reloading accounts\n");
   account_task
     = GNUNET_SCHEDULER_add_now (&find_accounts,
                                 NULL);
@@ -1110,6 +1145,8 @@ find_keys (const char *exchange_url)
   if (qs < 0)
   {
     GNUNET_break (0);
+    global_ret = EXIT_FAILURE;
+    GNUNET_SCHEDULER_shutdown ();
     return;
   }
   if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
@@ -1187,11 +1224,15 @@ keys_changed (void *cls,
        (0 == extra_size) )
   {
     GNUNET_break (0);
+    global_ret = EXIT_FAILURE;
+    GNUNET_SCHEDULER_shutdown ();
     return;
   }
   if ('\0' != url[extra_size - 1])
   {
     GNUNET_break (0);
+    global_ret = EXIT_FAILURE;
+    GNUNET_SCHEDULER_shutdown ();
     return;
   }
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
@@ -1225,11 +1266,15 @@ rule_triggered (void *cls,
        (0 == extra_size) )
   {
     GNUNET_break (0);
+    global_ret = EXIT_FAILURE;
+    GNUNET_SCHEDULER_shutdown ();
     return;
   }
   if ('\0' != text[extra_size - 1])
   {
     GNUNET_break (0);
+    global_ret = EXIT_FAILURE;
+    GNUNET_SCHEDULER_shutdown ();
     return;
   }
   space = memchr (extra,
@@ -1238,6 +1283,8 @@ rule_triggered (void *cls,
   if (NULL == space)
   {
     GNUNET_break (0);
+    global_ret = EXIT_FAILURE;
+    GNUNET_SCHEDULER_shutdown ();
     return;
   }
   if (GNUNET_OK !=
@@ -1247,12 +1294,16 @@ rule_triggered (void *cls,
                                      sizeof (h_wire)))
   {
     GNUNET_break (0);
+    global_ret = EXIT_FAILURE;
+    GNUNET_SCHEDULER_shutdown ();
     return;
   }
   exchange_url = &space[1];
   if (! TALER_is_web_url (exchange_url))
   {
     GNUNET_break (0);
+    global_ret = EXIT_FAILURE;
+    GNUNET_SCHEDULER_shutdown ();
     return;
   }
 
@@ -1400,6 +1451,16 @@ shutdown_task (void *cls)
     db_plugin->event_listen_cancel (eh_rule);
     eh_rule = NULL;
   }
+  if (NULL != eh_update_forced)
+  {
+    db_plugin->event_listen_cancel (eh_update_forced);
+    eh_update_forced = NULL;
+  }
+  if (NULL != keys_rule)
+  {
+    db_plugin->event_listen_cancel (keys_rule);
+    keys_rule = NULL;
+  }
   TALER_MERCHANTDB_plugin_unload (db_plugin);
   db_plugin = NULL;
   cfg = NULL;
@@ -1416,6 +1477,100 @@ shutdown_task (void *cls)
 }
 
 
+/**
+ * Function called when we urgently need to re-check the KYC status
+ * of some account. Finds the respective inquiry and re-launches
+ * the check, unless we are already doing it.
+ *
+ * @param cls NULL
+ * @param instance_id instance for which to force the check
+ * @param exchange_url base URL of the exchange to check
+ * @param h_wire hash of the wire account to check KYC status for
+ */
+static void
+force_check_now (void *cls,
+                 const char *instance_id,
+                 const char *exchange_url,
+                 const struct TALER_MerchantWireHashP *h_wire)
+{
+  for (struct Account *a = a_head;
+       NULL != a;
+       a = a->next)
+  {
+    if (0 !=
+        strcmp (instance_id,
+                a->instance_id))
+      continue;
+    if (0 !=
+        GNUNET_memcmp (h_wire,
+                       &a->h_wire))
+      continue;
+    for (struct Inquiry *i = a->i_head;
+         NULL != i;
+         i = i->next)
+    {
+      if (0 !=
+          strcmp (i->e->keys->exchange_url,
+                  exchange_url))
+        continue;
+      /* If we are not actively checking with the exchange, do start
+         to check immediately */
+      if (NULL != i->kyc)
+      {
+        i->due = GNUNET_TIME_absolute_get (); /* now! */
+        if (NULL != i->task)
+          GNUNET_SCHEDULER_cancel (i->task);
+        i->task = GNUNET_SCHEDULER_add_at (i->due,
+                                           &inquiry_work,
+                                           i);
+      }
+      return;
+    }
+  }
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "No inquiry at `%s' for exchange `%s' and h_wire `%s'. Likely the account is not eligible.\n",
+              instance_id,
+              exchange_url,
+              TALER_B2S (h_wire));
+  /* In this case, set the due date back to FOREVER */
+  flag_ineligible (instance_id,
+                   exchange_url,
+                   h_wire);
+}
+
+
+/**
+ * Function called when a KYC status update was forced by an
+ * application checking the KYC status of an account.
+ *
+ * @param cls closure (NULL)
+ * @param extra additional event data provided
+ * @param extra_size number of bytes in @a extra
+ */
+static void
+update_forced (void *cls,
+               const void *extra,
+               size_t extra_size)
+{
+  enum GNUNET_DB_QueryStatus qs;
+
+  (void) cls;
+  (void) extra;
+  (void) extra_size;
+  qs = db_plugin->account_kyc_get_outdated (
+    db_plugin->cls,
+    &force_check_now,
+    NULL);
+  if (qs < 0)
+  {
+    GNUNET_break (0);
+    global_ret = EXIT_FAILURE;
+    GNUNET_SCHEDULER_shutdown ();
+    return;
+  }
+}
+
+
 /**
  * First task.
  *
@@ -1512,6 +1667,19 @@ run (void *cls,
                                  &keys_changed,
                                  NULL);
   }
+  {
+    struct GNUNET_DB_EventHeaderP es = {
+      .size = htons (sizeof (es)),
+      .type = htons (TALER_DBEVENT_MERCHANT_EXCHANGE_KYC_UPDATE_FORCED)
+    };
+
+    eh_update_forced
+      = db_plugin->event_listen (db_plugin->cls,
+                                 &es,
+                                 GNUNET_TIME_UNIT_FOREVER_REL,
+                                 &update_forced,
+                                 NULL);
+  }
   {
     struct GNUNET_DB_EventHeaderP es = {
       .size = htons (sizeof (es)),
@@ -1528,6 +1696,7 @@ run (void *cls,
   GNUNET_CONFIGURATION_iterate_sections (cfg,
                                          &accept_exchanges,
                                          NULL);
+  fprintf (stderr, "NOW\n");
   {
     struct GNUNET_DB_EventHeaderP es = {
       .size = htons (sizeof (es)),
