@@ -259,7 +259,7 @@ launch_inquiries_at_exchange (struct Exchange *e)
        NULL != w;
        w = w->next)
   {
-    if (e->exchange_inquiries > EXCHANGE_INQUIRY_LIMIT)
+    if (e->exchange_inquiries >= EXCHANGE_INQUIRY_LIMIT)
       break;
     if ( (NULL == w->task) &&
          (NULL == w->wdh) )
@@ -334,7 +334,9 @@ sync_keys (struct Exchange *e)
   }
   if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
   {
-    GNUNET_break (0);
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Cannot launch inquiries at `%s': lacking /keys response\n",
+                e->exchange_url);
     return;
   }
   TALER_EXCHANGE_keys_decref (e->keys);
@@ -417,7 +419,7 @@ end_inquiry (struct Inquiry *w)
        (test_mode) )
   {
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                "No more open inquiries and in test mode. Existing.\n");
+                "No more open inquiries and in test mode. Exiting.\n");
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
@@ -1058,7 +1060,7 @@ find_work (void *cls)
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
-  if (qs == limit)
+  if (qs >= limit)
   {
     /* DB limited response, re-trigger DB interaction
        the moment we significantly fall below the
