@@ -441,7 +441,7 @@ credit_cb (
     /* Something went wrong, try again, but with back-off */
     w->delay = GNUNET_TIME_STD_BACKOFF (w->delay);
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Unexpected HTTP status code %u(%d) from bank\n",
+                "Unexpected HTTP status code %u(%d) from wire gateway\n",
                 http_status,
                 ec);
     break;
