@@ -27,38 +27,29 @@ SET search_path TO merchant;
 ALTER TABLE merchant_inventory
     ADD COLUMN price_array taler_amount_currency[]
         NOT NULL
-        DEFAULT ARRAY[]::taler_amount_currency[];
+        DEFAULT ARRAY[]::taler_amount_currency[],
+    ADD COLUMN total_stock_frac INT4 NOT NULL DEFAULT 0,
+    ADD COLUMN total_sold_frac INT4 NOT NULL DEFAULT 0,
+    ADD COLUMN total_lost_frac INT4 NOT NULL DEFAULT 0,
+    ADD COLUMN allow_fractional_quantity BOOL NOT NULL DEFAULT FALSE,
+    ADD COLUMN fractional_precision_level INT4 NOT NULL DEFAULT 0;
 COMMENT ON COLUMN merchant_inventory.price_array
     IS 'List of unit prices available for the product (multiple tiers supported).';
 
 UPDATE merchant_inventory
-SET price_array = ARRAY[price]::taler_amount_currency[]
-WHERE price IS NOT NULL; -- theoretically all objects, but just to be sure
+   SET price_array = ARRAY[price]::taler_amount_currency[]
+ WHERE price IS NOT NULL; -- theoretically all objects, but just to be sure
 
--- I assume we want to make drop price column at some point of time
+-- Note: price column is dropped in merchant-0028.sql
 
-ALTER TABLE merchant_inventory
-    ADD COLUMN total_stock_frac INT4 NOT NULL DEFAULT 0;
 COMMENT ON COLUMN merchant_inventory.total_stock_frac
     IS 'Fractional part of stock in units of 1/1000000 of the base value';
-
-ALTER TABLE merchant_inventory
-    ADD COLUMN total_sold_frac INT4 NOT NULL DEFAULT 0;
 COMMENT ON COLUMN merchant_inventory.total_sold_frac
     IS 'Fractional part of units sold in units of 1/1000000 of the base value';
-
-ALTER TABLE merchant_inventory
-    ADD COLUMN total_lost_frac INT4 NOT NULL DEFAULT 0;
 COMMENT ON COLUMN merchant_inventory.total_lost_frac
     IS 'Fractional part of units lost in units of 1/1000000 of the base value';
-
-ALTER TABLE merchant_inventory
-    ADD COLUMN allow_fractional_quantity BOOL NOT NULL DEFAULT FALSE;
 COMMENT ON COLUMN merchant_inventory.allow_fractional_quantity
     IS 'Whether fractional stock (total_stock_frac) should be honored for this product';
-
-ALTER TABLE merchant_inventory
-    ADD COLUMN fractional_precision_level INT4 NOT NULL DEFAULT 0;
 COMMENT ON COLUMN merchant_inventory.fractional_precision_level
     IS 'Preset number of decimal places for fractional quantities';
 
