@@ -1,6 +1,6 @@
 /*
    This file is part of TALER
-   Copyright (C) 2022 Taler Systems SA
+   Copyright (C) 2022, 2026 Taler Systems SA
 
    TALER is free software; you can redistribute it and/or modify it under the
    terms of the GNU General Public License as published by the Free Software
@@ -170,8 +170,11 @@ TMH_PG_account_kyc_get_status (
     .kyc_cb = kyc_cb,
     .kyc_cb_cls = kyc_cb_cls
   };
+  struct GNUNET_TIME_Absolute now
+    = GNUNET_TIME_absolute_get ();
   struct GNUNET_PQ_QueryParam params[] = {
     GNUNET_PQ_query_param_string (merchant_id),
+    GNUNET_PQ_query_param_absolute_time (&now),
     NULL == exchange_url
     ? GNUNET_PQ_query_param_null ()
     : GNUNET_PQ_query_param_string (exchange_url),
@@ -185,28 +188,18 @@ TMH_PG_account_kyc_get_status (
   check_connection (pg);
   PREPARE (pg,
            "lookup_kyc_status",
-           "SELECT"
-           " ma.h_wire"
-           ",ma.payto_uri"
-           ",mk.exchange_url"
-           ",mk.kyc_timestamp"
-           ",mk.kyc_ok"
-           ",mk.access_token"
-           ",mk.exchange_http_status"
-           ",mk.exchange_ec_code"
-           ",mk.aml_review"
-           ",mk.jaccount_limits::TEXT"
-           " FROM merchant_instances mi"
-           " JOIN merchant_accounts ma"
-           "   USING (merchant_serial)"
-           " LEFT JOIN merchant_kyc mk"
-           "   USING (account_serial)"
-           " WHERE (mi.merchant_id=$1)"
-           "   AND ma.active"
-           "   AND ( ($2::TEXT IS NULL)"
-           "      OR (mk.exchange_url=$2) )"
-           "   AND ( ($3::BYTEA IS NULL)"
-           "      OR (ma.h_wire=$3) );");
+           "SELECT "
+           "  out_h_wire AS h_wire"
+           " ,out_payto_uri AS payto_uri"
+           " ,out_exchange_url AS exchange_url"
+           " ,out_kyc_timestamp AS kyc_timestamp"
+           " ,out_kyc_ok AS kyc_ok"
+           " ,out_access_token AS access_token"
+           " ,out_exchange_http_status AS exchange_http_status"
+           " ,out_exchange_ec_code AS exchange_ec_code"
+           " ,out_aml_review AS aml_review"
+           " ,out_jaccount_limits::TEXT AS jaccount_limits"
+           " FROM merchant_do_account_kyc_get_status($1, $2, $3, $4);");
   qs = GNUNET_PQ_eval_prepared_multi_select (
     pg->conn,
     "lookup_kyc_status",
