@@ -33,6 +33,8 @@
  * @param h_wire hash of the wire account to check
  * @param exchange_url base URL of the exchange to check
  * @param timestamp timestamp to store
+ * @param next_time when should we next check the KYC status
+ * @param kyc_backoff what is the current backoff between KYC status checks
  * @param exchange_http_status HTTP status code returned last by the exchange
  * @param exchange_ec_code Taler error code returned last by the exchange
  * @param rule_gen generation of the rule in the exchange database
@@ -50,6 +52,8 @@ TMH_PG_account_kyc_set_status (
   const struct TALER_MerchantWireHashP *h_wire,
   const char *exchange_url,
   struct GNUNET_TIME_Timestamp timestamp,
+  struct GNUNET_TIME_Absolute next_time,
+  struct GNUNET_TIME_Relative kyc_backoff,
   unsigned int exchange_http_status,
   enum TALER_ErrorCode exchange_ec_code,
   uint64_t rule_gen,
