@@ -1,6 +1,6 @@
 --
 -- This file is part of TALER
--- Copyright (C) 2024 Taler Systems SA
+-- Copyright (C) 2024, 2026 Taler Systems SA
 --
 -- TALER is free software; you can redistribute it and/or modify it under the
 -- terms of the GNU General Public License as published by the Free Software
@@ -31,6 +31,8 @@ CREATE FUNCTION merchant_do_account_kyc_set_status (
   IN in_notify_str TEXT,
   IN in_notify2_str TEXT,
   IN in_rule_gen INT8,
+  IN in_next_time INT8,
+  IN in_kyc_backoff INT8,
   OUT out_no_instance BOOL,
   OUT out_no_account BOOL)
 LANGUAGE plpgsql
@@ -76,6 +78,8 @@ UPDATE merchant_kyc
       ,exchange_ec_code=in_exchange_ec_code
       ,access_token=in_access_token
       ,last_rule_gen=in_rule_gen
+      ,next_kyc_poll=in_next_time
+      ,kyc_backoff=in_kyc_backoff
  WHERE account_serial=my_account_serial
    AND exchange_url=in_exchange_url;
 
@@ -92,7 +96,9 @@ THEN
     ,exchange_http_status
     ,exchange_ec_code
     ,access_token
-    ,last_rule_gen)
+    ,last_rule_gen
+    ,next_kyc_poll
+    ,kyc_backoff)
   VALUES
     (in_timestamp
     ,in_kyc_ok
@@ -103,7 +109,9 @@ THEN
     ,in_exchange_http_status
     ,in_exchange_ec_code
     ,in_access_token
-    ,in_rule_gen);
+    ,in_rule_gen
+    ,in_next_time
+    ,in_kyc_backoff);
 END IF;
 
 EXECUTE FORMAT (
