@@ -32,10 +32,8 @@ DECLARE
   my_signkey_serial INT8;
   my_account_serial INT8;
   my_decose INT8;
-  my_order_serial INT8;
   my_expected_credit_serial INT8;
   my_wire_pending_cleared BOOL;
-  my_order_id TEXT;
 BEGIN
   -- Just to return something (for now).
   out_dummy=FALSE;
