@@ -38,6 +38,8 @@
  * @param[out] conflict set to true if a conflicting
  *        product already exists in the database
  * @param[out] no_cat set to index of non-existing category from @a cats, or -1 if all @a cats were found
+ * @param[out] no_group set to true if the product group in @a pd is unknown
+ * @param[out] no_pot set to true if the money pot in @a pd is unknown
  * @return database result code
  */
 enum GNUNET_DB_QueryStatus
@@ -49,7 +51,9 @@ TMH_PG_insert_product (void *cls,
                        const uint64_t *cats,
                        bool *no_instance,
                        bool *conflict,
-                       ssize_t *no_cat);
+                       ssize_t *no_cat,
+                       bool *no_group,
+                       bool *no_pot);
 
 
 #endif
