@@ -47,6 +47,7 @@ TMH_PG_insert_template (void *cls,
     : TALER_PQ_query_param_json (td->editable_defaults),
     GNUNET_PQ_query_param_end
   };
+  enum GNUNET_DB_QueryStatus qs;
 
   check_connection (pg);
   PREPARE (pg,
@@ -63,7 +64,9 @@ TMH_PG_insert_template (void *cls,
            " $2, $3, $4, $5::TEXT::JSONB, $6::TEXT::JSONB"
            " FROM merchant_instances"
            " WHERE merchant_id=$1");
-  return GNUNET_PQ_eval_prepared_non_select (pg->conn,
-                                             "insert_template",
-                                             params);
+  qs = GNUNET_PQ_eval_prepared_non_select (pg->conn,
+                                           "insert_template",
+                                           params);
+  GNUNET_PQ_cleanup_query_params_closures (params);
+  return qs;
 }
