@@ -43,6 +43,8 @@ TMH_PG_insert_token_family_key (
   struct GNUNET_TIME_Timestamp now
     = GNUNET_TIME_timestamp_get ();
   const char *cipher = NULL;
+
+#if DEBUG
   struct GNUNET_HashCode pub_hash;
 
   switch (pub->public_key->cipher)
@@ -64,11 +66,27 @@ TMH_PG_insert_token_family_key (
     GNUNET_break (0);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
-  GNUNET_assert (pub->public_key->cipher ==
-                 priv->private_key->cipher);
   GNUNET_assert (0 ==
                  GNUNET_memcmp (&pub_hash,
                                 &pub->public_key->pub_key_hash));
+#endif
+  switch (pub->public_key->cipher)
+  {
+  case GNUNET_CRYPTO_BSA_RSA:
+    cipher = "rsa";
+    break;
+  case GNUNET_CRYPTO_BSA_CS:
+    cipher = "cs";
+    break;
+  case GNUNET_CRYPTO_BSA_INVALID:
+    GNUNET_break (0);
+    return GNUNET_DB_STATUS_HARD_ERROR;
+  }
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Storing token public key with hash %s\n",
+              GNUNET_h2s (&pub->public_key->pub_key_hash));
+  GNUNET_assert (pub->public_key->cipher ==
+                 priv->private_key->cipher);
   GNUNET_assert (! GNUNET_TIME_absolute_is_zero (
                    valid_after.abs_time));
   GNUNET_assert (! GNUNET_TIME_absolute_is_zero (
