@@ -1,6 +1,6 @@
 /*
    This file is part of TALER
-   Copyright (C) 2022 Taler Systems SA
+   Copyright (C) 2022, 2026 Taler Systems SA
 
    TALER is free software; you can redistribute it and/or modify it under the
    terms of the GNU General Public License as published by the Free Software
@@ -69,6 +69,8 @@ TMH_PG_insert_transfer_details (
            ",out_no_exchange"
            ",out_duplicate"
            ",out_conflict"
+           ",out_order_id"
+           ",out_merchant_pub"
            " FROM merchant_do_insert_transfer_details"
            " ($1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11,$12,$13);");
 
@@ -119,6 +121,8 @@ TMH_PG_insert_transfer_details (
       bool no_account;
       bool no_exchange;
       bool conflict;
+      char *order_id = NULL;
+      struct TALER_MerchantPublicKeyP merchant_pub;
       struct GNUNET_PQ_ResultSpec rs[] = {
         GNUNET_PQ_result_spec_bool ("out_no_instance",
                                     &no_instance),
@@ -130,6 +134,14 @@ TMH_PG_insert_transfer_details (
                                     &duplicate),
         GNUNET_PQ_result_spec_bool ("out_conflict",
                                     &conflict),
+        GNUNET_PQ_result_spec_allow_null (
+          GNUNET_PQ_result_spec_string ("out_order_id",
+                                        &order_id),
+          NULL),
+        GNUNET_PQ_result_spec_allow_null (
+          GNUNET_PQ_result_spec_auto_from_type ("out_merchant_pub",
+                                                &merchant_pub),
+          NULL),
         GNUNET_PQ_result_spec_end
       };
 
@@ -149,6 +161,26 @@ TMH_PG_insert_transfer_details (
                     qs);
         return qs;
       }
+      if (NULL != order_id)
+      {
+        struct TMH_OrderPayEventP pay_eh = {
+          .header.size = htons (sizeof (pay_eh)),
+          .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_STATUS_CHANGED),
+          .merchant_pub = merchant_pub
+        };
+
+        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                    "Notifying clients about status change of order %s\n",
+                    order_id);
+        GNUNET_CRYPTO_hash (order_id,
+                            strlen (order_id),
+                            &pay_eh.h_order_id);
+        GNUNET_PQ_event_notify (pg->conn,
+                                &pay_eh.header,
+                                NULL,
+                                0);
+        GNUNET_free (order_id);
+      }
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                   "Transfer details inserted: %s%s%s%s%s\n",
                   no_instance ? "no instance " : "",
