@@ -34,7 +34,9 @@ CREATE FUNCTION merchant_do_insert_transfer_details (
   OUT out_no_account BOOL,
   OUT out_no_exchange BOOL,
   OUT out_duplicate BOOL,
-  OUT out_conflict BOOL)
+  OUT out_conflict BOOL,
+  OUT out_order_id TEXT,
+  OUT out_merchant_pub BYTEA)
 LANGUAGE plpgsql
 AS $$
 DECLARE
@@ -60,11 +62,10 @@ DECLARE
 BEGIN
 
 -- Which instance are we using?
-SELECT merchant_serial
-  INTO my_merchant_id
+SELECT merchant_serial, merchant_pub
+  INTO my_merchant_id, out_merchant_pub
   FROM merchant_instances
  WHERE merchant_id=in_instance_id;
-
 IF NOT FOUND
 THEN
   out_no_instance=TRUE;
@@ -230,7 +231,7 @@ LOOP
             INTO my_merchant_serial, my_order_id
             FROM merchant_contract_terms
            WHERE order_serial=my_affected_orders.order_serial;
-
+          out_order_id = my_order_id;
           -- Insert pending webhook if it exists
           INSERT INTO merchant_pending_webhooks
            (merchant_serial
