@@ -37,7 +37,7 @@
 enum GNUNET_DB_QueryStatus
 TMH_PG_lookup_account (void *cls,
                        const char *instance_id,
-                       const char *payto_uri,
+                       struct TALER_FullPayto payto_uri,
                        uint64_t *account_serial);
 
 #endif
