@@ -88,8 +88,6 @@ lookup_products_cb (void *cls,
                                  &pd.description_i18n),
       GNUNET_PQ_result_spec_string ("unit",
                                     &pd.unit),
-      TALER_PQ_result_spec_amount_with_currency ("price",
-                                                 &pd.price),
       TALER_PQ_result_spec_array_amount_with_currency (pg->conn,
                                                        "price_array",
                                                        &pd.price_array_length,
@@ -124,9 +122,21 @@ lookup_products_cb (void *cls,
                                           "categories",
                                           &num_categories,
                                           &categories),
+      GNUNET_PQ_result_spec_allow_null (
+        GNUNET_PQ_result_spec_uint64 ("product_group_serial",
+                                      &pd.product_group_id),
+        NULL),
+      GNUNET_PQ_result_spec_allow_null (
+        GNUNET_PQ_result_spec_uint64 ("money_pot_serial",
+                                      &pd.money_pot_id),
+        NULL),
+      GNUNET_PQ_result_spec_bool ("price_is_net",
+                                  &pd.price_is_net),
       GNUNET_PQ_result_spec_end
     };
 
+    pd.product_group_id = 0;
+    pd.money_pot_id = 0;
     if (GNUNET_OK !=
         GNUNET_PQ_extract_result (result,
                                   rs,
@@ -136,6 +146,7 @@ lookup_products_cb (void *cls,
       plc->extract_failed = true;
       return;
     }
+    pd.image_hash = NULL;
     plc->cb (plc->cb_cls,
              product_serial,
              product_id,
@@ -175,7 +186,6 @@ TMH_PG_lookup_all_products (void *cls,
            ",description_i18n::TEXT"
            ",product_name"
            ",unit"
-           ",price"
            ",price_array"
            ",taxes::TEXT"
            ",total_stock"
@@ -193,6 +203,9 @@ TMH_PG_lookup_all_products (void *cls,
            ",product_id"
            ",product_serial"
            ",t.category_array AS categories"
+           ",product_group_serial"
+           ",money_pot_serial"
+           ",price_is_net"
            " FROM merchant_inventory minv"
            " JOIN merchant_instances inst"
            "   USING (merchant_serial)"
