@@ -48,12 +48,11 @@ TMH_PG_lookup_product (void *cls,
            ",mi.description_i18n::TEXT"
            ",mi.product_name"
            ",mi.unit"
-           ",mi.price"
            ",mi.price_array"
            ",mi.taxes::TEXT"
-            ",mi.total_stock"
-            ",mi.total_stock_frac"
-            ",mi.allow_fractional_quantity"
+           ",mi.total_stock"
+           ",mi.total_stock_frac"
+           ",mi.allow_fractional_quantity"
            ",mi.fractional_precision_level"
            ",mi.total_sold"
            ",mi.total_sold_frac"
@@ -63,6 +62,9 @@ TMH_PG_lookup_product (void *cls,
            ",mi.address::TEXT"
            ",mi.next_restock"
            ",mi.minimum_age"
+           ",mi.product_group_serial"
+           ",mi.money_pot_serial"
+           ",mi.price_is_net"
            ",t.category_array AS categories"
            " FROM merchant_inventory mi"
            " JOIN merchant_instances inst"
@@ -110,8 +112,6 @@ TMH_PG_lookup_product (void *cls,
                                     &my_name),
       GNUNET_PQ_result_spec_string ("unit",
                                     &my_unit),
-      TALER_PQ_result_spec_amount_with_currency ("price",
-                                                 &pd->price),
       TALER_PQ_result_spec_array_amount_with_currency (pg->conn,
                                                        "price_array",
                                                        &my_price_array_length,
@@ -146,11 +146,23 @@ TMH_PG_lookup_product (void *cls,
                                           "categories",
                                           num_categories,
                                           &my_categories),
+      GNUNET_PQ_result_spec_allow_null (
+        GNUNET_PQ_result_spec_uint64 ("product_group_serial",
+                                      &pd->product_group_id),
+        NULL),
+      GNUNET_PQ_result_spec_allow_null (
+        GNUNET_PQ_result_spec_uint64 ("money_pot_serial",
+                                      &pd->money_pot_id),
+        NULL),
+      GNUNET_PQ_result_spec_bool ("price_is_net",
+                                  &pd->price_is_net),
       GNUNET_PQ_result_spec_end
     };
     enum GNUNET_DB_QueryStatus qs;
 
     check_connection (pg);
+    pd->product_group_id = 0;
+    pd->money_pot_id = 0;
     qs = GNUNET_PQ_eval_prepared_singleton_select (pg->conn,
                                                    "lookup_product",
                                                    params,
@@ -161,6 +173,7 @@ TMH_PG_lookup_product (void *cls,
     pd->unit = my_unit;
     pd->taxes = my_taxes;
     pd->image = my_image;
+    pd->image_hash = NULL;
     pd->address = my_address;
     pd->price_array = my_price_array;
     pd->price_array_length = my_price_array_length;
