@@ -48,6 +48,7 @@ TMH_PG_lookup_template (void *cls,
     GNUNET_PQ_query_param_string (template_id),
     GNUNET_PQ_query_param_end
   };
+  enum GNUNET_DB_QueryStatus qs;
 
   check_connection (pg);
   PREPARE (pg,
@@ -70,10 +71,12 @@ TMH_PG_lookup_template (void *cls,
       GNUNET_PQ_result_spec_end
     };
 
-    return GNUNET_PQ_eval_prepared_singleton_select (pg->conn,
-                                                     "lookup_template",
-                                                     params,
-                                                     rs_null);
+    qs = GNUNET_PQ_eval_prepared_singleton_select (pg->conn,
+                                                   "lookup_template",
+                                                   params,
+                                                   rs_null);
+    GNUNET_PQ_cleanup_query_params_closures (params);
+    return qs;
   }
   else
   {
@@ -96,9 +99,11 @@ TMH_PG_lookup_template (void *cls,
     memset (td,
             0,
             sizeof (*td));
-    return GNUNET_PQ_eval_prepared_singleton_select (pg->conn,
-                                                     "lookup_template",
-                                                     params,
-                                                     rs);
+    qs = GNUNET_PQ_eval_prepared_singleton_select (pg->conn,
+                                                   "lookup_template",
+                                                   params,
+                                                   rs);
+    GNUNET_PQ_cleanup_query_params_closures (params);
+    return qs;
   }
 }
