@@ -98,8 +98,8 @@ TMH_PG_lookup_token_family (
              ",name"
              ",cipher_choice"
              ",description"
-             ",description_i18n"
-             ",extra_data"
+             ",description_i18n::TEXT"
+             ",extra_data::TEXT"
              ",valid_after"
              ",valid_before"
              ",duration"
