@@ -538,7 +538,7 @@ BEGIN
              AND merchant_serial = my_instance_id
              AND range = my_range;
         END IF;
-        IF (my_i < array_length(my_ranges,1))
+        IF (my_i < COALESCE(array_length(my_ranges,1),0))
         THEN
           -- carry over all events into the next slot
           UPDATE merchant_statistic_interval_counter AS usic SET
@@ -728,7 +728,7 @@ BEGIN
              AND curr = my_currency
              AND range = my_range;
         END IF;
-        IF (my_i < array_length(my_ranges,1))
+        IF (my_i < COALESCE(array_length(my_ranges,1),0))
         THEN
           -- carry over all events into the next (larger) slot
           UPDATE merchant_statistic_interval_amount AS msia SET
@@ -738,7 +738,7 @@ BEGIN
                  THEN 1
                  ELSE 0
                END,
-            cumulative_frac = cumulative_frac + my_delta_value
+            cumulative_frac = cumulative_frac + my_delta_frac
               - CASE
                  WHEN cumulative_frac + my_delta_frac > 100000000
                  THEN 100000000
@@ -988,7 +988,7 @@ BEGIN
            AND merchant_serial=my_instance
            AND delta_curr = my_currency
            AND slot >= my_time - max_slot
-           AND slot  < my_time - max_slot;
+           AND slot  < my_time - min_slot;
         -- we only proceed if we had more then one match (optimization)
         IF FOUND AND my_sum.matches > 1
         THEN
@@ -1002,11 +1002,11 @@ BEGIN
              AND merchant_serial=my_instance
              AND delta_curr = my_currency
              AND slot >= my_time - max_slot
-             AND slot  < my_time - max_slot
+             AND slot  < my_time - min_slot
              AND aevent_serial_id > my_sum.rep_serial_id;
           -- Now update the representative to the sum
           UPDATE merchant_statistic_amount_event SET
-             delta_value = my_total_value
+             delta_value = my_total_val
             ,delta_frac = my_total_frac
            WHERE imeta_serial_id = my_meta
              AND merchant_serial = my_instance
@@ -1057,12 +1057,12 @@ BEGIN
       DELETE
         FROM merchant_statistic_bucket_amount
        WHERE bmeta_serial_id = my_rec.bmeta_serial_id
-         AND bucket_start >= my_end;
+         AND bucket_start < my_end;
     ELSE
       DELETE
         FROM merchant_statistic_bucket_counter
        WHERE bmeta_serial_id = my_rec.bmeta_serial_id
-         AND bucket_start >= my_end;
+         AND bucket_start < my_end;
     END IF;
   END LOOP;
 END $$;
