@@ -39,35 +39,42 @@ TMH_PG_update_product (void *cls,
                        bool *no_product,
                        bool *lost_reduced,
                        bool *sold_reduced,
-                       bool *stocked_reduced)
+                       bool *stocked_reduced,
+                       bool *no_group,
+                       bool *no_pot)
 {
   struct PostgresClosure *pg = cls;
   struct GNUNET_PQ_QueryParam params[] = {
     GNUNET_PQ_query_param_string (instance_id), /* $1 */
     GNUNET_PQ_query_param_string (product_id),
     GNUNET_PQ_query_param_string (pd->description),
-    TALER_PQ_query_param_json (pd->description_i18n),
+    TALER_PQ_query_param_json (pd->description_i18n), /* $4 */
     GNUNET_PQ_query_param_string (pd->unit),
     GNUNET_PQ_query_param_string (pd->image), /* $6 */
-    TALER_PQ_query_param_json (pd->taxes),
-    TALER_PQ_query_param_amount_with_currency (pg->conn,
-                                               &pd->price), /* $8 */
+    TALER_PQ_query_param_json (pd->taxes), /* $7 */
     TALER_PQ_query_param_array_amount_with_currency (
       pd->price_array_length,
       pd->price_array,
-      pg->conn), /* $9 */
-    GNUNET_PQ_query_param_uint64 (&pd->total_stock),  /* $10 */
-    GNUNET_PQ_query_param_uint32 (&pd->total_stock_frac), /* $11 */
-    GNUNET_PQ_query_param_bool (pd->allow_fractional_quantity), /* $12 */
+      pg->conn), /* $8 */
+    GNUNET_PQ_query_param_uint64 (&pd->total_stock),  /* $9 */
+    GNUNET_PQ_query_param_uint32 (&pd->total_stock_frac), /* $10 */
+    GNUNET_PQ_query_param_bool (pd->allow_fractional_quantity), /* $11 */
     GNUNET_PQ_query_param_uint32 (&pd->fractional_precision_level),
     GNUNET_PQ_query_param_uint64 (&pd->total_lost),
-    TALER_PQ_query_param_json (pd->address),
+    TALER_PQ_query_param_json (pd->address), /* $14 */
     GNUNET_PQ_query_param_timestamp (&pd->next_restock),
     GNUNET_PQ_query_param_uint32 (&pd->minimum_age),
     GNUNET_PQ_query_param_array_uint64 (num_cats,
                                         cats,
-                                        pg->conn),
-    GNUNET_PQ_query_param_string (pd->product_name),
+                                        pg->conn), /* $17 */
+    GNUNET_PQ_query_param_string (pd->product_name), /* $18 */
+    (0 == pd->product_group_id)
+    ? GNUNET_PQ_query_param_null ()
+    : GNUNET_PQ_query_param_uint64 (&pd->product_group_id),
+    (0 == pd->money_pot_id)
+    ? GNUNET_PQ_query_param_null ()
+    : GNUNET_PQ_query_param_uint64 (&pd->money_pot_id), /* $20 */
+    GNUNET_PQ_query_param_bool (pd->price_is_net),
     GNUNET_PQ_query_param_end
   };
   uint64_t ncat;
@@ -87,6 +94,10 @@ TMH_PG_update_product (void *cls,
       GNUNET_PQ_result_spec_uint64 ("no_cat",
                                     &ncat),
       &cats_found),
+    GNUNET_PQ_result_spec_bool ("no_group",
+                                no_group),
+    GNUNET_PQ_result_spec_bool ("no_pot",
+                                no_pot),
     GNUNET_PQ_result_spec_end
   };
   enum GNUNET_DB_QueryStatus qs;
@@ -108,9 +119,12 @@ TMH_PG_update_product (void *cls,
            ",out_no_product AS no_product"
            ",out_no_cat AS no_cat"
            ",out_no_instance AS no_instance"
+           ",out_no_group AS no_group"
+           ",out_no_pot AS no_pot"
            " FROM merchant_do_update_product"
            "($1,$2,$3,$4::TEXT::JSONB,$5,$6,$7::TEXT::JSONB,$8,$9"
-           ",$10,$11,$12,$13,$14,$15::TEXT::JSONB,$16,$17,$18,$19);");
+           ",$10,$11,$12,$13,$14::TEXT::JSONB,$15,$16,$17,$18"
+           ",$19,$20,$21);");
   qs = GNUNET_PQ_eval_prepared_singleton_select (pg->conn,
                                                  "update_product",
                                                  params,
