@@ -48,6 +48,8 @@
  *             exceed total_stock minus the existing total_sold;
  *             total_sold and total_stock must be larger or equal to
  *             the existing value;
+ * @param[out] no_group set to true if the product group in @a pd is unknown
+ * @param[out] no_pot set to true if the money pot in @a pd is unknown
  * @return database result code
  */
 enum GNUNET_DB_QueryStatus
@@ -62,6 +64,8 @@ TMH_PG_update_product (void *cls,
                        bool *no_product,
                        bool *lost_reduced,
                        bool *sold_reduced,
-                       bool *stocked_reduced);
+                       bool *stocked_reduced,
+                       bool *no_group,
+                       bool *no_pot);
 
 #endif
