@@ -20,28 +20,32 @@ CREATE FUNCTION merchant_do_update_product (
   IN in_instance_id TEXT,
   IN in_product_id TEXT,
   IN in_description TEXT,
-  IN in_description_i18n JSONB,
+  IN in_description_i18n JSONB, -- $4
   IN in_unit TEXT,
   IN in_image TEXT,
-  IN in_taxes JSONB,
-  IN in_price taler_amount_currency,
+  IN in_taxes JSONB, -- $7
   IN ina_price_list taler_amount_currency[],
   IN in_total_stock INT8,
   IN in_total_stock_frac INT4,
   IN in_allow_fractional_quantity BOOL,
   IN in_fractional_precision_level INT4,
-  IN in_total_lost INT8,
-  IN in_address JSONB,
+  IN in_total_lost INT8, -- NOTE: not in insert_product
+  IN in_address JSONB, -- $14
   IN in_next_restock INT8,
   IN in_minimum_age INT4,
-  IN ina_categories INT8[],
+  IN ina_categories INT8[], -- $17
   IN in_product_name TEXT,
+  IN in_product_group_id INT8, -- NULL for default
+  IN in_money_pot_id INT8, -- NULL for none
+  IN in_price_is_net BOOL, -- $21
   OUT out_no_instance BOOL,
   OUT out_no_product BOOL,
   OUT out_lost_reduced BOOL,
   OUT out_sold_reduced BOOL,
   OUT out_stocked_reduced BOOL,
-  OUT out_no_cat INT8)
+  OUT out_no_cat INT8,
+  OUT out_no_group BOOL,
+  OUT out_no_pot BOOL)
 LANGUAGE plpgsql
 AS $$
 DECLARE
@@ -50,10 +54,10 @@ DECLARE
   i INT8;
   ini_cat INT8;
   rec RECORD;
-  my_price taler_amount_currency;
-  my_price_array taler_amount_currency[];
 BEGIN
 
+out_no_group = FALSE;
+out_no_pot = FALSE;
 out_no_instance=FALSE;
 out_no_product=FALSE;
 out_lost_reduced=FALSE;
@@ -73,6 +77,30 @@ THEN
   RETURN;
 END IF;
 
+IF in_product_group_id IS NOT NULL
+THEN
+  PERFORM FROM merchant_product_groups
+         WHERE product_group_serial=in_product_group_id
+           AND merchant_serial=my_merchant_id;
+  IF NOT FOUND
+  THEN
+    out_no_group=TRUE;
+    RETURN;
+  END IF;
+END IF;
+
+IF in_money_pot_id IS NOT NULL
+THEN
+  PERFORM FROM merchant_money_pots
+         WHERE money_pot_serial=in_money_pot_id
+           AND merchant_serial=my_merchant_id;
+  IF NOT FOUND
+  THEN
+    out_no_pot=TRUE;
+    RETURN;
+  END IF;
+END IF;
+
 -- Check existing entry satisfies constraints
 SELECT total_stock
       ,total_stock_frac
@@ -92,14 +120,6 @@ END IF;
 
 my_product_serial = rec.product_serial;
 
-IF COALESCE (array_length(ina_price_list,1),0) = 0
-THEN
-  my_price_array := ARRAY[in_price]::taler_amount_currency[];
-ELSE
-  my_price_array := ina_price_list;
-END IF;
-my_price := my_price_array[1];
-
 IF rec.total_stock > in_total_stock
 THEN
   out_stocked_reduced=TRUE;
@@ -158,8 +178,7 @@ UPDATE merchant_inventory SET
                            'hex')
              END
   ,taxes=in_taxes
-  ,price=my_price
-  ,price_array=my_price_array
+  ,price_array=ina_price_list
   ,total_stock=in_total_stock
   ,total_stock_frac=in_total_stock_frac
   ,allow_fractional_quantity=in_allow_fractional_quantity
@@ -168,6 +187,9 @@ UPDATE merchant_inventory SET
   ,address=in_address
   ,next_restock=in_next_restock
   ,minimum_age=in_minimum_age
+  ,product_group_serial=in_product_group_id
+  ,money_pot_serial=in_money_pot_id
+  ,price_is_net=in_price_is_net
  WHERE merchant_serial=my_merchant_id
    AND product_serial=my_product_serial; -- could also match on product_id
 
