@@ -33,6 +33,7 @@
 #include "pg_helper.h"
 #include "pg_gc.h"
 #include "pg_insert_otp.h"
+#include "pg_account_kyc_get_outdated.h"
 #include "pg_get_kyc_status.h"
 #include "pg_get_kyc_limits.h"
 #include "pg_delete_otp.h"
@@ -50,7 +51,9 @@
 #include "pg_lookup_transfers.h"
 #include "pg_lookup_pending_deposits.h"
 #include "pg_lookup_categories.h"
+#include "pg_lookup_categories_by_ids.h"
 #include "pg_lookup_units.h"
+#include "pg_lookup_custom_units_by_names.h"
 #include "pg_select_category.h"
 #include "pg_update_category.h"
 #include "pg_insert_category.h"
@@ -59,6 +62,24 @@
 #include "pg_insert_unit.h"
 #include "pg_update_unit.h"
 #include "pg_delete_unit.h"
+#include "pg_insert_report.h"
+#include "pg_delete_report.h"
+#include "pg_update_report.h"
+#include "pg_check_report.h"
+#include "pg_select_reports.h"
+#include "pg_lookup_reports_pending.h"
+#include "pg_update_report_status.h"
+#include "pg_select_report.h"
+#include "pg_insert_product_group.h"
+#include "pg_delete_product_group.h"
+#include "pg_update_product_group.h"
+#include "pg_select_product_groups.h"
+#include "pg_insert_money_pot.h"
+#include "pg_delete_money_pot.h"
+#include "pg_update_money_pot.h"
+#include "pg_check_money_pots.h"
+#include "pg_select_money_pots.h"
+#include "pg_select_money_pot.h"
 #include "pg_update_wirewatch_progress.h"
 #include "pg_select_wirewatch_accounts.h"
 #include "pg_select_open_transfers.h"
@@ -85,8 +106,12 @@
 #include "pg_activate_account.h"
 #include "pg_lookup_products.h"
 #include "pg_lookup_all_products.h"
+#include "pg_lookup_inventory_products.h"
+#include "pg_lookup_inventory_products_filtered.h"
 #include "pg_lookup_product.h"
 #include "pg_lookup_product_image.h"
+#include "pg_lookup_statistics_amount_by_bucket2.h"
+#include "pg_lookup_statistics_counter_by_bucket2.h"
 #include "pg_delete_product.h"
 #include "pg_insert_product.h"
 #include "pg_update_product.h"
@@ -99,6 +124,7 @@
 #include "pg_lookup_orders.h"
 #include "pg_insert_order.h"
 #include "pg_insert_order_blinded_sigs.h"
+#include "pg_increment_money_pots.h"
 #include "pg_unlock_inventory.h"
 #include "pg_insert_order_lock.h"
 #include "pg_select_order_blinded_sigs.h"
@@ -468,6 +494,10 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_lookup_products;
   plugin->lookup_all_products
     = &TMH_PG_lookup_all_products;
+  plugin->lookup_inventory_products
+    = &TMH_PG_lookup_inventory_products;
+  plugin->lookup_inventory_products_filtered
+    = &TMH_PG_lookup_inventory_products_filtered;
   plugin->lookup_product
     = &TMH_PG_lookup_product;
   plugin->lookup_product_image_by_hash
@@ -628,8 +658,12 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_update_pending_webhook;
   plugin->lookup_categories
     = &TMH_PG_lookup_categories;
+  plugin->lookup_categories_by_ids
+    = &TMH_PG_lookup_categories_by_ids;
   plugin->lookup_units
     = &TMH_PG_lookup_units;
+  plugin->lookup_custom_units_by_names
+    = &TMH_PG_lookup_custom_units_by_names;
   plugin->select_category_by_name
     = &TMH_PG_select_category_by_name;
   plugin->get_kyc_status
@@ -672,6 +706,48 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_delete_token_family;
   plugin->update_token_family
     = &TMH_PG_update_token_family;
+  plugin->insert_report
+    = &TMH_PG_insert_report;
+  plugin->delete_report
+    = &TMH_PG_delete_report;
+  plugin->update_report
+    = &TMH_PG_update_report;
+  plugin->check_report
+    = &TMH_PG_check_report;
+  plugin->select_reports
+    = &TMH_PG_select_reports;
+  plugin->lookup_reports_pending
+    = &TMH_PG_lookup_reports_pending;
+  plugin->update_report_status
+    = &TMH_PG_update_report_status;
+  plugin->lookup_statistics_amount_by_bucket2
+    = &TMH_PG_lookup_statistics_amount_by_bucket2;
+  plugin->lookup_statistics_counter_by_bucket2
+    = &TMH_PG_lookup_statistics_counter_by_bucket2;
+  plugin->account_kyc_get_outdated
+    = &TMH_PG_account_kyc_get_outdated;
+  plugin->select_report
+    = &TMH_PG_select_report;
+  plugin->insert_product_group
+    = &TMH_PG_insert_product_group;
+  plugin->delete_product_group
+    = &TMH_PG_delete_product_group;
+  plugin->update_product_group
+    = &TMH_PG_update_product_group;
+  plugin->select_product_groups
+    = &TMH_PG_select_product_groups;
+  plugin->insert_money_pot
+    = &TMH_PG_insert_money_pot;
+  plugin->delete_money_pot
+    = &TMH_PG_delete_money_pot;
+  plugin->update_money_pot
+    = &TMH_PG_update_money_pot;
+  plugin->check_money_pots
+    = &TMH_PG_check_money_pots;
+  plugin->select_money_pots
+    = &TMH_PG_select_money_pots;
+  plugin->select_money_pot
+    = &TMH_PG_select_money_pot;
   plugin->insert_token_family_key
     = &TMH_PG_insert_token_family_key;
   plugin->update_contract_session
@@ -725,6 +801,8 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_lookup_donau_keys;
   plugin->lookup_order_charity
     = &TMH_PG_lookup_order_charity;
+  plugin->increment_money_pots
+    = &TMH_PG_increment_money_pots;
   plugin->upsert_donau_keys
     = &TMH_PG_upsert_donau_keys;
   plugin->update_donau_instance
