@@ -17,10 +17,8 @@ DECLARE
   my_signkey_serial INT8;
   my_account_serial INT8;
   my_decose INT8;
-  my_order_serial INT8;
   my_expected_credit_serial INT8;
   my_wire_pending_cleared BOOL;
-  my_order_id TEXT;
 BEGIN
   out_dummy=FALSE;
 SELECT signkey_serial
@@ -105,24 +103,28 @@ CREATE FUNCTION merchant_do_insert_product (
   IN in_instance_id TEXT,
   IN in_product_id TEXT,
   IN in_description TEXT,
-  IN in_description_i18n JSONB,
+  IN in_description_i18n JSONB, 
   IN in_unit TEXT,
   IN in_image TEXT,
-  IN in_taxes JSONB,
-  IN in_price taler_amount_currency,
+  IN in_taxes JSONB, 
   IN ina_price_list taler_amount_currency[],
-  IN in_total_stock INT8,
-  IN in_total_stock_frac INT4,
+  IN in_total_stock INT8, 
+  IN in_total_stock_frac INT4, 
   IN in_allow_fractional_quantity BOOL,
   IN in_fractional_precision_level INT4,
-  IN in_address JSONB,
+  IN in_address JSONB, 
   IN in_next_restock INT8,
   IN in_minimum_age INT4,
-  IN ina_categories INT8[],
+  IN ina_categories INT8[], 
   IN in_product_name TEXT,
+  IN in_product_group_id INT8, 
+  IN in_money_pot_id INT8, 
+  IN in_price_is_net BOOL, 
   OUT out_no_instance BOOL,
   OUT out_conflict BOOL,
-  OUT out_no_cat INT8)
+  OUT out_no_cat INT8,
+  OUT out_no_group BOOL,
+  OUT out_no_pot BOOL)
 LANGUAGE plpgsql
 AS $$
 DECLARE
@@ -130,9 +132,9 @@ DECLARE
   my_product_serial INT8;
   i INT8;
   ini_cat INT8;
-  my_price taler_amount_currency;
-  my_price_array taler_amount_currency[];
 BEGIN
+out_no_group = FALSE;
+out_no_pot = FALSE;
 SELECT merchant_serial
   INTO my_merchant_id
   FROM merchant_instances
@@ -145,13 +147,32 @@ THEN
   RETURN;
 END IF;
 out_no_instance=FALSE;
-IF COALESCE (array_length(ina_price_list,1),0) = 0
+IF in_product_group_id IS NOT NULL
 THEN
-  my_price_array := ARRAY[in_price]::taler_amount_currency[];
-ELSE
-  my_price_array := ina_price_list;
+  PERFORM FROM merchant_product_groups
+         WHERE product_group_serial=in_product_group_id
+           AND merchant_serial=my_merchant_id;
+  IF NOT FOUND
+  THEN
+    out_no_group=TRUE;
+    out_conflict=FALSE;
+    out_no_cat=NULL;
+    RETURN;
+  END IF;
+END IF;
+IF in_money_pot_id IS NOT NULL
+THEN
+  PERFORM FROM merchant_money_pots
+         WHERE money_pot_serial=in_money_pot_id
+           AND merchant_serial=my_merchant_id;
+  IF NOT FOUND
+  THEN
+    out_no_pot=TRUE;
+    out_conflict=FALSE;
+    out_no_cat=NULL;
+    RETURN;
+  END IF;
 END IF;
-my_price := my_price_array[1];
 INSERT INTO merchant_inventory
  (merchant_serial
  ,product_id
@@ -162,7 +183,6 @@ INSERT INTO merchant_inventory
  ,image
  ,image_hash
  ,taxes
- ,price
  ,price_array
  ,total_stock
  ,total_stock_frac
@@ -170,7 +190,10 @@ INSERT INTO merchant_inventory
  ,fractional_precision_level
  ,address
  ,next_restock
-  ,minimum_age
+ ,minimum_age
+ ,product_group_serial
+ ,money_pot_serial
+ ,price_is_net
 ) VALUES (
   my_merchant_id
  ,in_product_id
@@ -187,15 +210,18 @@ INSERT INTO merchant_inventory
                 'hex')
   END
  ,in_taxes
- ,my_price
- ,my_price_array
+ ,ina_price_list
  ,in_total_stock
  ,in_total_stock_frac
  ,in_allow_fractional_quantity
  ,in_fractional_precision_level
  ,in_address
  ,in_next_restock
- ,in_minimum_age)
+ ,in_minimum_age
+ ,in_product_group_id
+ ,in_money_pot_id
+ ,in_price_is_net
+ )
 ON CONFLICT (merchant_serial, product_id) DO NOTHING
  RETURNING product_serial
  INTO my_product_serial;
@@ -212,16 +238,18 @@ THEN
     AND unit=in_unit
     AND image=in_image
     AND taxes=in_taxes
-    AND price=my_price
     AND to_jsonb(COALESCE(price_array, ARRAY[]::taler_amount_currency[]))
-        = to_jsonb(COALESCE(my_price_array, ARRAY[]::taler_amount_currency[]))
+        = to_jsonb(COALESCE(ina_price_list, ARRAY[]::taler_amount_currency[])) 
     AND total_stock=in_total_stock
     AND total_stock_frac=in_total_stock_frac
     AND allow_fractional_quantity=in_allow_fractional_quantity
     AND fractional_precision_level=in_fractional_precision_level
     AND address=in_address
     AND next_restock=in_next_restock
-    AND minimum_age=in_minimum_age;
+    AND minimum_age=in_minimum_age
+    AND product_group_serial IS NOT DISTINCT FROM in_product_group_id
+    AND money_pot_serial IS NOT DISTINCT FROM in_money_pot_id
+    AND price_is_net=in_price_is_net;
   IF NOT FOUND
   THEN
     out_conflict=TRUE;
@@ -246,7 +274,7 @@ THEN
     INTO i
     FROM merchant_product_categories
     WHERE product_serial=my_product_serial;
-  IF i != array_length(ina_categories,1)
+  IF i != COALESCE(array_length(ina_categories,1),0)
   THEN
     out_conflict=TRUE;
     out_no_cat=NULL;
@@ -400,7 +428,9 @@ CREATE FUNCTION merchant_do_insert_transfer_details (
   OUT out_no_account BOOL,
   OUT out_no_exchange BOOL,
   OUT out_duplicate BOOL,
-  OUT out_conflict BOOL)
+  OUT out_conflict BOOL,
+  OUT out_order_id TEXT,
+  OUT out_merchant_pub BYTEA)
 LANGUAGE plpgsql
 AS $$
 DECLARE
@@ -424,8 +454,8 @@ DECLARE
   ini_coin_value taler_amount_currency;
   ini_deposit_fee taler_amount_currency;
 BEGIN
-SELECT merchant_serial
-  INTO my_merchant_id
+SELECT merchant_serial, merchant_pub
+  INTO my_merchant_id, out_merchant_pub
   FROM merchant_instances
  WHERE merchant_id=in_instance_id;
 IF NOT FOUND
@@ -566,6 +596,7 @@ LOOP
             INTO my_merchant_serial, my_order_id
             FROM merchant_contract_terms
            WHERE order_serial=my_affected_orders.order_serial;
+          out_order_id = my_order_id;
           INSERT INTO merchant_pending_webhooks
            (merchant_serial
            ,webhook_serial
@@ -595,28 +626,32 @@ CREATE FUNCTION merchant_do_update_product (
   IN in_instance_id TEXT,
   IN in_product_id TEXT,
   IN in_description TEXT,
-  IN in_description_i18n JSONB,
+  IN in_description_i18n JSONB, 
   IN in_unit TEXT,
   IN in_image TEXT,
-  IN in_taxes JSONB,
-  IN in_price taler_amount_currency,
+  IN in_taxes JSONB, 
   IN ina_price_list taler_amount_currency[],
   IN in_total_stock INT8,
   IN in_total_stock_frac INT4,
   IN in_allow_fractional_quantity BOOL,
   IN in_fractional_precision_level INT4,
-  IN in_total_lost INT8,
-  IN in_address JSONB,
+  IN in_total_lost INT8, 
+  IN in_address JSONB, 
   IN in_next_restock INT8,
   IN in_minimum_age INT4,
-  IN ina_categories INT8[],
+  IN ina_categories INT8[], 
   IN in_product_name TEXT,
+  IN in_product_group_id INT8, 
+  IN in_money_pot_id INT8, 
+  IN in_price_is_net BOOL, 
   OUT out_no_instance BOOL,
   OUT out_no_product BOOL,
   OUT out_lost_reduced BOOL,
   OUT out_sold_reduced BOOL,
   OUT out_stocked_reduced BOOL,
-  OUT out_no_cat INT8)
+  OUT out_no_cat INT8,
+  OUT out_no_group BOOL,
+  OUT out_no_pot BOOL)
 LANGUAGE plpgsql
 AS $$
 DECLARE
@@ -625,9 +660,9 @@ DECLARE
   i INT8;
   ini_cat INT8;
   rec RECORD;
-  my_price taler_amount_currency;
-  my_price_array taler_amount_currency[];
 BEGIN
+out_no_group = FALSE;
+out_no_pot = FALSE;
 out_no_instance=FALSE;
 out_no_product=FALSE;
 out_lost_reduced=FALSE;
@@ -643,6 +678,28 @@ THEN
   out_no_instance=TRUE;
   RETURN;
 END IF;
+IF in_product_group_id IS NOT NULL
+THEN
+  PERFORM FROM merchant_product_groups
+         WHERE product_group_serial=in_product_group_id
+           AND merchant_serial=my_merchant_id;
+  IF NOT FOUND
+  THEN
+    out_no_group=TRUE;
+    RETURN;
+  END IF;
+END IF;
+IF in_money_pot_id IS NOT NULL
+THEN
+  PERFORM FROM merchant_money_pots
+         WHERE money_pot_serial=in_money_pot_id
+           AND merchant_serial=my_merchant_id;
+  IF NOT FOUND
+  THEN
+    out_no_pot=TRUE;
+    RETURN;
+  END IF;
+END IF;
 SELECT total_stock
       ,total_stock_frac
       ,total_lost
@@ -658,13 +715,6 @@ THEN
   RETURN;
 END IF;
 my_product_serial = rec.product_serial;
-IF COALESCE (array_length(ina_price_list,1),0) = 0
-THEN
-  my_price_array := ARRAY[in_price]::taler_amount_currency[];
-ELSE
-  my_price_array := ina_price_list;
-END IF;
-my_price := my_price_array[1];
 IF rec.total_stock > in_total_stock
 THEN
   out_stocked_reduced=TRUE;
@@ -715,8 +765,7 @@ UPDATE merchant_inventory SET
                            'hex')
              END
   ,taxes=in_taxes
-  ,price=my_price
-  ,price_array=my_price_array
+  ,price_array=ina_price_list
   ,total_stock=in_total_stock
   ,total_stock_frac=in_total_stock_frac
   ,allow_fractional_quantity=in_allow_fractional_quantity
@@ -725,6 +774,9 @@ UPDATE merchant_inventory SET
   ,address=in_address
   ,next_restock=in_next_restock
   ,minimum_age=in_minimum_age
+  ,product_group_serial=in_product_group_id
+  ,money_pot_serial=in_money_pot_id
+  ,price_is_net=in_price_is_net
  WHERE merchant_serial=my_merchant_id
    AND product_serial=my_product_serial; 
 ASSERT FOUND,'SELECTED it earlier, should UPDATE it now';
@@ -802,6 +854,8 @@ CREATE FUNCTION merchant_do_account_kyc_set_status (
   IN in_notify_str TEXT,
   IN in_notify2_str TEXT,
   IN in_rule_gen INT8,
+  IN in_next_time INT8,
+  IN in_kyc_backoff INT8,
   OUT out_no_instance BOOL,
   OUT out_no_account BOOL)
 LANGUAGE plpgsql
@@ -840,6 +894,8 @@ UPDATE merchant_kyc
       ,exchange_ec_code=in_exchange_ec_code
       ,access_token=in_access_token
       ,last_rule_gen=in_rule_gen
+      ,next_kyc_poll=in_next_time
+      ,kyc_backoff=in_kyc_backoff
  WHERE account_serial=my_account_serial
    AND exchange_url=in_exchange_url;
 IF NOT FOUND
@@ -854,7 +910,9 @@ THEN
     ,exchange_http_status
     ,exchange_ec_code
     ,access_token
-    ,last_rule_gen)
+    ,last_rule_gen
+    ,next_kyc_poll
+    ,kyc_backoff)
   VALUES
     (in_timestamp
     ,in_kyc_ok
@@ -865,7 +923,9 @@ THEN
     ,in_exchange_http_status
     ,in_exchange_ec_code
     ,in_access_token
-    ,in_rule_gen);
+    ,in_rule_gen
+    ,in_next_time
+    ,in_kyc_backoff);
 END IF;
 EXECUTE FORMAT (
    'NOTIFY %s'
@@ -1428,7 +1488,7 @@ BEGIN
              AND merchant_serial = my_instance_id
              AND range = my_range;
         END IF;
-        IF (my_i < array_length(my_ranges,1))
+        IF (my_i < COALESCE(array_length(my_ranges,1),0))
         THEN
           UPDATE merchant_statistic_interval_counter AS usic SET
             cumulative_number = cumulative_number + my_delta,
@@ -1598,7 +1658,7 @@ BEGIN
              AND curr = my_currency
              AND range = my_range;
         END IF;
-        IF (my_i < array_length(my_ranges,1))
+        IF (my_i < COALESCE(array_length(my_ranges,1),0))
         THEN
           UPDATE merchant_statistic_interval_amount AS msia SET
             cumulative_value = cumulative_value + my_delta_value
@@ -1607,7 +1667,7 @@ BEGIN
                  THEN 1
                  ELSE 0
                END,
-            cumulative_frac = cumulative_frac + my_delta_value
+            cumulative_frac = cumulative_frac + my_delta_frac
               - CASE
                  WHEN cumulative_frac + my_delta_frac > 100000000
                  THEN 100000000
@@ -1821,7 +1881,7 @@ BEGIN
            AND merchant_serial=my_instance
            AND delta_curr = my_currency
            AND slot >= my_time - max_slot
-           AND slot < my_time - max_slot;
+           AND slot < my_time - min_slot;
         IF FOUND AND my_sum.matches > 1
         THEN
           my_total_frac = my_sum.total_frac % 100000000;
@@ -1831,10 +1891,10 @@ BEGIN
              AND merchant_serial=my_instance
              AND delta_curr = my_currency
              AND slot >= my_time - max_slot
-             AND slot < my_time - max_slot
+             AND slot < my_time - min_slot
              AND aevent_serial_id > my_sum.rep_serial_id;
           UPDATE merchant_statistic_amount_event SET
-             delta_value = my_total_value
+             delta_value = my_total_val
             ,delta_frac = my_total_frac
            WHERE imeta_serial_id = my_meta
              AND merchant_serial = my_instance
@@ -1880,12 +1940,12 @@ BEGIN
       DELETE
         FROM merchant_statistic_bucket_amount
        WHERE bmeta_serial_id = my_rec.bmeta_serial_id
-         AND bucket_start >= my_end;
+         AND bucket_start < my_end;
     ELSE
       DELETE
         FROM merchant_statistic_bucket_counter
        WHERE bmeta_serial_id = my_rec.bmeta_serial_id
-         AND bucket_start >= my_end;
+         AND bucket_start < my_end;
     END IF;
   END LOOP;
 END $$;
@@ -2283,6 +2343,266 @@ END;
 $$ LANGUAGE plpgsql;
 COMMENT ON FUNCTION handle_category_changes
   IS 'Trigger function to handle pending webhooks for category changes';
+SET search_path TO merchant;
+DROP FUNCTION IF EXISTS merchant_do_update_product_group;
+CREATE FUNCTION merchant_do_update_product_group (
+  IN in_instance_id TEXT,
+  IN in_product_group_serial TEXT,
+  IN in_name TEXT,
+  IN in_description TEXT,
+  OUT out_conflict BOOL,
+  OUT out_not_found BOOL)
+LANGUAGE plpgsql
+AS $$
+DECLARE
+  my_merchant_id INT8;
+BEGIN
+SELECT merchant_serial
+  INTO my_merchant_id
+  FROM merchant_instances
+  WHERE merchant_id=in_instance_id;
+BEGIN
+  UPDATE merchant_product_groups SET
+    product_group_name=in_name
+   ,product_group_description=in_description
+   WHERE merchant_serial=my_merchant_id
+     AND product_group_serial=in_product_group_serial;
+  out_not_found = NOT FOUND;
+  out_conflict = FALSE;
+  RETURN;
+EXCEPTION
+  WHEN unique_violation
+  THEN
+    out_conflict = TRUE;
+    RETURN;
+END;
+END $$;
+DROP FUNCTION IF EXISTS merchant_do_update_money_pot;
+CREATE FUNCTION merchant_do_update_money_pot (
+  IN in_instance_id TEXT,
+  IN in_money_pot_serial INT8,
+  IN in_name TEXT,
+  IN in_description TEXT,
+  IN in_old_totals taler_amount_currency[], 
+  IN in_new_totals taler_amount_currency[], 
+  OUT out_conflict_total BOOL,
+  OUT out_conflict_name BOOL,
+  OUT out_not_found BOOL)
+LANGUAGE plpgsql
+AS $$
+DECLARE
+  my_merchant_id INT8;
+BEGIN
+SELECT merchant_serial
+  INTO my_merchant_id
+  FROM merchant_instances
+  WHERE merchant_id=in_instance_id;
+IF NOT FOUND
+THEN
+  out_conflict_total = FALSE;
+  out_conflict_name = FALSE;
+  out_not_found = TRUE;
+  RETURN;
+END IF;
+BEGIN
+  UPDATE merchant_money_pots SET
+    money_pot_name=in_name
+   ,money_pot_description=in_description
+   ,pot_totals=COALESCE(in_new_totals, pot_totals)
+   WHERE merchant_serial=my_merchant_id
+     AND money_pot_serial=in_money_pot_serial
+     AND ( (in_old_totals IS NULL) OR (pot_totals=in_old_totals) );
+  IF NOT FOUND
+  THEN
+    PERFORM FROM merchant_money_pots
+           WHERE merchant_serial=my_merchant_id
+             AND money_pot_serial=in_money_pot_serial;
+    out_conflict_total = FOUND;
+    out_not_found = NOT FOUND;
+    out_conflict_name = FALSE;
+  ELSE
+    out_conflict_total = FALSE;
+    out_not_found = FALSE;
+    out_conflict_name = FALSE;
+  END IF;
+  RETURN;
+EXCEPTION
+  WHEN unique_violation
+  THEN
+    out_conflict_name = TRUE;
+    out_conflict_total = FALSE;
+    out_not_found = FALSE;
+    RETURN;
+END;
+END $$;
+SET search_path TO merchant;
+DROP FUNCTION IF EXISTS merchant_do_increment_money_pots;
+CREATE FUNCTION merchant_do_increment_money_pots (
+  IN in_instance_id TEXT,
+  IN ina_money_pots_ids INT8[],
+  IN ina_increments taler_amount_currency[],
+  OUT out_not_found BOOL)
+LANGUAGE plpgsql
+AS $$
+DECLARE
+  my_merchant_id INT8;
+  i INT;
+  ini_current_pot_id INT8;
+  ini_current_increment taler_amount_currency;
+  my_totals taler_amount_currency[];
+  currency_found BOOL;
+  j INT;
+BEGIN
+SELECT merchant_serial
+  INTO my_merchant_id
+  FROM merchant_instances
+  WHERE merchant_id=in_instance_id;
+IF NOT FOUND
+THEN
+  out_not_found = TRUE;
+  RETURN;
+END IF;
+out_not_found = FALSE;
+IF ( COALESCE(array_length(ina_money_pots_ids, 1), 0) !=
+     COALESCE(array_length(ina_increments, 1), 0) )
+THEN
+  RAISE EXCEPTION 'Array lengths must match';
+END IF;
+FOR i IN 1..COALESCE(array_length(ina_money_pots_ids, 1),0)
+  LOOP
+    ini_current_pot_id = ina_money_pots_ids[i];
+    ini_current_increment = ina_increments[i];
+    SELECT pot_totals
+      INTO my_totals
+      FROM merchant_money_pots
+      WHERE money_pot_serial = ini_current_pot_id
+        AND merchant_serial = my_merchant_id;
+    IF NOT FOUND
+    THEN
+      out_not_found = TRUE;
+    ELSE
+      currency_found = FALSE;
+      FOR j IN 1..COALESCE(array_length(my_totals, 1), 0)
+      LOOP
+        IF (my_totals[j]).currency = (ini_current_increment).currency
+        THEN
+          my_totals[j].frac
+            = my_totals[j].frac + ini_current_increment.frac;
+          my_totals[j].val
+            = my_totals[j].val + ini_current_increment.val;
+          IF my_totals[j].frac >= 100000000
+          THEN
+            my_totals[j].frac = my_totals[j].frac - 100000000;
+            my_totals[j].val = my_totals[j].val + 1;
+          END IF;
+          currency_found = TRUE;
+          EXIT; 
+        END IF;
+      END LOOP;
+      IF NOT currency_found
+      THEN
+        my_totals = array_append(my_totals, ini_current_increment);
+      END IF;
+      UPDATE merchant_money_pots
+        SET pot_totals = my_totals
+        WHERE money_pot_serial = ini_current_pot_id
+          AND merchant_serial = my_merchant_id;
+    END IF;
+  END LOOP;
+END $$;
+DROP FUNCTION IF EXISTS merchant_do_account_kyc_get_status;
+CREATE FUNCTION merchant_do_account_kyc_get_status (
+  IN in_merchant_id TEXT,
+  IN in_now INT8,
+  IN in_exchange_url TEXT, 
+  IN in_h_wire BYTEA 
+) RETURNS TABLE (
+  out_h_wire BYTEA, 
+  out_payto_uri TEXT, 
+  out_exchange_url TEXT,
+  out_kyc_timestamp INT8,
+  out_kyc_ok BOOLEAN,
+  out_access_token BYTEA,
+  out_exchange_http_status INT4,
+  out_exchange_ec_code INT4,
+  out_aml_review BOOLEAN,
+  out_jaccount_limits TEXT
+)
+LANGUAGE plpgsql
+AS $$
+DECLARE
+  my_merchant_serial INT8;
+  my_account_serial INT8;
+  my_h_wire BYTEA;
+  my_payto_uri TEXT;
+  my_kyc_record RECORD;
+BEGIN
+  SELECT merchant_serial
+    INTO my_merchant_serial
+    FROM merchant_instances
+   WHERE merchant_id = in_merchant_id;
+  IF NOT FOUND
+  THEN
+    RETURN;
+  END IF;
+  FOR my_account_serial, my_h_wire, my_payto_uri
+  IN SELECT account_serial, h_wire, payto_uri
+    FROM merchant_accounts
+    WHERE merchant_serial = my_merchant_serial
+      AND active
+      AND (in_h_wire IS NULL OR h_wire = in_h_wire)
+  LOOP
+    FOR my_kyc_record IN
+      SELECT
+        mk.kyc_serial_id
+       ,mk.exchange_url
+       ,mk.kyc_timestamp
+       ,mk.kyc_ok
+       ,mk.access_token
+       ,mk.exchange_http_status
+       ,mk.exchange_ec_code
+       ,mk.aml_review
+       ,mk.jaccount_limits::TEXT
+      FROM merchant_kyc mk
+      WHERE mk.account_serial = my_account_serial
+        AND (in_exchange_url IS NULL OR mk.exchange_url = in_exchange_url)
+    LOOP
+      UPDATE merchant_kyc
+         SET next_kyc_poll=in_now
+       WHERE kyc_serial_id = my_kyc_record.kyc_serial_id;
+      NOTIFY XDQM4Z4N0D3GX0H9JEXH70EBC2T3KY7HC0TJB0Z60D2H781RXR6AG; 
+      RETURN QUERY
+      SELECT
+        my_h_wire,
+        my_payto_uri,
+        my_kyc_record.exchange_url,
+        my_kyc_record.kyc_timestamp,
+        my_kyc_record.kyc_ok,
+        my_kyc_record.access_token,
+        my_kyc_record.exchange_http_status,
+        my_kyc_record.exchange_ec_code,
+        my_kyc_record.aml_review,
+        my_kyc_record.jaccount_limits::TEXT;
+    END LOOP; 
+    IF NOT FOUND
+    THEN
+      RETURN QUERY
+      SELECT
+        my_h_wire,
+        my_payto_uri,
+        NULL::TEXT,
+        NULL::INT8,
+        NULL::BOOLEAN,
+        NULL::BYTEA,
+        NULL::INT4,
+        NULL::INT4,
+        NULL::BOOLEAN,
+        NULL::TEXT;
+    END IF;
+  END LOOP; 
+END $$;
+COMMENT ON FUNCTION merchant_do_account_kyc_get_status
+  IS 'Returns the KYC status of selected exchanges and accounts, but ALSO resets the next_kyc_check time for all returned data points to the current time (in_now argument)';
 DROP PROCEDURE IF EXISTS merchant_do_gc;
 CREATE PROCEDURE merchant_do_gc(in_now INT8)
 LANGUAGE plpgsql
