@@ -413,6 +413,31 @@ TALER_TESTING_cmd_merchant_post_products3 (
   struct GNUNET_TIME_Timestamp next_restock,
   unsigned int http_status);
 
+/**
+ * Define a "POST /products" CMD with explicit unit_price array.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the
+ *        POST /products request.
+ * @param product_id the ID of the product to create
+ * @param description name of the product
+ * @param unit unit in which the product is measured
+ * @param unit_prices array of unit prices as strings
+ * @param unit_prices_len length of @a unit_prices
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_products_with_unit_prices (
+  const char *label,
+  const char *merchant_url,
+  const char *product_id,
+  const char *description,
+  const char *unit,
+  const char *const *unit_prices,
+  size_t unit_prices_len,
+  unsigned int http_status);
+
 
 /**
  * Define a "POST /products" CMD, simple version
@@ -434,6 +459,86 @@ TALER_TESTING_cmd_merchant_post_products (const char *label,
                                           const char *price,
                                           unsigned int http_status);
 
+/**
+ * Define a "POST /products" CMD with category IDs.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the
+ *        POST /products request.
+ * @param product_id the ID of the product to create
+ * @param description name of the product
+ * @param price price of the product
+ * @param num_cats length of the @a cats array
+ * @param cats array of category IDs
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_products_with_categories (
+  const char *label,
+  const char *merchant_url,
+  const char *product_id,
+  const char *description,
+  const char *unit,
+  const char *price,
+  unsigned int num_cats,
+  const uint64_t *cats,
+  bool unit_allow_fraction,
+  uint32_t unit_precision_level,
+  unsigned int http_status);
+
+/**
+ * Define a "POST /private/categories" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the request.
+ * @param name category name
+ * @param name_i18n category name translations
+ * @param expected_category_id expected category id (0 to ignore)
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_categories (
+  const char *label,
+  const char *merchant_url,
+  const char *name,
+  json_t *name_i18n,
+  uint64_t expected_category_id,
+  unsigned int http_status);
+
+/**
+ * Define a "GET /templates/$ID" wallet CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the request.
+ * @param template_id template ID to fetch
+ * @param expected_products_len expected number of products (0 to ignore)
+ * @param expected_product_id first expected product id (can be NULL)
+ * @param expected_unit expected unit for @a expected_product_id (can be NULL)
+ * @param expected_unit_allow_fraction expected fractional flag
+ * @param expected_unit_precision_level expected precision
+ * @param expected_product_id2 second expected product id (can be NULL)
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_wallet_get_template (
+  const char *label,
+  const char *merchant_url,
+  const char *template_id,
+  size_t expected_products_len,
+  const char *expected_product_id,
+  const char *expected_unit,
+  bool expected_unit_allow_fraction,
+  uint32_t expected_unit_precision_level,
+  json_t *expected_unit_name_short_i18n,
+  const char *expected_product_id2,
+  const char *expected_product_id3,
+  uint64_t expected_category_id1,
+  uint64_t expected_category_id2,
+  unsigned int http_status);
+
 
 /**
  * Define a "PATCH /products/$ID" CMD.
@@ -498,6 +603,31 @@ TALER_TESTING_cmd_merchant_patch_product2 (
   struct GNUNET_TIME_Timestamp next_restock,
   unsigned int http_status);
 
+/**
+ * Define a "PATCH /products/$ID" CMD with explicit unit_price array.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the
+ *        PATCH /product request.
+ * @param product_id the ID of the product to query
+ * @param description description of the product
+ * @param unit unit in which the product is measured
+ * @param unit_prices array of unit prices as strings
+ * @param unit_prices_len length of @a unit_prices
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_patch_product_with_unit_prices (
+  const char *label,
+  const char *merchant_url,
+  const char *product_id,
+  const char *description,
+  const char *unit,
+  const char *const *unit_prices,
+  size_t unit_prices_len,
+  unsigned int http_status);
+
 
 /**
  * Define a "GET /products" CMD.
@@ -1877,6 +2007,29 @@ TALER_TESTING_cmd_merchant_post_using_templates (
   struct GNUNET_TIME_Timestamp pay_deadline,
   unsigned int http_status);
 
+/**
+ * Define a "POST /using-templates" CMD with a raw JSON request body.
+ *
+ * @param label command label.
+ * @param template_ref label of command that created the template to use
+ * @param otp_ref label of command that created OTP device we use (or NULL for no OTP)
+ * @param merchant_url base URL of the merchant serving the
+ *        POST /using-templates request.
+ * @param using_template_id template ID to use
+ * @param details raw JSON request body to send
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_using_templates2 (
+  const char *label,
+  const char *template_ref,
+  const char *otp_ref,
+  const char *merchant_url,
+  const char *using_template_id,
+  const json_t *details,
+  unsigned int http_status);
+
 
 /* ****** Token Families ******* */
 
@@ -2248,6 +2401,18 @@ TALER_TESTING_cmd_merchant_get_statisticsamount (const char *label,
                                                  uint64_t intervals_length,
                                                  unsigned int http_status);
 
+
+/**
+ * Run the taler-merchant-donaukeyupdate command (with -t).
+ *
+ * @param label command label
+ * @param config_filename configuration file to pass to the command
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_exec_donaukeyupdate (const char *label,
+                                       const char *config_filename);
+
+
 /* ****** Specific traits supported by this component ******* */
 
 
@@ -2263,6 +2428,7 @@ TALER_TESTING_cmd_merchant_get_statisticsamount (const char *label,
   op (instance_name, const char) \
   op (instance_id, const char) \
   op (address, const json_t) \
+  op (category_id, const uint64_t) \
   op (product_description, const char) \
   op (product_image, const char) \
   op (product_stock, const int64_t) \
@@ -2325,20 +2491,20 @@ TALER_TESTING_cmd_merchant_get_statisticsamount (const char *label,
  * @param op macro to call
  */
 #define TALER_MERCHANT_TESTING_INDEXED_TRAITS(op) \
-  op (coin_reference, const char) \
-  op (paths, const char) \
-  op (payto_uris, const struct TALER_FullPayto) \
-  op (h_wires, const struct TALER_MerchantWireHashP) \
-  op (amounts, const struct TALER_Amount)         \
-  op (urls, const char) \
-  op (http_methods, const char)   \
-  op (http_header, const char)   \
-  op (http_body, const void)   \
-  op (http_body_size, const size_t)   \
-  op (planchet_secrets, const struct TALER_PlanchetMasterSecretP)   \
-  op (token_priv, const struct TALER_TokenUsePrivateKeyP)   \
-  op (token_issue_sig, const struct TALER_TokenIssueSignature)   \
-  op (token_issue_pub, const struct TALER_TokenIssuePublicKey)
+        op (coin_reference, const char) \
+        op (paths, const char) \
+        op (payto_uris, const struct TALER_FullPayto) \
+        op (h_wires, const struct TALER_MerchantWireHashP) \
+        op (amounts, const struct TALER_Amount)         \
+        op (urls, const char) \
+        op (http_methods, const char)   \
+        op (http_header, const char)   \
+        op (http_body, const void)   \
+        op (http_body_size, const size_t)   \
+        op (planchet_secrets, const struct TALER_PlanchetMasterSecretP)   \
+        op (token_priv, const struct TALER_TokenUsePrivateKeyP)   \
+        op (token_issue_sig, const struct TALER_TokenIssueSignature)   \
+        op (token_issue_pub, const struct TALER_TokenIssuePublicKey)
 
 
 TALER_MERCHANT_TESTING_SIMPLE_TRAITS (TALER_TESTING_MAKE_DECL_SIMPLE_TRAIT)
