@@ -1,6 +1,6 @@
 /*
   This file is part of GNU Taler
-  Copyright (C) 2024 Taler Systems SA
+  Copyright (C) 2024, 2025 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -24,6 +24,7 @@
 #include <gnunet/gnunet_common.h>
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_json_lib.h>
+
 #include <stdint.h>
 #include <taler/taler_util.h>
 #include <jansson.h>
@@ -32,8 +33,8 @@
  * Fixed-point base for inventory quantities (powers of 10).
  * Six decimal digits are supported to match the maximum unit precision.
  */
-#define MERCHANT_UNIT_FRAC_BASE 1000000U
-#define MERCHANT_UNIT_FRAC_MAX_DIGITS 6U
+#define TALER_MERCHANT_UNIT_FRAC_BASE 1000000U
+#define TALER_MERCHANT_UNIT_FRAC_MAX_DIGITS 6U
 
 /**
  * Return default project data used by Taler merchant.
@@ -42,6 +43,130 @@ const struct GNUNET_OS_ProjectData *
 TALER_MERCHANT_project_data (void);
 
 
+/**
+ * Kind of fixed-decimal value the helpers operate on.
+ * Primarily distinguishes how special sentinel values (such as "-1"
+ * meaning infinity for stock) must be encoded.
+ */
+enum TALER_MERCHANT_ValueKind
+{
+  TALER_MERCHANT_VK_QUANTITY,  /* -1 is illegal                    */
+  TALER_MERCHANT_VK_STOCK      /* -1 means "infinity"              */
+};
+
+
+/**
+ * Parse decimal quantity expressed as string for request handling.
+ *
+ * @param value string to parse
+ * @param[out] integer_part result integer component
+ * @param[out] fractional_part result fractional component (0..TALER_MERCHANT_UNIT_FRAC_BASE-1)
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR on validation failure
+ */
+enum GNUNET_GenericReturnValue
+TALER_MERCHANT_vk_parse_fractional_string (
+  const char *value,
+  int64_t *integer_part,
+  uint32_t *fractional_part);
+
+
+/**
+ * Extract a fixed-decimal number that may be supplied either
+ *   - as pure integer  (e.g. "total_stock"), or
+ *   - as decimal text (e.g. "unit_total_stock").
+ *
+ * Rules:
+ *  - If both forms are missing          -> error.
+ *  - If both are present                -> they must match and the decimal must have no fraction.
+ *  - For kind == TALER_MERCHANT_VK_STOCK the integer value -1 represents infinity.
+ *
+ * @param kind             See #TALER_MERCHANT_ValueKind
+ * @param allow_fractional False: any fractional part is rejected
+ * @param int_missing      True if client omitted the integer field
+ * @param int_raw          Raw integer (undefined if @a int_missing is true)
+ * @param str_missing      True if client omitted the string field
+ * @param str_raw          Raw UTF-8 string (undefined if @a str_missing is true)
+ * @param[out] int_out     Canonicalised integer part
+ * @param[out] frac_out    Canonicalised fractional part
+ * @param[out] error_param Set to offending field name on failure
+ * @param int_field        Integer field name (for error reporting)
+ * @param str_field        String field name (for error reporting)
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR otherwise
+ */
+enum GNUNET_GenericReturnValue
+TALER_MERCHANT_vk_process_quantity_inputs (
+  enum TALER_MERCHANT_ValueKind kind,
+  bool allow_fractional,
+  bool int_missing,
+  int64_t int_raw,
+  bool str_missing,
+  const char *str_raw,
+  uint64_t *int_out,
+  uint32_t *frac_out,
+  const char **error_param);
+
+/**
+ * Format a fixed-decimal pair into canonical string representation.
+ * Recognises INT64_MAX / INT32_MAX as the "-1" sentinel used for
+ * infinite stock if @a kind equals #TALER_MERCHANT_VK_STOCK.
+ *
+ * @param kind specifies whether sentinel values are permitted
+ * @param integer integer portion
+ * @param fractional fractional portion (0..TALER_MERCHANT_UNIT_FRAC_BASE-1 or sentinel)
+ * @param buffer output buffer
+ * @param buffer_length length of @a buffer
+ */
+void
+TALER_MERCHANT_vk_format_fractional_string (
+  enum TALER_MERCHANT_ValueKind kind,
+  uint64_t integer,
+  uint32_t fractional,
+  size_t buffer_length,
+  char buffer[static buffer_length]);
+
+
+/**
+ * Check if @a image_data_url is a valid image
+ * data URL. Does not validate the actual payload,
+ * only the syntax and that it properly claims to
+ * be an image.
+ *
+ * FIXME: use in TALER_MERCHANT_parse_product!
+ *
+ * @param image_data_url string to check
+ * @return true if @a image_data_url is a data
+ *         URL with an "image/" mime-type
+ */
+bool
+TALER_MERCHANT_image_data_url_valid (const char *image_data_url);
+
+
+/**
+ * Check if @a email is a valid email address.
+ *
+ * FIXME: move to libgnunetutil?
+ *
+ * @param email string to check
+ * @return true if @a email is a valid e-mail address
+ */
+bool
+TALER_MERCHANT_email_valid (const char *email);
+
+
+/**
+ * Check if @a email is a valid international phone number
+ * with "+CC" prefix.
+ *
+ * @param phone phone number to check
+ * @param allow_letters to allow "A-Z" letters representing digits
+ * @return normalized phone number if @a phone is valid,
+ *   NULL if @a phone is not a phone number
+ */
+char *
+TALER_MERCHANT_phone_validate_normalize (const char *phone,
+                                         bool allow_letters);
+
+
 /**
  * Channel used to transmit MFA authorization request.
  */
@@ -108,6 +233,183 @@ enum TALER_MERCHANT_MFA_CriticalOperation
 
 };
 
+/**
+ * Template type discriminator.
+ */
+enum TALER_MERCHANT_TemplateType
+{
+  TALER_MERCHANT_TEMPLATE_TYPE_INVALID = 0,
+  TALER_MERCHANT_TEMPLATE_TYPE_FIXED_ORDER,
+  TALER_MERCHANT_TEMPLATE_TYPE_INVENTORY_CART,
+  TALER_MERCHANT_TEMPLATE_TYPE_PAIVANA
+};
+
+/**
+ * Determine template type from string.
+ *
+ * @param template_type string value (NULL means fixed-order)
+ * @return template type (defaults to fixed order)
+ */
+enum TALER_MERCHANT_TemplateType
+TALER_MERCHANT_template_type_from_string (const char *template_type);
+
+/**
+ * Convert template type to its string representation.
+ *
+ * @param template_type template type to convert
+ * @return string name or NULL for invalid types
+ */
+const char *
+TALER_MERCHANT_template_type_to_string (
+  enum TALER_MERCHANT_TemplateType template_type);
+
+/**
+ * Determine template type from a template contract.
+ *
+ * @param template_contract contract JSON
+ * @return template type (defaults to fixed order)
+ */
+enum TALER_MERCHANT_TemplateType
+TALER_MERCHANT_template_type_from_contract (const json_t *template_contract);
+
+/**
+ * Template contract fields for inventory templates.
+ */
+struct TALER_MERCHANT_TemplateContractInventory
+{
+  /**
+   * Selected categories from the template contract.
+   */
+  const json_t *selected_categories;
+
+  /**
+   * Selected products from the template contract.
+   */
+  const json_t *selected_products;
+
+  /**
+   * Whether all products are selectable.
+   */
+  bool selected_all;
+
+  /**
+   * Template requires exactly one selection.
+   */
+  bool choose_one;
+
+};
+
+/**
+ * Template contract fields for paivana templates.
+ */
+struct TALER_MERCHANT_TemplateContractPaivana
+{
+  /**
+   * Paivana website regular expression.
+   * NULL to allow any site.
+   */
+  const char *website_regex;
+
+  /**
+   * Array of possible specific contracts the wallet/customer may choose
+   * from by selecting the respective index when signing the deposit
+   * confirmation.
+   */
+  struct TALER_MERCHANT_ContractChoice *choices;
+
+  /**
+   * Length of the @e choices array.
+   */
+  unsigned int choices_len;
+
+};
+
+/**
+ * Parsed template contract.
+ */
+struct TALER_MERCHANT_TemplateContract
+{
+  /**
+   * Template type.
+   */
+  enum TALER_MERCHANT_TemplateType type;
+
+  /**
+   * Summary from the template contract.
+   */
+  const char *summary;
+
+  /**
+   * Currency from the template contract.
+   */
+  const char *currency;
+
+  /**
+   * Amount from the template contract.
+   */
+  struct TALER_Amount amount;
+
+  /**
+   * True if @e amount was not provided.
+   */
+  bool no_amount;
+
+  /**
+   * Template allows tips.
+   */
+  bool request_tip;
+
+  /**
+   * Minimum age required by the template.
+   */
+  uint32_t minimum_age;
+
+  /**
+   * How long does the customer have to pay for the order.
+   * 0 if not specified (use instance default).
+   */
+  struct GNUNET_TIME_Relative pay_duration;
+
+  union
+  {
+
+    /**
+     * Parsed fields for inventory templates.
+     */
+    struct TALER_MERCHANT_TemplateContractInventory inventory;
+
+    /**
+     * Parsed fields for paivana templates.
+     */
+    struct TALER_MERCHANT_TemplateContractPaivana paivana;
+
+  } details;
+
+};
+
+/**
+ * Parse template contract JSON into @a out.
+ *
+ * @param template_contract JSON object containing the template contract
+ * @param[out] out parsed template contract
+ * @param[out] error_name pointer to the name of the failed field, or NULL
+ * @return #GNUNET_SYSERR if @a template_contract is malformed; #GNUNET_OK otherwise
+ */
+enum GNUNET_GenericReturnValue
+TALER_MERCHANT_template_contract_parse (
+  const json_t *template_contract,
+  struct TALER_MERCHANT_TemplateContract *out,
+  const char **error_name);
+
+/**
+ * Check if @a template_contract is valid.
+ *
+ * @param template_contract template contract to validate
+ * @return true if @a template_contract is valid
+ */
+bool
+TALER_MERCHANT_template_contract_valid (const json_t *template_contract);
+
 
 /**
  * Convert critical operation enumeration value to string
@@ -187,6 +489,18 @@ struct TALER_MERCHANT_MFA_BodySalt
 };
 
 
+/**
+ * @brief Token used to authorize report generation.
+ */
+struct TALER_MERCHANT_ReportToken
+{
+  /**
+   * Salt.
+   */
+  uint64_t salt[256 / 64];
+};
+
+
 /**
  * Hash the given request @a body with the given @a salt to
  * produce @a h_body for MFA checks.
@@ -444,6 +758,16 @@ struct TALER_MERCHANT_ContractChoice
    */
   struct TALER_Amount amount;
 
+  /**
+   * Tip included by the customer (part of the total amount).
+   */
+  struct TALER_Amount tip;
+
+  /**
+   * True if @e tip was not provided.
+   */
+  bool no_tip;
+
   /**
    * Human readable description of the semantics of the choice within the
    * contract to be shown to the user at payment.
@@ -602,6 +926,147 @@ struct TALER_MERCHANT_ContractTokenFamily
   } details;
 };
 
+/**
+ * Specifies the quantity of a product (to be) sold.
+ */
+struct TALER_MERCHANT_ProductQuantity
+{
+  /**
+   * Integer component of the quantity.
+   */
+  uint64_t integer;
+
+  /**
+   * Fractional component of the quantity, in the
+   * range of 0..TALER_MERCHANT_UNIT_FRAC_BASE-1.
+   */
+  uint32_t fractional;
+
+};
+
+
+/**
+ * How to round when computing with amounts?
+ */
+enum TALER_MERCHANT_RoundMode
+{
+  TALER_MERCHANT_ROUND_NEAREST,
+  TALER_MERCHANT_ROUND_UP,
+  TALER_MERCHANT_ROUND_DOWN
+};
+
+
+/**
+ * Multiply the @a unit_price by the quantity given in @a factor.
+ * Round the result using the given rounding mode @a rm to a
+ * multiple of the @a atomic_amount.
+ *
+ * @param[out] result where to store the result
+ * @param unit_price price for one item
+ * @param factor quantity to purchase, can be factional
+ * @param rm rounding mode to apply
+ * @param atomic_amount granularity to round to
+ * @return #GNUNET_OK on success
+ *   #GNUNET_NO on integer overflow (resulting amount cannot be represented)
+ *   #GNUNET_SYSERR on internal failure (e.g. currency
+ *     @a unit_price and @a atomic_amount do not match)
+ */
+enum GNUNET_GenericReturnValue
+TALER_MERCHANT_amount_multiply_by_quantity (
+  struct TALER_Amount *result,
+  const struct TALER_Amount *unit_price,
+  const struct TALER_MERCHANT_ProductQuantity *factor,
+  enum TALER_MERCHANT_RoundMode rm,
+  const struct TALER_Amount *atomic_amount);
+
+
+/**
+ * Details about a product (to be) sold.
+ */
+struct TALER_MERCHANT_ProductSold
+{
+
+  /**
+   * Merchant-internal identifier for the product. NULL for none.
+   */
+  char *product_id;
+
+  /**
+   * Name of the product. NULL for pre **v20** contract terms.
+   */
+  char *product_name;
+
+  /**
+   * Human-readable product description.
+   */
+  char *description;
+
+  /**
+   * Map from IETF BCP 47 language tags to localized descriptions.
+   * NULL if no translations are available.
+   */
+  json_t *description_i18n;
+
+  /**
+   * Quantity (in multiples of @e unit) to be sold.
+   */
+  struct TALER_MERCHANT_ProductQuantity unit_quantity;
+
+  /**
+   * Unit in which the product is measured (liters, kilograms, packages,
+   * etc.). Can be NULL if not specified.
+   */
+  char *unit;
+
+  /**
+   * Length of the @e prices array.
+   */
+  unsigned int prices_length;
+
+  /**
+   * Array of prices (in different currencies) for @e unit_quantity
+   * units of this product (these are longer the per-unit prices!).
+   */
+  struct TALER_Amount *prices;
+
+  /**
+   * True if the @e prices given are the net price,
+   * false if they are the gross price.  Note that even @e prices are the
+   * gross price, @e taxes may be missing if the merchant configured
+   * gross @e prices but did not configure any @e taxes.
+   * Similarly, the merchant may have configured net @e prices
+   * for products but deals with taxes on a per-order basis. Thus, it
+   * may not always be possible to compute the gross price from the net
+   * price for an individual product, necessitating this flag.
+   */
+  bool prices_are_net;
+
+  /**
+   * An optional base64-encoded image of the product.
+   */
+  char *image;
+
+  /**
+   * A list of taxes paid by the merchant for this product. Can be NULL.
+   * Will likely change soon!
+   */
+  json_t *taxes;
+
+  /**
+   * Time indicating when this product should be delivered.
+   * #GNUNET_TIME_UNIT_FOREVER_TS for unknown / not specified.
+   */
+  struct GNUNET_TIME_Timestamp delivery_date;
+
+  /**
+   * Money pot to use for this product, overrides value from
+   * the inventory if given.  Not useful to wallets, only for
+   * merchant-internal accounting.
+   */
+  uint64_t product_money_pot;
+
+};
+
 
 /**
  * Struct to hold contract terms.
@@ -687,10 +1152,15 @@ struct TALER_MERCHANT_Contract
    */
   json_t *fulfillment_message_i18n;
 
+  /**
+   * Length of the @e products array.
+   */
+  size_t products_len;
+
   /**
    * Array of products that are part of the purchase.
    */
-  json_t *products;
+  struct TALER_MERCHANT_ProductSold *products;
 
   /**
    * Timestamp of the contract.
@@ -723,7 +1193,6 @@ struct TALER_MERCHANT_Contract
    */
   struct GNUNET_TIME_Timestamp delivery_date;
 
-
   /**
    * Merchant public key.
    */
@@ -731,13 +1200,12 @@ struct TALER_MERCHANT_Contract
 
   /**
    * The hash of the merchant instance's wire details.
-   * TODO: appropriate type
    */
   struct TALER_MerchantWireHashP h_wire;
 
   /**
    * Wire transfer method identifier for the wire method associated with
-   h_wire.
+   * @e h_wire.
    */
   char *wire_method;
 
@@ -768,6 +1236,15 @@ struct TALER_MERCHANT_Contract
    */
   uint8_t minimum_age;
 
+  /**
+   * Default money pot to use for this product, applies to the
+   * amount remaining that was not claimed by money pots of
+   * products or taxes.  Not useful to wallets, only for
+   * merchant-internal accounting.  If zero, the remaining
+   * account is simply not accounted for in any money pot.
+   */
+  uint64_t default_money_pot;
+
   /**
    * Specified version of the contract.
    */
@@ -793,6 +1270,16 @@ struct TALER_MERCHANT_Contract
        */
       struct TALER_Amount brutto;
 
+      /**
+       * Tip included by the customer (part of the total amount).
+       */
+      struct TALER_Amount tip;
+
+      /**
+       * True if @e tip was not provided.
+       */
+      bool no_tip;
+
       /**
       * Maximum fee as given by the client request.
       */
@@ -860,6 +1347,21 @@ TALER_MERCHANT_json_spec_cit (const char *name,
                               enum TALER_MERCHANT_ContractInputType *cit);
 
 
+/**
+ * Provide specification to parse given JSON array to contract terms
+ * choices. All fields from @a choices elements are copied.
+ *
+ * @param name name of the choices field in the JSON
+ * @param[out] choices where the contract choices array has to be written
+ * @param[out] choices_len length of the @a choices array
+ */
+struct GNUNET_JSON_Specification
+TALER_MERCHANT_spec_choices (
+  const char *name,
+  struct TALER_MERCHANT_ContractChoice **choices,
+  unsigned int *choices_len);
+
+
 /**
  * Parse JSON contract terms choice input.
  *
@@ -877,6 +1379,31 @@ TALER_MERCHANT_parse_choice_input (
   bool order);
 
 
+/**
+ * Check if @a taxes is an array of valid Taxes in the sense of
+ * Taler's API definition.
+ *
+ * @param taxes array to check
+ * @return true if @a taxes is an array and all
+ *         entries are valid Taxes.
+ */
+bool
+TALER_MERCHANT_taxes_array_valid (const json_t *taxes);
+
+
+/**
+ * Parse JSON product given in @a p, returning the result in
+ * @a r.
+ *
+ * @param p JSON specifying a ``ProductSold`` to parse
+ * @param[out] r where to write the result
+ * @return #GNUNET_OK on success
+ */
+enum GNUNET_GenericReturnValue
+TALER_MERCHANT_parse_product_sold (const json_t *p,
+                                   struct TALER_MERCHANT_ProductSold *r);
+
+
 /**
  * Provide specification to parse an JSON contract output type.
  * The value is provided as a descriptive string.
@@ -907,6 +1434,17 @@ TALER_MERCHANT_parse_choice_output (
   bool order);
 
 
+/**
+ * Serialize @a p to JSON.
+ *
+ * @param p product to serialize
+ * @return JSON object representing the product @a p
+ */
+json_t *
+TALER_MERCHANT_product_sold_serialize (
+  const struct TALER_MERCHANT_ProductSold *p);
+
+
 /**
  * Serialize contract terms into JSON object.
  *
@@ -976,6 +1514,15 @@ TALER_MERCHANT_contract_choice_free (
   struct TALER_MERCHANT_ContractChoice *choice);
 
 
+/**
+ * Release memory inside of @a product, but not @a product itself.
+ *
+ * @param[in] product data structure to clean up
+ */
+void
+TALER_MERCHANT_product_sold_free (struct TALER_MERCHANT_ProductSold *product);
+
+
 /**
  * Free the @a contract and all fields in it.
  *
@@ -984,4 +1531,5 @@ TALER_MERCHANT_contract_choice_free (
 void
 TALER_MERCHANT_contract_free (struct TALER_MERCHANT_Contract *contract);
 
+
 #endif
