@@ -38,7 +38,7 @@ TALER_MERCHANT_format_fractional_string (uint64_t integer,
 {
   GNUNET_assert (NULL != buffer);
   GNUNET_assert (0 < buffer_length);
-  GNUNET_assert (fractional < MERCHANT_UNIT_FRAC_BASE);
+  GNUNET_assert (fractional < TALER_MERCHANT_UNIT_FRAC_BASE);
 
   if (0 == fractional)
   {
@@ -49,13 +49,13 @@ TALER_MERCHANT_format_fractional_string (uint64_t integer,
     return;
   }
   {
-    char frac_buf[MERCHANT_UNIT_FRAC_MAX_DIGITS + 1];
+    char frac_buf[TALER_MERCHANT_UNIT_FRAC_MAX_DIGITS + 1];
     size_t idx;
 
     GNUNET_snprintf (frac_buf,
                      sizeof (frac_buf),
                      "%0*u",
-                     MERCHANT_UNIT_FRAC_MAX_DIGITS,
+                     TALER_MERCHANT_UNIT_FRAC_MAX_DIGITS,
                      (unsigned int) fractional);
     for (idx = strlen (frac_buf); idx > 0; idx--)
     {
