@@ -103,12 +103,9 @@ parse_instances (const json_t *json,
 
     json_array_foreach (ia, index, value) {
       struct TALER_MERCHANT_InstanceInformation *ii = &iis[index];
-      const char *uts;
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_string ("name",
                                  &ii->name),
-        GNUNET_JSON_spec_string ("user_type",
-                                 &uts),
         GNUNET_JSON_spec_string ("id",
                                  &ii->id),
         GNUNET_JSON_spec_fixed_auto ("merchant_pub",
