@@ -16,7 +16,7 @@
 */
 /**
  * @file merchant_api_get_orders.c
- * @brief Implementation of the GET /orders request of the merchant's HTTP API
+ * @brief Implementation of the GET /private/orders request of the merchant's HTTP API
  * @author Christian Grothoff
  */
 #include "platform.h"
@@ -94,6 +94,9 @@ parse_orders (const json_t *ia,
     size_t index;
     json_t *value;
 
+    memset (oes,
+            0,
+            sizeof (oes));
     json_array_foreach (ia, index, value) {
       struct TALER_MERCHANT_OrderEntry *ie = &oes[index];
       struct GNUNET_JSON_Specification spec[] = {
@@ -105,6 +108,14 @@ parse_orders (const json_t *ia,
                                  &ie->order_serial),
         TALER_JSON_spec_amount_any ("amount",
                                     &ie->amount),
+        GNUNET_JSON_spec_mark_optional (
+          TALER_JSON_spec_amount_any ("refund_amount",
+                                      &ie->refund_amount),
+          NULL),
+        GNUNET_JSON_spec_mark_optional (
+          TALER_JSON_spec_amount_any ("pending_refund_amount",
+                                      &ie->pending_refund_amount),
+          NULL),
         GNUNET_JSON_spec_string ("summary",
                                  &ie->summary),
         GNUNET_JSON_spec_bool ("refundable",
