@@ -131,7 +131,17 @@ handle_refund_finished (void *cls,
     /* Nothing really to verify, merchant says we need to authenticate. */
     break;
   case MHD_HTTP_NOT_FOUND:
+    /* order unknown (or not paid) */
+    rr.hr.ec = TALER_JSON_get_error_code (json);
+    rr.hr.hint = TALER_JSON_get_error_hint (json);
+    break;
   case MHD_HTTP_CONFLICT:
+    /* amount not acceptable */
+    rr.hr.ec = TALER_JSON_get_error_code (json);
+    rr.hr.hint = TALER_JSON_get_error_hint (json);
+    break;
+  case MHD_HTTP_GONE:
+    /* too late, wire deadline is past */
     rr.hr.ec = TALER_JSON_get_error_code (json);
     rr.hr.hint = TALER_JSON_get_error_hint (json);
     break;
