@@ -96,26 +96,28 @@ handle_post_using_templates_finished (void *cls,
 }
 
 
-struct TALER_MERCHANT_UsingTemplatesPostHandle *
-TALER_MERCHANT_using_templates_post (
+/**
+ * Helper to submit a POST /templates/$ID request.
+ *
+ * @param ctx the context
+ * @param backend_url HTTP base URL for the backend
+ * @param template_id identifier of the template to use
+ * @param req_obj JSON request body (consumed)
+ * @param cb function to call with the backend's result
+ * @param cb_cls closure for @a cb
+ * @return the request handle; NULL upon error
+ */
+static struct TALER_MERCHANT_UsingTemplatesPostHandle *
+using_templates_post_internal (
   struct GNUNET_CURL_Context *ctx,
   const char *backend_url,
   const char *template_id,
-  const char *summary,
-  const struct TALER_Amount *amount,
+  json_t *req_obj,
   TALER_MERCHANT_PostOrdersCallback cb,
   void *cb_cls)
 {
   struct TALER_MERCHANT_UsingTemplatesPostHandle *utph;
-  json_t *req_obj;
 
-  req_obj = GNUNET_JSON_PACK (
-    GNUNET_JSON_pack_allow_null (
-      GNUNET_JSON_pack_string ("summary",
-                               summary)),
-    GNUNET_JSON_pack_allow_null (
-      TALER_JSON_pack_amount ("amount",
-                              amount)));
   utph = GNUNET_new (struct TALER_MERCHANT_UsingTemplatesPostHandle);
   utph->ctx = ctx;
   utph->cb = cb;
@@ -159,6 +161,60 @@ TALER_MERCHANT_using_templates_post (
 }
 
 
+struct TALER_MERCHANT_UsingTemplatesPostHandle *
+TALER_MERCHANT_using_templates_post (
+  struct GNUNET_CURL_Context *ctx,
+  const char *backend_url,
+  const char *template_id,
+  const char *summary,
+  const struct TALER_Amount *amount,
+  TALER_MERCHANT_PostOrdersCallback cb,
+  void *cb_cls)
+{
+  json_t *req_obj;
+
+  req_obj = GNUNET_JSON_PACK (
+    GNUNET_JSON_pack_string ("template_type",
+                             "fixed-order"),
+    GNUNET_JSON_pack_allow_null (
+      GNUNET_JSON_pack_string ("summary",
+                               summary)),
+    GNUNET_JSON_pack_allow_null (
+      TALER_JSON_pack_amount ("amount",
+                              amount)));
+  return using_templates_post_internal (ctx,
+                                        backend_url,
+                                        template_id,
+                                        req_obj,
+                                        cb,
+                                        cb_cls);
+}
+
+
+struct TALER_MERCHANT_UsingTemplatesPostHandle *
+TALER_MERCHANT_using_templates_post2 (
+  struct GNUNET_CURL_Context *ctx,
+  const char *backend_url,
+  const char *template_id,
+  const json_t *details,
+  TALER_MERCHANT_PostOrdersCallback cb,
+  void *cb_cls)
+{
+  if (NULL == details)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "No request body provided for using templates.\n");
+    return NULL;
+  }
+  return using_templates_post_internal (ctx,
+                                        backend_url,
+                                        template_id,
+                                        (json_t *) details,
+                                        cb,
+                                        cb_cls);
+}
+
+
 void
 TALER_MERCHANT_using_templates_post_cancel (
   struct TALER_MERCHANT_UsingTemplatesPostHandle *utph)
