@@ -32,6 +32,10 @@
 #include <taler/taler_signatures.h>
 #include <taler/taler_curl_lib.h>
 
+/**
+ * Maximum number of refunds we return.
+ */
+#define MAX_REFUNDS 1024
 
 /**
  * Handle for a (public) POST /orders/ID/refund operation.
@@ -123,6 +127,14 @@ handle_refund_finished (void *cls,
         break;
       }
       refund_len = json_array_size (refunds);
+      if ( (json_array_size (refunds) != (size_t)  refund_len) ||
+           (refund_len > MAX_REFUNDS) )
+      {
+        GNUNET_break (0);
+        wrr.hr.ec = TALER_EC_GENERIC_ALLOCATION_FAILURE;
+        wrr.hr.http_status = 0;
+        break;
+      }
       {
         struct TALER_MERCHANT_RefundDetail rds[GNUNET_NZL (refund_len)];
 
