@@ -55,17 +55,13 @@ run (void *cls,
   const char *pw = args[0];
   struct TALER_MERCHANTDB_InstanceAuthSettings ias;
   enum GNUNET_DB_QueryStatus qs;
+  unsigned int nxt = 1;
 
-  if (NULL != args[1])
-  {
-    fprintf (stderr,
-             "Superfluous command-line option `%s' specified\n",
-             args[1]);
-    global_ret = -1;
-    return;
-  }
   if (NULL == pw)
+  {
     pw = getenv ("TALER_MERCHANT_PASSWORD");
+    nxt = 0;
+  }
   if (NULL == pw)
   {
     fprintf (stderr,
@@ -73,6 +69,14 @@ run (void *cls,
     global_ret = -1;
     return;
   }
+  if (NULL != args[nxt])
+  {
+    fprintf (stderr,
+             "Superfluous command-line option `%s' specified\n",
+             args[nxt]);
+    global_ret = -1;
+    return;
+  }
   if (NULL == instance)
     instance = GNUNET_strdup ("admin");
   cfg = GNUNET_CONFIGURATION_dup (config);
