@@ -68,6 +68,8 @@ TOKEN=$(jq -e -r .access_token < $LAST_RESPONSE)
 
 echo " OK" >&2
 
+echo -n "Setting up bank account..." >&2
+
 STATUS=$(curl -H "Content-Type: application/json" -X POST \
     -H "Authorization: Bearer $TOKEN" \
     http://localhost:9966/private/accounts \
@@ -97,7 +99,8 @@ NEW_SECRET="different_value"
 taler-merchant-exchangekeyupdate \
     -c "${CONF}" \
     -L DEBUG \
-    2> taler-merchant-exchangekeyupdate2.log &
+    -t \
+    2> taler-merchant-exchangekeyupdate2.log 
 taler-merchant-passwd \
     -c "${CONF}" \
     -L DEBUG \
@@ -151,6 +154,17 @@ fi
 
 TOKEN=$(jq -e -r .access_token < $LAST_RESPONSE)
 
+echo -n "Making sure merchant KYC data is current ..." >&2
+
+taler-merchant-kyccheck \
+    -c "${CONF}" \
+    -L DEBUG \
+    -t \
+    2> taler-merchant-kyccheck.log
+
+sleep 1
+echo " OK"
+
 echo -n "Creating order to test auth is ok..." >&2
 STATUS=$(curl -H "Content-Type: application/json" -X POST \
     'http://localhost:9966/private/orders' \
@@ -343,6 +357,8 @@ fi
 
 echo " OK" >&2
 
+echo -n "Creating refreshable login token..." >&2
+
 STATUS=$(curl -H "Content-Type: application/json" -X POST \
     -H 'Authorization: Basic '"$BASIC_AUTH2" \
     http://localhost:9966/instances/second/private/token \
