@@ -19,7 +19,7 @@ then
 else
     ACCOUNT="exchange-account-1"
     BANK_FLAGS="-ns -d iban -u $ACCOUNT"
-    BANK_URL="http://localhost:18082/"
+    BANK_URL="http://localhost:8082/"
     echo -n "Testing for libeufin-bank"
     libeufin-bank --help >/dev/null </dev/null || exit_skip " MISSING"
     echo " FOUND"
@@ -254,9 +254,10 @@ VALID_BEFORE="{\"t_s\": $(date +%s -d "+30 days")}" # 30 days from now
 DURATION="{\"d_us\": $(expr 30 \* 24 \* 60 \* 60 \* 1000000)}" # 30 days
 STATUS=$(curl 'http://localhost:9966/private/tokenfamilies' \
               -d "{\"kind\": \"discount\", \"slug\":\"test-discount\", \"name\": \"Test discount\", \"description\": \"Less money $$\", \"description_i18n\": {\"en\": \"Less money $$\", \"es\": \"Menos dinero $$\"}, \"valid_after\": $VALID_AFTER, \"valid_before\": $VALID_BEFORE, \"duration\": $DURATION, \"validity_granularity\": $DURATION}" \
-              -w "%{http_code}" -s -o /dev/null)
+              -w "%{http_code}" -s -o "$LAST_RESPONSE")
 if [ "$STATUS" != "204" ]
 then
+    cat "$LAST_RESPONSE" >&2
     exit_fail "Expected '204 OK' response. Got instead $STATUS"
 fi
 echo "Ok"
@@ -270,10 +271,11 @@ VALID_BEFORE="{\"t_s\": $(date +%s -d "+30 days")}" # 30 days from now
 DURATION="{\"d_us\": $(expr 30 \* 24 \* 60 \* 60 \* 1000000)}" # 30 days
 STATUS=$(curl 'http://localhost:9966/private/tokenfamilies' \
               -d "{\"kind\": \"subscription\", \"slug\":\"test-subscription\", \"name\": \"Test subscription\", \"description\": \"Money per month\", \"description_i18n\": {\"en\": \"Money $$$ per month\", \"es\": \"Dinero $$$ al mes\"}, \"valid_after\": $VALID_AFTER, \"valid_before\": $VALID_BEFORE, \"duration\": $DURATION, \"validity_granularity\": $DURATION}" \
-              -w "%{http_code}" -s -o /dev/null)
+              -w "%{http_code}" -s -o "$LAST_RESPONSE")
 if [ "$STATUS" != "204" ]
 then
-    exit_fail "Expected '204 OK' response. Got instead $STATUS"
+   cat "$LAST_RESPONSE" >&2
+   exit_fail "Expected '204 OK' response. Got instead $STATUS"
 fi
 echo "Ok"
 
@@ -310,7 +312,7 @@ echo "OK"
 
 echo -n "Claming order with token family ..."
 STATUS=$(curl http://localhost:9966/orders/"$ORDER_ID"/claim \
-    -d '{"nonce":"","token":"'"$TOKEN"'"}' \
+    -d '{"nonce":"0FTTZ475NBECCZN97VFTN6DEKWVKJ8AQY9PWR6VS36JZQFS66YG0","token":"'"$TOKEN"'"}' \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
 if [ "$STATUS" != "200" ]
@@ -362,7 +364,7 @@ ORDER_ID=$(jq -r .order_id < "$LAST_RESPONSE")
 TOKEN=$(jq -r .token < "$LAST_RESPONSE")
 
 STATUS=$(curl http://localhost:9966/orders/"$ORDER_ID"/claim \
-    -d '{"nonce":"","token":"'"$TOKEN"'"}' \
+    -d '{"nonce":"0FTTZ475NBECCZN97VFTN6DEKWVKJ8AQY9PWR6VS36JZQFS66YG0","token":"'"$TOKEN"'"}' \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
 if [ "$STATUS" != "200" ]
@@ -374,12 +376,14 @@ fi
 QUANTITY=$(jq -r .contract_terms.products[0].quantity < "$LAST_RESPONSE")
 if [ "$QUANTITY" != "1" ]
 then
+    cat < "$LAST_RESPONSE" >&2
     exit_fail "Expected quantity 1. got: $QUANTITY"
 fi
 
 IMAGE=$(jq -r .contract_terms.products[0].image < "$LAST_RESPONSE")
 if [ "$IMAGE" != "$RANDOM_IMG" ]
 then
+    cat "$LAST_RESPONSE" >&2
     exit_fail "Expected $RANDOM_IMG but got something else: $IMAGE"
 fi
 echo "OK"
@@ -416,7 +420,7 @@ ORDER_ID=$(jq -e -r .order_id < "$LAST_RESPONSE")
 TOKEN=$(jq -e -r .token < "$LAST_RESPONSE")
 
 STATUS=$(curl http://localhost:9966/orders/"$ORDER_ID"/claim \
-    -d '{"nonce":"","token":"'"$TOKEN"'"}' \
+    -d '{"nonce":"1ZRJCEAGM5N98P6ATH0NSR9SP3RKQJQQ05MQJAJA57T8YNPCPWEG","token":"'"$TOKEN"'"}' \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
 if [ "$STATUS" != "200" ]
@@ -472,7 +476,7 @@ STATUS=$(curl "http://localhost:9966/private/orders/${ORDER_ID}" \
 if [ "$STATUS" != "200" ]
 then
     jq . < "$LAST_RESPONSE"
-    exit_fail "Expected 200, getting order info before claming it. got: $STATUS"
+    exit_fail "Expected 200, getting order info. got: $STATUS"
 fi
 
 PAY_URL=$(jq -e -r .taler_pay_uri < "$LAST_RESPONSE")
@@ -523,20 +527,22 @@ NOW=$(date +%s)
 TO_SLEEP=$((1200 + WIRE_DEADLINE - NOW ))
 echo "Waiting $TO_SLEEP secs for wire transfer"
 
-echo -n "Perform wire transfers ..."
+echo -n "Call taler-exchange-aggregator ..."
 taler-exchange-aggregator \
     -y \
     -c "$CONF" \
     -T "${TO_SLEEP}"000000 \
     -t \
     -L INFO &> aggregator.log
+echo " DONE"
+echo -n "Call taler-exchange-transfer ..."
 taler-exchange-transfer \
     -c "$CONF" \
     -t \
     -L INFO &> transfer.log
 echo " DONE"
 echo -n "Give time to Nexus to route the payment to Sandbox..."
-# FIXME-MS: trigger immediate update at nexus
+# FIXME: trigger immediate update at nexus
 # NOTE: once libeufin can do long-polling, we should
 # be able to reduce the delay here and run aggregator/transfer
 # always in the background via setup
