@@ -70,7 +70,8 @@ depositcheck_run (void *cls,
                                "taler-merchant-depositcheck",
                                "-c", ws->config_filename,
                                "-t", /* exit when done */
-                               "-L", "DEBUG",
+                               "-T", "1200s",
+                               "-L", "INFO",
                                NULL);
   if (NULL == ws->depositcheck_proc)
   {
