@@ -539,16 +539,16 @@ orders_run2 (void *cls,
         GNUNET_break (0);
         break;
       }
-      scaled = llround (frac * (double) MERCHANT_UNIT_FRAC_BASE);
+      scaled = llround (frac * (double) TALER_MERCHANT_UNIT_FRAC_BASE);
       if (scaled < 0)
       {
         GNUNET_break (0);
         break;
       }
-      if (scaled >= (long long) MERCHANT_UNIT_FRAC_BASE)
+      if (scaled >= (long long) TALER_MERCHANT_UNIT_FRAC_BASE)
       {
         quantity_int++;
-        scaled -= MERCHANT_UNIT_FRAC_BASE;
+        scaled -= TALER_MERCHANT_UNIT_FRAC_BASE;
       }
       quantity_frac_local = (uint32_t) scaled;
       pd.quantity = quantity_int;
