@@ -75,6 +75,26 @@ struct PostProductsState
    */
   struct TALER_Amount price;
 
+  /**
+   * Array of unit prices (may point to @e price for single-entry usage).
+   */
+  struct TALER_Amount *unit_prices;
+
+  /**
+   * Number of entries in @e unit_prices.
+   */
+  size_t unit_prices_len;
+
+  /**
+   * True if @e unit_prices must be freed.
+   */
+  bool owns_unit_prices;
+
+  /**
+   * True if we should send an explicit unit_price array.
+   */
+  bool use_unit_price_array;
+
   /**
    * base64-encoded product image
    */
@@ -130,6 +150,16 @@ struct PostProductsState
    */
   struct GNUNET_TIME_Timestamp next_restock;
 
+  /**
+   * Categories array length.
+   */
+  unsigned int num_cats;
+
+  /**
+   * Categories array.
+   */
+  uint64_t *cats;
+
   /**
    * Expected HTTP response code.
    */
@@ -140,21 +170,17 @@ struct PostProductsState
 static void
 post_products_update_unit_total_stock (struct PostProductsState *pps)
 {
-  uint64_t stock;
-  uint32_t frac;
-
   if (-1 == pps->total_stock)
   {
-    stock = (uint64_t) INT64_MAX;
-    frac = (uint32_t) INT32_MAX;
+    pps->total_stock = INT64_MAX;
+    pps->total_stock_frac = INT32_MAX;
   }
-  else
+  else if (! pps->unit_allow_fraction)
   {
-    stock = (uint64_t) pps->total_stock;
-    frac = pps->unit_allow_fraction ? pps->total_stock_frac : 0;
+    pps->total_stock_frac = 0;
   }
-  TALER_MERCHANT_format_stock_string (stock,
-                                      frac,
+  TALER_MERCHANT_format_stock_string ((uint64_t) pps->total_stock,
+                                      pps->total_stock_frac,
                                       pps->unit_total_stock,
                                       sizeof (pps->unit_total_stock));
 }
@@ -278,7 +304,9 @@ post_products_run (void *cls,
   struct PostProductsState *pis = cls;
 
   pis->is = is;
-  if (pis->use_fractional)
+  if (pis->use_fractional ||
+      pis->use_unit_price_array ||
+      (0 < pis->num_cats))
   {
     pis->iph = TALER_MERCHANT_products_post4 (
       TALER_TESTING_interpreter_get_context (is),
@@ -287,8 +315,8 @@ post_products_run (void *cls,
       pis->description,
       pis->description_i18n,
       pis->unit,
-      &pis->price,
-      1,
+      pis->unit_prices,
+      pis->unit_prices_len,
       pis->image,
       pis->taxes,
       pis->total_stock,
@@ -300,8 +328,8 @@ post_products_run (void *cls,
       pis->address,
       pis->next_restock,
       pis->minimum_age,
-      0,
-      NULL,
+      pis->num_cats,
+      pis->cats,
       &post_products_cb,
       pis);
   }
@@ -392,10 +420,13 @@ post_products_cleanup (void *cls,
                 "POST /products operation did not complete\n");
     TALER_MERCHANT_products_post_cancel (pis->iph);
   }
+  if (pis->owns_unit_prices)
+    GNUNET_free (pis->unit_prices);
   json_decref (pis->description_i18n);
   GNUNET_free (pis->image);
   json_decref (pis->taxes);
   json_decref (pis->address);
+  GNUNET_free (pis->cats);
   GNUNET_free (pis);
 }
 
@@ -434,6 +465,10 @@ TALER_TESTING_cmd_merchant_post_products2 (
   GNUNET_assert (GNUNET_OK ==
                  TALER_string_to_amount (price,
                                          &pis->price));
+  pis->unit_prices = &pis->price;
+  pis->unit_prices_len = 1;
+  pis->owns_unit_prices = false;
+  pis->use_unit_price_array = false;
   pis->image = GNUNET_strdup (image);
   pis->taxes = taxes; /* ownership taken */
   pis->total_stock = total_stock;
@@ -443,6 +478,8 @@ TALER_TESTING_cmd_merchant_post_products2 (
   pis->minimum_age = minimum_age;
   pis->address = address; /* ownership taken */
   pis->next_restock = next_restock;
+  pis->num_cats = 0;
+  pis->cats = NULL;
   post_products_update_unit_total_stock (pis);
   {
     struct TALER_TESTING_Command cmd = {
@@ -505,6 +542,65 @@ TALER_TESTING_cmd_merchant_post_products3 (
 }
 
 
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_products_with_unit_prices (
+  const char *label,
+  const char *merchant_url,
+  const char *product_id,
+  const char *description,
+  const char *unit,
+  const char *const *unit_prices,
+  size_t unit_prices_len,
+  unsigned int http_status)
+{
+  struct PostProductsState *pis;
+
+  GNUNET_assert (0 < unit_prices_len);
+  GNUNET_assert (NULL != unit_prices);
+  pis = GNUNET_new (struct PostProductsState);
+  pis->merchant_url = merchant_url;
+  pis->product_id = product_id;
+  pis->http_status = http_status;
+  pis->description = description;
+  pis->description_i18n = json_pack ("{s:s}", "en", description);
+  pis->unit = unit;
+  pis->unit_precision_level = default_precision_from_unit (unit);
+  pis->unit_prices = GNUNET_new_array (unit_prices_len,
+                                       struct TALER_Amount);
+  pis->unit_prices_len = unit_prices_len;
+  pis->owns_unit_prices = true;
+  pis->use_unit_price_array = true;
+  for (size_t i = 0; i < unit_prices_len; i++)
+    GNUNET_assert (GNUNET_OK ==
+                   TALER_string_to_amount (unit_prices[i],
+                                           &pis->unit_prices[i]));
+  pis->price = pis->unit_prices[0];
+  pis->image = GNUNET_strdup ("");
+  pis->taxes = json_array ();
+  pis->total_stock = 4;
+  pis->total_stock_frac = 0;
+  pis->unit_allow_fraction = false;
+  pis->use_fractional = false;
+  pis->minimum_age = 0;
+  pis->address = json_pack ("{s:s}", "street", "my street");
+  pis->next_restock = GNUNET_TIME_UNIT_ZERO_TS;
+  pis->num_cats = 0;
+  pis->cats = NULL;
+  post_products_update_unit_total_stock (pis);
+  {
+    struct TALER_TESTING_Command cmd = {
+      .cls = pis,
+      .label = label,
+      .run = &post_products_run,
+      .cleanup = &post_products_cleanup,
+      .traits = &post_products_traits
+    };
+
+    return cmd;
+  }
+}
+
+
 struct TALER_TESTING_Command
 TALER_TESTING_cmd_merchant_post_products (
   const char *label,
@@ -532,4 +628,55 @@ TALER_TESTING_cmd_merchant_post_products (
 }
 
 
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_products_with_categories (
+  const char *label,
+  const char *merchant_url,
+  const char *product_id,
+  const char *description,
+  const char *unit,
+  const char *price,
+  unsigned int num_cats,
+  const uint64_t *cats,
+  bool unit_allow_fraction,
+  uint32_t unit_precision_level,
+  unsigned int http_status)
+{
+  struct TALER_TESTING_Command cmd;
+
+  cmd = TALER_TESTING_cmd_merchant_post_products2 (
+    label,
+    merchant_url,
+    product_id,
+    description,
+    json_pack ("{s:s}", "en", description),
+    unit,
+    price,
+    "",
+    json_array (),
+    4, /* total stock */
+    0, /* minimum age */
+    json_pack ("{s:s}", "street", "my street"),
+    GNUNET_TIME_UNIT_ZERO_TS,
+    http_status);
+  {
+    struct PostProductsState *pis = cmd.cls;
+
+    pis->num_cats = num_cats;
+    if (0 < num_cats)
+    {
+      pis->cats = GNUNET_new_array (num_cats,
+                                    uint64_t);
+      for (unsigned int i = 0; i < num_cats; i++)
+        pis->cats[i] = cats[i];
+    }
+    pis->unit_allow_fraction = unit_allow_fraction;
+    pis->unit_precision_level = unit_precision_level;
+    if (unit_allow_fraction)
+      pis->use_fractional = true;
+  }
+  return cmd;
+}
+
+
 /* end of testing_api_cmd_post_products.c */
