@@ -71,6 +71,11 @@ struct PostUsingTemplatesState
    */
   struct TALER_Amount amount;
 
+  /**
+   * Raw request body for inventory templates (if set).
+   */
+  json_t *details;
+
   /**
    * Label of a command that created the template we should use.
    */
@@ -376,16 +381,32 @@ post_using_templates_run (void *cls,
                                          &tis->otp_alg))
       TALER_TESTING_FAIL (is);
   }
-  tis->iph = TALER_MERCHANT_using_templates_post (
-    TALER_TESTING_interpreter_get_context (is),
-    tis->merchant_url,
-    template_id,
-    tis->summary,
-    TALER_amount_is_valid (&tis->amount)
-    ? &tis->amount
-    : NULL,
-    &post_using_templates_cb,
-    tis);
+  GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_WEAK,
+                              &tis->nonce,
+                              sizeof (struct GNUNET_CRYPTO_EddsaPublicKey));
+  if (NULL != tis->details)
+  {
+    tis->iph = TALER_MERCHANT_using_templates_post2 (
+      TALER_TESTING_interpreter_get_context (is),
+      tis->merchant_url,
+      template_id,
+      tis->details,
+      &post_using_templates_cb,
+      tis);
+  }
+  else
+  {
+    tis->iph = TALER_MERCHANT_using_templates_post (
+      TALER_TESTING_interpreter_get_context (is),
+      tis->merchant_url,
+      template_id,
+      tis->summary,
+      TALER_amount_is_valid (&tis->amount)
+      ? &tis->amount
+      : NULL,
+      &post_using_templates_cb,
+      tis);
+  }
   GNUNET_assert (NULL != tis->iph);
 }
 
@@ -450,6 +471,7 @@ post_using_templates_cleanup (void *cls,
   }
   json_decref (tis->order_terms);
   json_decref (tis->contract_terms);
+  json_decref (tis->details);
   GNUNET_free (tis->order_id);
   GNUNET_free (tis);
 }
@@ -604,4 +626,39 @@ TALER_TESTING_cmd_merchant_post_using_templates (
 }
 
 
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_using_templates2 (
+  const char *label,
+  const char *template_ref,
+  const char *otp_ref,
+  const char *merchant_url,
+  const char *using_template_id,
+  const json_t *details,
+  unsigned int http_status)
+{
+  struct PostUsingTemplatesState *tis;
+
+  tis = GNUNET_new (struct PostUsingTemplatesState);
+  tis->template_ref = template_ref;
+  tis->otp_ref = otp_ref;
+  tis->merchant_url = merchant_url;
+  tis->using_template_id = using_template_id;
+  tis->http_status = http_status;
+  tis->with_claim = true;
+  if (NULL != details)
+    tis->details = json_incref ((json_t *) details);
+  {
+    struct TALER_TESTING_Command cmd = {
+      .cls = tis,
+      .label = label,
+      .run = &post_using_templates_run,
+      .cleanup = &post_using_templates_cleanup,
+      .traits = &post_using_templates_traits
+    };
+
+    return cmd;
+  }
+}
+
+
 /* end of testing_api_cmd_post_using_templates.c */
