@@ -148,16 +148,17 @@ post_webhooks_run (void *cls,
   struct PostWebhooksState *wis = cls;
 
   wis->is = is;
-  wis->iph = TALER_MERCHANT_webhooks_post (is->ctx,
-                                           wis->merchant_url,
-                                           wis->webhook_id,
-                                           wis->event_type,
-                                           wis->url,
-                                           wis->http_method,
-                                           wis->header_template,
-                                           wis->body_template,
-                                           &post_webhooks_cb,
-                                           wis);
+  wis->iph = TALER_MERCHANT_webhooks_post (
+    TALER_TESTING_interpreter_get_context (is),
+    wis->merchant_url,
+    wis->webhook_id,
+    wis->event_type,
+    wis->url,
+    wis->http_method,
+    wis->header_template,
+    wis->body_template,
+    &post_webhooks_cb,
+    wis);
   GNUNET_assert (NULL != wis->iph);
 }
 
@@ -180,12 +181,12 @@ post_webhooks_traits (void *cls,
 {
   struct PostWebhooksState *pws = cls;
   struct TALER_TESTING_Trait traits[] = {
-    TALER_TESTING_make_trait_event_type (&pws->event_type),
-    TALER_TESTING_make_trait_url (&pws->url),
-    TALER_TESTING_make_trait_http_method (&pws->http_method),
-    TALER_TESTING_make_trait_header_template (&pws->header_template),
-    TALER_TESTING_make_trait_body_template (&pws->body_template),
-    TALER_TESTING_make_trait_webhook_id (&pws->webhook_id),
+    TALER_TESTING_make_trait_event_type (pws->event_type),
+    TALER_TESTING_make_trait_url (pws->url),
+    TALER_TESTING_make_trait_http_method (pws->http_method),
+    TALER_TESTING_make_trait_header_template (pws->header_template),
+    TALER_TESTING_make_trait_body_template (pws->body_template),
+    TALER_TESTING_make_trait_webhook_id (pws->webhook_id),
     TALER_TESTING_trait_end (),
   };
 
