@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2024 Taler Systems SA
+  (C) 2024, 2025 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Lesser General Public License as published by the Free Software
@@ -17,6 +17,7 @@
  * @file util/contract_parse.c
  * @brief shared logic for contract terms parsing
  * @author Iván Ávalos
+ * @author Christian Grothoff
  */
 #include "platform.h"
 #include <gnunet/gnunet_common.h>
@@ -361,7 +362,12 @@ parse_choices (
     GNUNET_break_op (0);
     return GNUNET_SYSERR;
   }
-
+  if (0 == json_array_size (root))
+  {
+    /* empty list of choices is not allowed */
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
   GNUNET_array_grow (*choices,
                      *choices_len,
                      json_array_size (root));
@@ -369,11 +375,15 @@ parse_choices (
   for (unsigned int i = 0; i < *choices_len; i++)
   {
     struct TALER_MERCHANT_ContractChoice *choice = &(*choices)[i];
-    const json_t *jinputs;
-    const json_t *joutputs;
+    const json_t *jinputs = NULL;
+    const json_t *joutputs = NULL;
     struct GNUNET_JSON_Specification spec[] = {
       TALER_JSON_spec_amount_any ("amount",
                                   &choice->amount),
+      GNUNET_JSON_spec_mark_optional (
+        TALER_JSON_spec_amount_any ("tip",
+                                    &choice->tip),
+        &choice->no_tip),
       GNUNET_JSON_spec_mark_optional (
         GNUNET_JSON_spec_string_copy ("description",
                                       &choice->description),
@@ -382,12 +392,18 @@ parse_choices (
         GNUNET_JSON_spec_object_copy ("description_i18n",
                                       &choice->description_i18n),
         NULL),
-      TALER_JSON_spec_amount_any ("max_fee",
-                                  &choice->max_fee),
-      GNUNET_JSON_spec_array_const ("inputs",
-                                    &jinputs),
-      GNUNET_JSON_spec_array_const ("outputs",
-                                    &joutputs),
+      GNUNET_JSON_spec_mark_optional (
+        TALER_JSON_spec_amount_any ("max_fee",
+                                    &choice->max_fee),
+        NULL),
+      GNUNET_JSON_spec_mark_optional (
+        GNUNET_JSON_spec_array_const ("inputs",
+                                      &jinputs),
+        NULL),
+      GNUNET_JSON_spec_mark_optional (
+        GNUNET_JSON_spec_array_const ("outputs",
+                                      &joutputs),
+        NULL),
       GNUNET_JSON_spec_end ()
     };
     const char *ename;
@@ -407,7 +423,19 @@ parse_choices (
       GNUNET_break_op (0);
       return GNUNET_SYSERR;
     }
+    if ( (! choice->no_tip) &&
+         (GNUNET_OK !=
+          TALER_amount_cmp_currency (&choice->amount,
+                                     &choice->tip)) )
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Tip currency does not match amount currency in choice #%u\n",
+                  i);
+      GNUNET_break_op (0);
+      return GNUNET_SYSERR;
+    }
 
+    if (NULL != jinputs)
     {
       const json_t *jinput;
       size_t idx;
@@ -444,6 +472,7 @@ parse_choices (
       }
     }
 
+    if (NULL != joutputs)
     {
       const json_t *joutput;
       size_t idx;
@@ -486,16 +515,8 @@ parse_choices (
 }
 
 
-/**
- * Provide specification to parse given JSON array to contract terms
- * choices. All fields from @a choices elements are copied.
- *
- * @param name name of the choices field in the JSON
- * @param[out] choices where the contract choices array has to be written
- * @param[out] choices_len length of the @a choices array
- */
-static struct GNUNET_JSON_Specification
-spec_choices (
+struct GNUNET_JSON_Specification
+TALER_MERCHANT_spec_choices (
   const char *name,
   struct TALER_MERCHANT_ContractChoice **choices,
   unsigned int *choices_len)
@@ -1026,6 +1047,10 @@ parse_contract_v0 (
   struct GNUNET_JSON_Specification espec[] = {
     TALER_JSON_spec_amount_any ("amount",
                                 &contract->details.v0.brutto),
+    GNUNET_JSON_spec_mark_optional (
+      TALER_JSON_spec_amount_any ("tip",
+                                  &contract->details.v0.tip),
+      &contract->details.v0.no_tip),
     TALER_JSON_spec_amount_any ("max_fee",
                                 &contract->details.v0.max_fee),
     GNUNET_JSON_spec_end ()
@@ -1056,6 +1081,16 @@ parse_contract_v0 (
                 "'max_fee' in database does not match currency of contract price");
     return GNUNET_SYSERR;
   }
+  if ( (! contract->details.v0.no_tip) &&
+       (GNUNET_OK !=
+        TALER_amount_cmp_currency (&contract->details.v0.tip,
+                                   &contract->details.v0.brutto)) )
+  {
+    GNUNET_break (0);
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "'tip' in database does not match currency of contract price");
+    return GNUNET_SYSERR;
+  }
 
   return res;
 }
@@ -1074,7 +1109,7 @@ parse_contract_v1 (
   struct TALER_MERCHANT_Contract *contract)
 {
   struct GNUNET_JSON_Specification espec[] = {
-    spec_choices (
+    TALER_MERCHANT_spec_choices (
       "choices",
       &contract->details.v1.choices,
       &contract->details.v1.choices_len),
@@ -1106,12 +1141,295 @@ parse_contract_v1 (
 }
 
 
+/**
+ * Parse the given unit quantity string @a s and store the result in @a q.
+ *
+ * @param s quantity to parse
+ * @param[out] q where to store the result
+ * @return #GNUNET_OK on success
+ */
+static enum GNUNET_GenericReturnValue
+parse_unit_quantity (const char *s,
+                     struct TALER_MERCHANT_ProductQuantity *q)
+{
+  const char *ptr;
+  uint64_t integer = 0;
+  uint32_t frac = 0;
+  unsigned int digits = 0;
+
+  if (NULL == s)
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  ptr = s;
+  if ('\0' == *ptr)
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  if ('-' == *ptr)
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  if (! isdigit ((unsigned char) *ptr))
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  while (isdigit ((unsigned char) *ptr))
+  {
+    unsigned int digit = (unsigned int) (*ptr - '0');
+
+    /* We intentionally allow at most INT64_MAX (as -1 has special meanings),
+       even though the data type would support UINT64_MAX */
+    if (integer > (INT64_MAX - digit) / 10)
+    {
+      GNUNET_break_op (0);
+      return GNUNET_SYSERR;
+    }
+    integer = integer * 10 + digit;
+    ptr++;
+  }
+  if ('.' == *ptr)
+  {
+    ptr++;
+    if ('\0' == *ptr)
+    {
+      GNUNET_break_op (0);
+      return GNUNET_SYSERR;
+    }
+    while (isdigit ((unsigned char) *ptr))
+    {
+      unsigned int digit = (unsigned int) (*ptr - '0');
+
+      if (digits >= TALER_MERCHANT_UNIT_FRAC_MAX_DIGITS)
+      {
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
+      frac = (uint32_t) (frac * 10 + digit);
+      digits++;
+      ptr++;
+    }
+    while (digits < TALER_MERCHANT_UNIT_FRAC_MAX_DIGITS)
+    {
+      frac *= 10;
+      digits++;
+    }
+  }
+  if ('\0' != *ptr)
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  q->integer = integer;
+  q->fractional = frac;
+  return GNUNET_OK;
+}
+
+
+bool
+TALER_MERCHANT_taxes_array_valid (const json_t *taxes)
+{
+  json_t *tax;
+  size_t idx;
+
+  if (! json_is_array (taxes))
+    return false;
+  json_array_foreach (taxes, idx, tax)
+  {
+    struct TALER_Amount amount;
+    const char *name;
+    struct GNUNET_JSON_Specification spec[] = {
+      GNUNET_JSON_spec_string ("name",
+                               &name),
+      TALER_JSON_spec_amount_any ("tax",
+                                  &amount),
+      GNUNET_JSON_spec_end ()
+    };
+    enum GNUNET_GenericReturnValue res;
+    const char *ename;
+    unsigned int eline;
+
+    res = GNUNET_JSON_parse (tax,
+                             spec,
+                             &ename,
+                             &eline);
+    if (GNUNET_OK != res)
+    {
+      GNUNET_break_op (0);
+      return false;
+    }
+  }
+  return true;
+}
+
+
+enum GNUNET_GenericReturnValue
+TALER_MERCHANT_parse_product_sold (const json_t *pj,
+                                   struct TALER_MERCHANT_ProductSold *r)
+{
+  bool no_quantity;
+  bool no_unit_quantity;
+  bool no_price;
+  uint64_t legacy_quantity;
+  const char *unit_quantity_s;
+  struct TALER_Amount price;
+  const json_t *prices = NULL;
+  struct GNUNET_JSON_Specification spec[] = {
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string_copy ("product_id",
+                                    &r->product_id),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string_copy ("product_name",
+                                    &r->product_name),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string_copy ("description",
+                                    &r->description),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_object_copy ("description_i18n",
+                                    &r->description_i18n),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_uint64 ("quantity",
+                               &legacy_quantity),
+      &no_quantity),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("unit_quantity",
+                               &unit_quantity_s),
+      &no_unit_quantity),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string_copy ("unit",
+                                    &r->unit),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      TALER_JSON_spec_amount_any ("price",
+                                  &price),
+      &no_price),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_array_const ("prices",
+                                    &prices),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string_copy ("image",
+                                    &r->image),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_array_copy ("taxes",
+                                   &r->taxes),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_timestamp ("delivery_date",
+                                  &r->delivery_date),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_uint64 ("product_money_pot",
+                               &r->product_money_pot),
+      NULL),
+    GNUNET_JSON_spec_end ()
+  };
+  enum GNUNET_GenericReturnValue res;
+  const char *ename;
+  unsigned int eline;
+
+  r->delivery_date = GNUNET_TIME_UNIT_FOREVER_TS;
+  res = GNUNET_JSON_parse (pj,
+                           spec,
+                           &ename,
+                           &eline);
+  if (GNUNET_OK != res)
+  {
+    GNUNET_break (0);
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Failed to parse product at field %s\n",
+                ename);
+    return GNUNET_SYSERR;
+  }
+  if (! no_quantity)
+  {
+    r->unit_quantity.integer = legacy_quantity;
+    r->unit_quantity.fractional = 0;
+  }
+  if (! no_unit_quantity)
+  {
+    if (GNUNET_OK !=
+        parse_unit_quantity (unit_quantity_s,
+                             &r->unit_quantity))
+    {
+      GNUNET_break (0);
+      return GNUNET_SYSERR;
+    }
+  }
+  if ( (! no_quantity) && (! no_unit_quantity) )
+  {
+    GNUNET_break ( (0 == r->unit_quantity.fractional) &&
+                   (legacy_quantity == r->unit_quantity.integer) );
+  }
+  if ( (NULL != r->image) &&
+       (! TALER_MERCHANT_image_data_url_valid (r->image)) )
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  if ( (NULL != r->taxes) &&
+       (! TALER_MERCHANT_taxes_array_valid (r->taxes)) )
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  if (NULL != prices)
+  {
+    size_t len = json_array_size (prices);
+    size_t i;
+    json_t *price_i;
+
+    GNUNET_assert (len < UINT_MAX);
+    r->prices = GNUNET_new_array ((unsigned int) len,
+                                  struct TALER_Amount);
+    json_array_foreach (prices, i, price_i)
+    {
+      struct GNUNET_JSON_Specification pspec[] = {
+        TALER_JSON_spec_amount_any (NULL,
+                                    &r->prices[i]),
+        GNUNET_JSON_spec_end ()
+      };
+
+      res = GNUNET_JSON_parse (price_i,
+                               pspec,
+                               &ename,
+                               &eline);
+      if (GNUNET_OK != res)
+      {
+        GNUNET_break (0);
+        GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                    "Failed to parse price at index %u\n",
+                    (unsigned int) i);
+        return GNUNET_SYSERR;
+      }
+    }
+  }
+  else if (! no_price)
+  {
+    r->prices = GNUNET_new_array (1,
+                                  struct TALER_Amount);
+    r->prices[0] = price;
+  }
+  return GNUNET_OK;
+}
+
+
 struct TALER_MERCHANT_Contract *
 TALER_MERCHANT_contract_parse (json_t *input,
                                bool nonce_optional)
 {
   struct TALER_MERCHANT_Contract *contract
     = GNUNET_new (struct TALER_MERCHANT_Contract);
+  const json_t *products = NULL;
   struct GNUNET_JSON_Specification espec[] = {
     spec_contract_version ("version",
                            &contract->version),
@@ -1140,8 +1458,10 @@ TALER_MERCHANT_contract_parse (json_t *input,
       GNUNET_JSON_spec_object_copy ("fulfillment_message_i18n",
                                     &contract->fulfillment_message_i18n),
       NULL),
-    GNUNET_JSON_spec_array_copy ("products",
-                                 &contract->products),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_array_const ("products",
+                                    &products),
+      NULL),
     GNUNET_JSON_spec_timestamp ("timestamp",
                                 &contract->timestamp),
     GNUNET_JSON_spec_timestamp ("refund_deadline",
@@ -1189,6 +1509,10 @@ TALER_MERCHANT_contract_parse (json_t *input,
       GNUNET_JSON_spec_uint8 ("minimum_age",
                               &contract->minimum_age),
       NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_uint64 ("default_money_pot",
+                               &contract->default_money_pot),
+      NULL),
     GNUNET_JSON_spec_end ()
   };
 
@@ -1209,7 +1533,31 @@ TALER_MERCHANT_contract_parse (json_t *input,
                 ename);
     goto cleanup;
   }
+  if (NULL != products)
+  {
+    contract->products_len = json_array_size (products);
+    if (0 != contract->products_len)
+    {
+      size_t i;
+      json_t *p;
 
+      contract->products = GNUNET_new_array (contract->products_len,
+                                             struct TALER_MERCHANT_ProductSold);
+      json_array_foreach (products, i, p)
+      {
+        if (GNUNET_OK !=
+            TALER_MERCHANT_parse_product_sold (p,
+                                               &contract->products[i]))
+        {
+          GNUNET_break (0);
+          GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                      "Failed to parse product at offset %u\n",
+                      (unsigned int) i);
+          goto cleanup;
+        }
+      }
+    }
+  }
   switch (contract->version)
   {
   case TALER_MERCHANT_CONTRACT_VERSION_0:
@@ -1237,6 +1585,20 @@ cleanup:
 }
 
 
+void
+TALER_MERCHANT_product_sold_free (struct TALER_MERCHANT_ProductSold *product)
+{
+  GNUNET_free (product->product_id);
+  GNUNET_free (product->product_name);
+  GNUNET_free (product->description);
+  json_decref (product->description_i18n);
+  GNUNET_free (product->prices);
+  GNUNET_free (product->unit);
+  GNUNET_free (product->image);
+  json_decref (product->taxes);
+}
+
+
 void
 TALER_MERCHANT_contract_free (
   struct TALER_MERCHANT_Contract *contract)
@@ -1270,8 +1632,9 @@ TALER_MERCHANT_contract_free (
   }
   if (NULL != contract->products)
   {
-    json_decref (contract->products);
-    contract->products = NULL;
+    for (size_t i = 0; i<contract->products_len; i++)
+      TALER_MERCHANT_product_sold_free (&contract->products[i]);
+    GNUNET_free (contract->products);
   }
   GNUNET_free (contract->wire_method);
   if (NULL != contract->exchanges)
