@@ -1,6 +1,6 @@
 /*
   This file is part of GNU Taler
-  Copyright (C) 2024 Taler Systems SA
+  Copyright (C) 2024, 2025 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -17,6 +17,7 @@
  * @file util/contract_serialize.c
  * @brief shared logic for contract terms serialization
  * @author Iván Ávalos
+ * @author Christian Grothoff
  */
 
 #include "platform.h"
@@ -179,6 +180,11 @@ TALER_MERCHANT_json_from_contract_choice (
   return GNUNET_JSON_PACK (
     TALER_JSON_pack_amount ("amount",
                             &choice->amount),
+    GNUNET_JSON_pack_allow_null (
+      TALER_JSON_pack_amount ("tip",
+                              choice->no_tip
+                              ? NULL
+                              : &choice->tip)),
     GNUNET_JSON_pack_allow_null (
       GNUNET_JSON_pack_string ("description",
                                choice->description)),
@@ -340,6 +346,11 @@ json_from_contract_v0 (
   return GNUNET_JSON_PACK (
     TALER_JSON_pack_amount ("amount",
                             &input->details.v0.brutto),
+    GNUNET_JSON_pack_allow_null (
+      TALER_JSON_pack_amount ("tip",
+                              input->details.v0.no_tip
+                              ? NULL
+                              : &input->details.v0.tip)),
     TALER_JSON_pack_amount ("max_fee",
                             &input->details.v0.max_fee));
 }
@@ -384,12 +395,101 @@ json_from_contract_v1 (
 }
 
 
+/**
+ * Convert quantity @a q into a string for JSON serialization
+ *
+ * @param q quantity to convert
+ * @return formatted string
+ */
+static const char *
+quantity_to_string (const struct TALER_MERCHANT_ProductQuantity *q)
+{
+  static char res[64];
+
+  TALER_MERCHANT_vk_format_fractional_string (TALER_MERCHANT_VK_QUANTITY,
+                                              q->integer,
+                                              q->fractional,
+                                              sizeof (res),
+                                              res);
+  return res;
+}
+
+
+json_t *
+TALER_MERCHANT_product_sold_serialize (
+  const struct TALER_MERCHANT_ProductSold *p)
+{
+  json_t *prices;
+
+  prices = json_array ();
+  GNUNET_assert (NULL != prices);
+  for (unsigned int i = 0; i<p->prices_length; i++)
+    GNUNET_assert (0 ==
+                   json_array_append_new (prices,
+                                          TALER_JSON_from_amount (
+                                            &p->prices[i])));
+  return GNUNET_JSON_PACK (
+    GNUNET_JSON_pack_allow_null (
+      GNUNET_JSON_pack_string ("product_id",
+                               p->product_id)),
+    GNUNET_JSON_pack_allow_null (
+      GNUNET_JSON_pack_string ("product_name",
+                               p->product_name)),
+    GNUNET_JSON_pack_allow_null (
+      GNUNET_JSON_pack_string ("description",
+                               p->description)),
+    GNUNET_JSON_pack_allow_null (
+      GNUNET_JSON_pack_object_incref ("description_i18n",
+                                      (json_t *) p->description_i18n)),
+    GNUNET_JSON_pack_allow_null (
+      ( (0 != p->unit_quantity.integer) ||
+        (0 != p->unit_quantity.fractional) )
+      ? GNUNET_JSON_pack_string ("unit_quantity",
+                                 quantity_to_string (&p->unit_quantity))
+      : GNUNET_JSON_pack_string ("dummy",
+                                 NULL) ),
+    /* Legacy */
+    GNUNET_JSON_pack_allow_null (
+      (0 == p->unit_quantity.fractional)
+      ? GNUNET_JSON_pack_uint64 ("quantity",
+                                 p->unit_quantity.integer)
+      : GNUNET_JSON_pack_string ("dummy",
+                                 NULL) ),
+    GNUNET_JSON_pack_allow_null (
+      GNUNET_JSON_pack_string ("unit",
+                               p->unit)),
+    /* Deprecated, use prices! */
+    GNUNET_JSON_pack_allow_null (
+      TALER_JSON_pack_amount ("price",
+                              0 < p->prices_length
+                              ? &p->prices[0]
+                              : NULL)),
+    GNUNET_JSON_pack_array_steal ("prices",
+                                  prices),
+    GNUNET_JSON_pack_allow_null (
+      GNUNET_JSON_pack_string ("image",
+                               p->image)),
+    GNUNET_JSON_pack_allow_null (
+      GNUNET_JSON_pack_array_incref ("taxes",
+                                     (json_t *) p->taxes)),
+    GNUNET_JSON_pack_allow_null (
+      GNUNET_TIME_absolute_is_never (p->delivery_date.abs_time)
+      ? GNUNET_JSON_pack_string ("dummy",
+                                 NULL)
+      : GNUNET_JSON_pack_timestamp ("delivery_date",
+                                    p->delivery_date)),
+    GNUNET_JSON_pack_uint64 ("product_money_pot",
+                             p->product_money_pot));
+}
+
+
 json_t *
 TALER_MERCHANT_contract_serialize (
   const struct TALER_MERCHANT_Contract *input,
   bool nonce_optional)
 {
   json_t *details;
+  json_t *products;
 
   switch (input->version)
   {
@@ -408,6 +508,17 @@ TALER_MERCHANT_contract_serialize (
   return NULL;
 
 success:
+  products = json_array ();
+  GNUNET_assert (NULL != products);
+  for (size_t i = 0; i<input->products_len; i++)
+  {
+    GNUNET_assert (
+      0 ==
+      json_array_append_new (products,
+                             TALER_MERCHANT_product_sold_serialize (
+                               &input->products[i])));
+  }
+
   return GNUNET_JSON_PACK (
     GNUNET_JSON_pack_uint64 ("version",
                              input->version),
@@ -431,7 +542,7 @@ success:
       GNUNET_JSON_pack_object_steal ("fulfillment_message_i18n",
                                      input->fulfillment_message_i18n)),
     GNUNET_JSON_pack_array_steal ("products",
-                                  input->products),
+                                  products),
     GNUNET_JSON_pack_timestamp ("timestamp",
                                 input->timestamp),
     GNUNET_JSON_pack_timestamp ("refund_deadline",
@@ -473,6 +584,12 @@ success:
     GNUNET_JSON_pack_allow_null (
       GNUNET_JSON_pack_uint64 ("minimum_age",
                                input->minimum_age)),
+    (0 == input->default_money_pot)
+    ? GNUNET_JSON_pack_allow_null (
+      GNUNET_JSON_pack_string ("dummy",
+                               NULL))
+    : GNUNET_JSON_pack_uint64 ("default_money_pot",
+                               input->default_money_pot),
     GNUNET_JSON_pack_object_steal (NULL,
                                    details));
 }
